using System;
using System.Linq;
using System.Collections;
using System.Collections.Generic;
using NHibernate.DomainModel.Northwind.Entities;

namespace NHibernate.Test.Linq
{
	public static class NorthwindDbCreator
	{
		private static readonly DateTime KnownDate = new DateTime(2010, 06, 17);

		public static void CreateMiscTestData(ISession session)
		{
			var roles = new[]
            {
                new Role()
                {
                    Name = "Admin",
                    IsActive = true,
                    Entity = new AnotherEntity()
                    {
                        Output = "output"
                    }
                },
                new Role()
                {
                    Name = "User",
                    IsActive = false
                }
            };

			var users = new[]
        	{
        		new User("ayende", KnownDate)
                {
                    Role = roles[0],
                    InvalidLoginAttempts = 4,
                    Enum1 = EnumStoredAsString.Medium,
                    Enum2 = EnumStoredAsInt32.High,
                    Component = new UserComponent()
                    {
                        Property1 = "test1",
                        Property2 = "test2",
                        OtherComponent = new UserComponent2()
                        {
                            OtherProperty1 = "othertest1"
                        }
                    }
                },
        		new User("rahien", new DateTime(1998, 12, 31))
                {
                    Role = roles[1],
                    InvalidLoginAttempts = 5,
                    Enum1 = EnumStoredAsString.Small,
                    Component = new UserComponent()
                    {
                        Property2 = "test2"
                    }
                },
        		new User("nhibernate", new DateTime(2000, 1, 1))
                {
                    InvalidLoginAttempts = 6,
                    LastLoginDate = DateTime.Now.AddDays(-1),
                    Enum1 = EnumStoredAsString.Medium,
					Features = FeatureSet.HasAll
                }
        	};

			var timesheets = new[]
            {
                new Timesheet
                {
                    SubmittedDate = KnownDate,
                    Submitted = true
                },
                new Timesheet
                {
                    SubmittedDate = KnownDate.AddDays(-1),
                    Submitted = false, 
                    Entries = new List<TimesheetEntry>
                    {
                        new TimesheetEntry
                        {
                            EntryDate = KnownDate,
                            NumberOfHours = 6,
							Comments = "testing 123"
                        },
                        new TimesheetEntry
                        {
                            EntryDate = KnownDate.AddDays(1),
                            NumberOfHours = 14
                        }
                    }
                },
                new Timesheet
                {
                    SubmittedDate = DateTime.Now.AddDays(1),
                    Submitted = true,
                    Entries = new List<TimesheetEntry>
                    {
                        new TimesheetEntry
                        {
                            EntryDate = DateTime.Now.AddMinutes(20),
                            NumberOfHours = 4
                        },
                        new TimesheetEntry
                        {
                            EntryDate = DateTime.Now.AddMinutes(10),
                            NumberOfHours = 8,
							Comments = "testing 456"
                        },
                        new TimesheetEntry
                        {
                            EntryDate = DateTime.Now.AddMinutes(13),
                            NumberOfHours = 7
                        },
                        new TimesheetEntry
                        {
                            EntryDate = DateTime.Now.AddMinutes(45),
                            NumberOfHours = 38
                        }
                    }
                }
            };

			((IList<User>)timesheets[0].Users).Add(users[0]);
			((IList<User>)timesheets[1].Users).Add(users[0]);
			((IList<User>)timesheets[0].Users).Add(users[1]);

			var animals = new Animal[]
			{
				new Animal() { SerialNumber = "123", BodyWeight = 100 },
				new Lizard() { SerialNumber = "789", BodyWeight = 40, BodyTemperature = 14 },
				new Lizard() { SerialNumber = "1234", BodyWeight = 30, BodyTemperature = 18 },
				new Dog() { SerialNumber = "5678", BodyWeight = 156, BirthDate = new DateTime(1980, 07, 11) },
				new Dog() { SerialNumber = "9101", BodyWeight = 205, BirthDate = new DateTime(1980, 12, 13) },
				new Cat() { SerialNumber = "1121", BodyWeight = 115, Pregnant = true }
			};

			animals[0].Children = new[] { animals[3], animals[4] }.ToList();
			animals[5].Father = animals[3];
			animals[5].Mother = animals[4];

			animals[1].Children = new[] { animals[5] }.ToList();

			List<AnotherEntity> otherEntities = new List<AnotherEntity>();
			// AnotherEntity with only Output set is created above.
			otherEntities.Add(new AnotherEntity { Input = "input" });
			otherEntities.Add(new AnotherEntity { Input = "i/o", Output = "i/o" });
			otherEntities.Add(new AnotherEntity()); // Input and Output both null.
			otherEntities.Add(new AnotherEntity { Input = "input", Output = "output" });

			foreach (AnotherEntity otherEntity in otherEntities)
				session.Save(otherEntity);

			foreach (Role role in roles)
				session.Save(role);

			foreach (User user in users)
				session.Save(user);

			foreach (Timesheet timesheet in timesheets)
				session.Save(timesheet);

			foreach (Animal animal in animals)
				session.Save(animal);
		}

		public static void CreatePatientData(ISession session)
		{
			State newYork = new State
			{
				Abbreviation = "NY",
				FullName = "New York"
			};
			State florida = new State
			{
				Abbreviation = "FL",
				FullName = "Florida"
			};

			Physician drDobbs = new Physician
			{
				Name = "Dr Dobbs"
			};
			Physician drWatson = new Physician
			{
				Name = "Dr Watson"
			};

			PatientRecord bobBarkerRecord = new PatientRecord
			{
				Name = new PatientName
				{
					FirstName = "Bob",
					LastName = "Barker"
				},
				Address = new PatientAddress
				{
					AddressLine1 = "123 Main St",
					City = "New York",
					State = newYork,
					ZipCode = "10001"
				},
				BirthDate = new DateTime(1930, 1, 1),
				Gender = Gender.Male
			};

			PatientRecord johnDoeRecord1 = new PatientRecord
			{
				Name = new PatientName
				{
					FirstName = "John",
					LastName = "Doe"
				},
				Address = new PatientAddress
				{
					AddressLine1 = "123 Main St",
					City = "Tampa",
					State = florida,
					ZipCode = "33602"
				},
				BirthDate = new DateTime(1969, 1, 1),
				Gender = Gender.Male
			};

			PatientRecord johnDoeRecord2 = new PatientRecord
			{
				Name = new PatientName
				{
					FirstName = "John",
					LastName = "Doe"
				},
				Address = new PatientAddress
				{
					AddressLine1 = "123 Main St",
					AddressLine2 = "Apt 2",
					City = "Tampa",
					State = florida,
					ZipCode = "33602"
				},
				BirthDate = new DateTime(1969, 1, 1)
			};

			Patient bobBarker = new Patient(new[] { bobBarkerRecord }, false, drDobbs);
			Patient johnDoe = new Patient(new[] { johnDoeRecord1, johnDoeRecord2 }, true, drWatson);

			session.Save(newYork);
			session.Save(florida);
			session.Save(drDobbs);
			session.Save(drWatson);
			session.Save(bobBarker);
			session.Save(johnDoe);
		}

		public static void CreateNorthwindData(IStatelessSession session)
		{
			var shippers = new List<Shipper>();
			var shipper = new Shipper { ShipperId = 1, CompanyName = "Speedy Express", PhoneNumber = "(503) 555-9831", Reference = new Guid("356E4A7E-B027-4321-BA40-E2677E6502CF") }; session.Insert(shipper); shippers.Add(shipper);
			shipper = new Shipper { ShipperId = 2, CompanyName = "United Package", PhoneNumber = "(503) 555-3199", Reference = new Guid("6DFCD0D7-4D2E-4525-A502-3EA9AA52E965") }; session.Insert(shipper); shippers.Add(shipper);
			shipper = new Shipper { ShipperId = 3, CompanyName = "Federal Shipping", PhoneNumber = "(503) 555-9931", Reference = new Guid("716F114B-E253-4166-8C76-46E6F340B58F") }; session.Insert(shipper); shippers.Add(shipper);

			var categories = new List<ProductCategory>();
			var category = new ProductCategory { CategoryId = 1, Name = "Beverages", Description = "Soft drinks, coffees, teas, beers, and ales" }; session.Insert(category); categories.Add(category);
			category = new ProductCategory { CategoryId = 2, Name = "Condiments", Description = "Sweet and savory sauces, relishes, spreads, and seasonings" }; session.Insert(category); categories.Add(category);
			category = new ProductCategory { CategoryId = 3, Name = "Confections", Description = "Desserts, candies, and sweet breads" }; session.Insert(category); categories.Add(category);
			category = new ProductCategory { CategoryId = 4, Name = "Dairy Products", Description = "Cheeses" }; session.Insert(category); categories.Add(category);
			category = new ProductCategory { CategoryId = 5, Name = "Grains/Cereals", Description = "Breads, crackers, pasta, and cereal" }; session.Insert(category); categories.Add(category);
			category = new ProductCategory { CategoryId = 6, Name = "Meat/Poultry", Description = "Prepared meats" }; session.Insert(category); categories.Add(category);
			category = new ProductCategory { CategoryId = 7, Name = "Produce", Description = "Dried fruit and bean curd" }; session.Insert(category); categories.Add(category);
			category = new ProductCategory { CategoryId = 8, Name = "Seafood", Description = "Seaweed and fish" }; session.Insert(category); categories.Add(category);

			var suppliers = new List<Supplier>();
			var supplier = new Supplier { SupplierId = 1, CompanyName = "Exotic Liquids", ContactName = "Charlotte Cooper", ContactTitle = "Purchasing Manager", HomePage = "", Address = new Address("49 Gilbert St.", "London", "", "EC1 4SD", "UK", "(171) 555-2222", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 2, CompanyName = "New Orleans Cajun Delights", ContactName = "Shelley Burke", ContactTitle = "Order Administrator", HomePage = "#CAJUN.HTM#", Address = new Address("P.O. Box 78934", "New Orleans", "LA", "70117", "USA", "(100) 555-4822", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 3, CompanyName = "Grandma Kelly's Homestead", ContactName = "Regina Murphy", ContactTitle = "Sales Representative", HomePage = "", Address = new Address("707 Oxford Rd.", "Ann Arbor", "MI", "48104", "USA", "(313) 555-5735", "(313) 555-3349") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 4, CompanyName = "Tokyo Traders", ContactName = "Yoshi Nagase", ContactTitle = "Marketing Manager", HomePage = "", Address = new Address("9-8 Sekimai Musashino-shi", "Tokyo", "", "100", "Japan", "(03) 3555-5011", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 5, CompanyName = "Cooperativa de Quesos 'Las Cabras'", ContactName = "Antonio del Valle Saavedra", ContactTitle = "Export Administrator", HomePage = "", Address = new Address("Calle del Rosal 4", "Oviedo", "Asturias", "33007", "Spain", "(98) 598 76 54", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 6, CompanyName = "Mayumi's", ContactName = "Mayumi Ohno", ContactTitle = "Marketing Representative", HomePage = "Mayumi's (on the World Wide Web)#http://www.microsoft.com/accessdev/sampleapps/mayumi.htm#", Address = new Address("92 Setsuko Chuo-ku", "Osaka", "", "545", "Japan", "(06) 431-7877", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 7, CompanyName = "Pavlova, Ltd.", ContactName = "Ian Devling", ContactTitle = "Marketing Manager", HomePage = "", Address = new Address("74 Rose St. Moonie Ponds", "Melbourne", "Victoria", "3058", "Australia", "(03) 444-2343", "(03) 444-6588") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 8, CompanyName = "Specialty Biscuits, Ltd.", ContactName = "Peter Wilson", ContactTitle = "Sales Representative", HomePage = "", Address = new Address("29 King's Way", "Manchester", "", "M14 GSD", "UK", "(161) 555-4448", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 9, CompanyName = "PB Knckebrd AB", ContactName = "Lars Peterson", ContactTitle = "Sales Agent", HomePage = "", Address = new Address("Kaloadagatan 13", "Gteborg", "", "S-345 67", "Sweden", "031-987 65 43", "031-987 65 91") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 10, CompanyName = "Refrescos Americanas LTDA", ContactName = "Carlos Diaz", ContactTitle = "Marketing Manager", HomePage = "", Address = new Address("Av. das Americanas 12.890", "Sao Paulo", "", "5442", "Brazil", "(11) 555 4640", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 11, CompanyName = "Heli Swaren GmbH & Co. KG", ContactName = "Petra Winkler", ContactTitle = "Sales Manager", HomePage = "", Address = new Address("Tiergartenstrae 5", "Berlin", "", "10785", "Germany", "(010) 9984510", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 12, CompanyName = "Plutzer Lebensmittelgromrkte AG", ContactName = "Martin Bein", ContactTitle = "International Marketing Mgr.", HomePage = "Plutzer (on the World Wide Web)#http://www.microsoft.com/accessdev/sampleapps/plutzer.htm#", Address = new Address("Bogenallee 51", "Frankfurt", "", "60439", "Germany", "(069) 992755", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 13, CompanyName = "Nord-Ost-Fisch Handelsgesellschaft mbH", ContactName = "Sven Petersen", ContactTitle = "Coordinator Foreign Markets", HomePage = "", Address = new Address("Frahmredder 112a", "Cuxhaven", "", "27478", "Germany", "(04721) 8713", "(04721) 8714") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 14, CompanyName = "Formaggi Fortini s.r.l.", ContactName = "Elio Rossi", ContactTitle = "Sales Representative", HomePage = "#FORMAGGI.HTM#", Address = new Address("Viale Dante, 75", "Ravenna", "", "48100", "Italy", "(0544) 60323", "(0544) 60603") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 15, CompanyName = "Norske Meierier", ContactName = "Beate Vileid", ContactTitle = "Marketing Manager", HomePage = "", Address = new Address("Hatlevegen 5", "Sandvika", "", "1320", "Norway", "(0)2-953010", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 16, CompanyName = "Bigfoot Breweries", ContactName = "Cheryl Saylor", ContactTitle = "Regional Account Rep.", HomePage = "", Address = new Address("3400 - 8th Avenue Suite 210", "Bend", "OR", "97101", "USA", "(503) 555-9931", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 17, CompanyName = "Svensk Sjfda AB", ContactName = "Michael Bjrn", ContactTitle = "Sales Representative", HomePage = "", Address = new Address("Brovallavgen 231", "Stockholm", "", "S-123 45", "Sweden", "08-123 45 67", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 18, CompanyName = "Aux joyeux ecclsiastiques", ContactName = "Guylne Nodier", ContactTitle = "Sales Manager", HomePage = "", Address = new Address("203, Rue des Francs-Bourgeois", "Paris", "", "75004", "France", "(1) 03.83.00.68", "(1) 03.83.00.62") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 19, CompanyName = "New England Seafood Cannery", ContactName = "Robb Merchant", ContactTitle = "Wholesale Account Agent", HomePage = "", Address = new Address("Order Processing Dept. 2100 Paul Revere Blvd.", "Boston", "MA", "02134", "USA", "(617) 555-3267", "(617) 555-3389") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 20, CompanyName = "Leka Trading", ContactName = "Chandra Leka", ContactTitle = "Owner", HomePage = "", Address = new Address("471 Serangoon Loop, Suite #402", "Singapore", "", "0512", "Singapore", "555-8787", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 21, CompanyName = "Lyngbysild", ContactName = "Niels Petersen", ContactTitle = "Sales Manager", HomePage = "", Address = new Address("Lyngbysild Fiskebakken 10", "Lyngby", "", "2800", "Denmark", "43844108", "43844115") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 22, CompanyName = "Zaanse Snoepfabriek", ContactName = "Dirk Luchte", ContactTitle = "Accounting Manager", HomePage = "", Address = new Address("Verkoop Rijnweg 22", "Zaandam", "", "9999 ZZ", "Netherlands", "(12345) 1212", "(12345) 1210") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 23, CompanyName = "Karkki Oy", ContactName = "Anne Heikkonen", ContactTitle = "Product Manager", HomePage = "", Address = new Address("Valtakatu 12", "Lappeenranta", "", "53120", "Finland", "(953) 10956", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 24, CompanyName = "G'day, Mate", ContactName = "Wendy Mackenzie", ContactTitle = "Sales Representative", HomePage = "G'day Mate (on the World Wide Web)#http://www.microsoft.com/accessdev/sampleapps/gdaymate.htm#", Address = new Address("170 Prince Edward Parade Hunter's Hill", "Sydney", "NSW", "2042", "Australia", "(02) 555-5914", "(02) 555-4873") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 25, CompanyName = "Ma Maison", ContactName = "Jean-Guy Lauzon", ContactTitle = "Marketing Manager", HomePage = "", Address = new Address("2960 Rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", "(514) 555-9022", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 26, CompanyName = "Pasta Buttini s.r.l.", ContactName = "Giovanni Giudici", ContactTitle = "Order Administrator", HomePage = "", Address = new Address("Via dei Gelsomini, 153", "Salerno", "", "84100", "Italy", "(089) 6547665", "(089) 6547667") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 27, CompanyName = "Escargots Nouveaux", ContactName = "Marie Delamare", ContactTitle = "Sales Manager", HomePage = "", Address = new Address("22, rue H. Voiron", "Montceau", "", "71300", "France", "85.57.00.07", "") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 28, CompanyName = "Gai pturage", ContactName = "Eliane Noz", ContactTitle = "Sales Representative", HomePage = "", Address = new Address("Bat. B 3, rue des Alpes", "Annecy", "", "74000", "France", "38.76.98.06", "38.76.98.58") }; session.Insert(supplier); suppliers.Add(supplier);
			supplier = new Supplier { SupplierId = 29, CompanyName = "Forts d'rables", ContactName = "Chantal Goulet", ContactTitle = "Accounting Manager", HomePage = "", Address = new Address("148 rue Chasseur", "Ste-Hyacinthe", "Qubec", "J2S 7S8", "Canada", "(514) 555-2955", "(514) 555-2921") }; session.Insert(supplier); suppliers.Add(supplier);

			var employees = new List<Employee>();
			var employee = new Employee { EmployeeId = 1, LastName = "Davolio", FirstName = "Nancy", Title = "Sales Representative", TitleOfCourtesy = "Ms.", BirthDate = DateTime.Parse("Dec  8 1948 12:00AM"), HireDate = DateTime.Parse("May  1 1992 12:00AM"), Address = new Address("507 - 20th Ave. E.Apt. 2A", "Seattle", "WA", "98122", "USA", "(206) 555-9857", null), Extension = "5467", Notes = "Education includes a BA in psychology from Colorado State University in 1970.  She also completed 'The Art of the Cold Call.'  Nancy is a member of Toastmasters International." }; session.Insert(employee); employees.Add(employee);
			employee = new Employee { EmployeeId = 2, LastName = "Fuller", FirstName = "Andrew", Title = "Vice President, Sales", TitleOfCourtesy = "Dr.", BirthDate = DateTime.Parse("Feb 19 1952 12:00AM"), HireDate = DateTime.Parse("Aug 14 1992 12:00AM"), Address = new Address("908 W. Capital Way", "Tacoma", "WA", "98401", "USA", "(206) 555-9482", null), Extension = "3457", Notes = "Andrew received his BTS commercial in 1974 and a Ph.D. in international marketing from the University of Dallas in 1981.  He is fluent in French and Italian and reads German.  He joined the company as a sales representative, was promoted to sales manager in January 1992 and to vice president of sales in March 1993.  Andrew is a member of the Sales Management Roundtable, the Seattle Chamber of Commerce, and the Pacific Rim Importers Association." }; session.Insert(employee); employees.Add(employee);
			employee = new Employee { EmployeeId = 3, LastName = "Leverling", FirstName = "Janet", Title = "Sales Representative", TitleOfCourtesy = "Ms.", BirthDate = DateTime.Parse("Aug 30 1963 12:00AM"), HireDate = DateTime.Parse("Apr  1 1992 12:00AM"), Address = new Address("722 Moss Bay Blvd.", "Kirkland", "WA", "98033", "USA", "(206) 555-3412", null), Extension = "3355", Notes = "Janet has a BS degree in chemistry from Boston College (1984).  She has also completed a certificate program in food retailing management.  Janet was hired as a sales associate in 1991 and promoted to sales representative in February 1992." }; session.Insert(employee); employees.Add(employee);
			employee = new Employee { EmployeeId = 4, LastName = "Peacock", FirstName = "Margaret", Title = "Sales Representative", TitleOfCourtesy = "Mrs.", BirthDate = DateTime.Parse("Sep 19 1937 12:00AM"), HireDate = DateTime.Parse("May  3 1993 12:00AM"), Address = new Address("4110 Old Redmond Rd.", "Redmond", "WA", "98052", "USA", "(206) 555-8122", null), Extension = "5176", Notes = "Margaret holds a BA in English literature from Concordia College (1958) and an MA from the American Institute of Culinary Arts (1966).  She was assigned to the London office temporarily from July through November 1992." }; session.Insert(employee); employees.Add(employee);
			employee = new Employee { EmployeeId = 5, LastName = "Buchanan", FirstName = "Steven", Title = "Sales Manager", TitleOfCourtesy = "Mr.", BirthDate = DateTime.Parse("Mar  4 1955 12:00AM"), HireDate = DateTime.Parse("Oct 17 1993 12:00AM"), Address = new Address("14 Garrett Hill", "London", "", "SW1 8JR", "UK", "(71) 555-4848", null), Extension = "3453", Notes = "Steven Buchanan graduated from St. Andrews University, Scotland, with a BSC degree in 1976.  Upon joining the company as a sales representative in 1992, he spent 6 months in an orientation program at the Seattle office and then returned to his permanent post in London.  He was promoted to sales manager in March 1993.  Mr. Buchanan has completed the courses 'Successful Telemarketing' and 'International Sales Management.'  He is fluent in French." }; session.Insert(employee); employees.Add(employee);
			employee = new Employee { EmployeeId = 6, LastName = "Suyama", FirstName = "Michael", Title = "Sales Representative", TitleOfCourtesy = "Mr.", BirthDate = DateTime.Parse("Jul  2 1963 12:00AM"), HireDate = DateTime.Parse("Oct 17 1993 12:00AM"), Address = new Address("Coventry HouseMiner Rd.", "London", "", "EC2 7JR", "UK", "(71) 555-7773", null), Extension = "428", Notes = "Michael is a graduate of Sussex University (MA, economics, 1983) and the University of California at Los Angeles (MBA, marketing, 1986).  He has also taken the courses 'Multi-Cultural Selling' and 'Time Management for the Sales Professional.'  He is fluent in Japanese and can read and write French, Portuguese, and Spanish." }; session.Insert(employee); employees.Add(employee);
			employee = new Employee { EmployeeId = 7, LastName = "King", FirstName = "Robert", Title = "Sales Representative", TitleOfCourtesy = "Mr.", BirthDate = DateTime.Parse("May 29 1960 12:00AM"), HireDate = DateTime.Parse("Jan  2 1994 12:00AM"), Address = new Address("Edgeham HollowWinchester Way", "London", "", "RG1 9SP", "UK", "(71) 555-5598", null), Extension = "465", Notes = "Robert King served in the Peace Corps and traveled extensively before completing his degree in English at the University of Michigan in 1992, the year he joined the company.  After completing a course entitled 'Selling in Europe,' he was transferred to the London office in March 1993." }; session.Insert(employee); employees.Add(employee);
			employee = new Employee { EmployeeId = 8, LastName = "Callahan", FirstName = "Laura", Title = "Inside Sales Coordinator", TitleOfCourtesy = "Ms.", BirthDate = DateTime.Parse("Jan  9 1958 12:00AM"), HireDate = DateTime.Parse("Mar  5 1994 12:00AM"), Address = new Address("4726 - 11th Ave. N.E.", "Seattle", "WA", "98105", "USA", "(206) 555-1189", null), Extension = "2344", Notes = "Laura received a BA in psychology from the University of Washington.  She has also completed a course in business French.  She reads and writes French." }; session.Insert(employee); employees.Add(employee);
			employee = new Employee { EmployeeId = 9, LastName = "Dodsworth", FirstName = "Anne", Title = "Sales Representative", TitleOfCourtesy = "Ms.", BirthDate = DateTime.Parse("Jan 27 1966 12:00AM"), HireDate = DateTime.Parse("Nov 15 1994 12:00AM"), Address = new Address("7 Houndstooth Rd.", "London", "", "WG2 7LT", "UK", "(71) 555-4444", null), Extension = "452", Notes = "Anne has a BA degree in English from St. Lawrence College.  She is fluent in French and German." }; session.Insert(employee); employees.Add(employee);

			employees.Where(e => e.LastName == "Davolio").First().Superior = employees.Where(e => e.LastName == "Fuller").First();
			employees.Where(e => e.LastName == "Leverling").First().Superior = employees.Where(e => e.LastName == "Fuller").First();
			employees.Where(e => e.LastName == "Peacock").First().Superior = employees.Where(e => e.LastName == "Fuller").First();
			employees.Where(e => e.LastName == "Buchanan").First().Superior = employees.Where(e => e.LastName == "Fuller").First();
			employees.Where(e => e.LastName == "Suyama").First().Superior = employees.Where(e => e.LastName == "Buchanan").First();
			employees.Where(e => e.LastName == "King").First().Superior = employees.Where(e => e.LastName == "Buchanan").First();
			employees.Where(e => e.LastName == "Callahan").First().Superior = employees.Where(e => e.LastName == "Fuller").First();
			employees.Where(e => e.LastName == "Dodsworth").First().Superior = employees.Where(e => e.LastName == "Buchanan").First();

			var customers = new List<Customer>();
			var customer = new Customer { CustomerId = "ALFKI", CompanyName = "Alfreds Futterkiste", ContactName = "Maria Anders", ContactTitle = "Sales Representative", Address = new Address("Obere Str. 57", "Berlin", "", "12209", "Germany", "030-0074321", "030-0076545") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "ANATR", CompanyName = "Ana Trujillo Emparedados y helados", ContactName = "Ana Trujillo", ContactTitle = "Owner", Address = new Address("Avda. de la Constitucin 2222", "Mxico D.F.", "", "05021", "Mexico", "(5) 555-4729", "(5) 555-3745") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "ANTON", CompanyName = "Antonio Moreno Taquera", ContactName = "Antonio Moreno", ContactTitle = "Owner", Address = new Address("Mataderos  2312", "Mxico D.F.", "", "05023", "Mexico", "(5) 555-3932", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "AROUT", CompanyName = "Around the Horn", ContactName = "Thomas Hardy", ContactTitle = "Sales Representative", Address = new Address("120 Hanover Sq.", "London", "", "WA1 1DP", "UK", "(171) 555-7788", "(171) 555-6750") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "BERGS", CompanyName = "Berglunds snabbkp", ContactName = "Christina Berglund", ContactTitle = "Order Administrator", Address = new Address("Berguvsvgen  8", "Lule", "", "S-958 22", "Sweden", "0921-12 34 65", "0921-12 34 67") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "BLAUS", CompanyName = "Blauer See Delikatessen", ContactName = "Hanna Moos", ContactTitle = "Sales Representative", Address = new Address("Forsterstr. 57", "Mannheim", "", "68306", "Germany", "0621-08460", "0621-08924") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "BLONP", CompanyName = "Blondesddsl pre et fils", ContactName = "Frdrique Citeaux", ContactTitle = "Marketing Manager", Address = new Address("24, place Klber", "Strasbourg", "", "67000", "France", "88.60.15.31", "88.60.15.32") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "BOLID", CompanyName = "Blido Comidas preparadas", ContactName = "Martn Sommer", ContactTitle = "Owner", Address = new Address("C/ Araquil, 67", "Madrid", "", "28023", "Spain", "(91) 555 22 82", "(91) 555 91 99") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "BONAP", CompanyName = "Bon app'", ContactName = "Laurence Lebihan", ContactTitle = "Owner", Address = new Address("12, rue des Bouchers", "Marseille", "", "13008", "France", "91.24.45.40", "91.24.45.41") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "BOTTM", CompanyName = "Bottom-Dollar Markets", ContactName = "Elizabeth Lincoln", ContactTitle = "Accounting Manager", Address = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", "(604) 555-4729", "(604) 555-3745") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "BSBEV", CompanyName = "B's Beverages", ContactName = "Victoria Ashworth", ContactTitle = "Sales Representative", Address = new Address("Fauntleroy Circus", "London", "", "EC2 5NT", "UK", "(171) 555-1212", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "CACTU", CompanyName = "Cactus Comidas para llevar", ContactName = "Patricio Simpson", ContactTitle = "Sales Agent", Address = new Address("Cerrito 333", "Buenos Aires", "", "1010", "Argentina", "(1) 135-5555", "(1) 135-4892") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "CENTC", CompanyName = "Centro comercial Moctezuma", ContactName = "Francisco Chang", ContactTitle = "Marketing Manager", Address = new Address("Sierras de Granada 9993", "Mxico D.F.", "", "05022", "Mexico", "(5) 555-3392", "(5) 555-7293") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "CHOPS", CompanyName = "Chop-suey Chinese", ContactName = "Yang Wang", ContactTitle = "Owner", Address = new Address("Hauptstr. 29", "Bern", "", "3012", "Switzerland", "0452-076545", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "COMMI", CompanyName = "Comrcio Mineiro", ContactName = "Pedro Afonso", ContactTitle = "Sales Associate", Address = new Address("Av. dos Lusadas, 23", "Sao Paulo", "SP", "05432-043", "Brazil", "(11) 555-7647", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "CONSH", CompanyName = "Consolidated Holdings", ContactName = "Elizabeth Brown", ContactTitle = "Sales Representative", Address = new Address("Berkeley Gardens 12  Brewery", "London", "", "WX1 6LT", "UK", "(171) 555-2282", "(171) 555-9199") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "DRACD", CompanyName = "Drachenblut Delikatessen", ContactName = "Sven Ottlieb", ContactTitle = "Order Administrator", Address = new Address("Walserweg 21", "Aachen", "", "52066", "Germany", "0241-039123", "0241-059428") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "DUMON", CompanyName = "Du monde entier", ContactName = "Janine Labrune", ContactTitle = "Owner", Address = new Address("67, rue des Cinquante Otages", "Nantes", "", "44000", "France", "40.67.88.88", "40.67.89.89") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "EASTC", CompanyName = "Eastern Connection", ContactName = "Ann Devon", ContactTitle = "Sales Agent", Address = new Address("35 King George", "London", "", "WX3 6FW", "UK", "(171) 555-0297", "(171) 555-3373") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "ERNSH", CompanyName = "Ernst Handel", ContactName = "Roland Mendel", ContactTitle = "Sales Manager", Address = new Address("Kirchgasse 6", "Graz", "", "8010", "Austria", "7675-3425", "7675-3426") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "FAMIA", CompanyName = "Familia Arquibaldo", ContactName = "Aria Cruz", ContactTitle = "Marketing Assistant", Address = new Address("Rua Ors, 92", "Sao Paulo", "SP", "05442-030", "Brazil", "(11) 555-9857", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "FISSA", CompanyName = "FISSA Fabrica Inter. Salchichas S.A.", ContactName = "Diego Roel", ContactTitle = "Accounting Manager", Address = new Address("C/ Moralzarzal, 86", "Madrid", "", "28034", "Spain", "(91) 555 94 44", "(91) 555 55 93") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "FOLIG", CompanyName = "Folies gourmandes", ContactName = "Martine Ranc", ContactTitle = "Assistant Sales Agent", Address = new Address("184, chausse de Tournai", "Lille", "", "59000", "France", "20.16.10.16", "20.16.10.17") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "FOLKO", CompanyName = "Folk och f HB", ContactName = "Maria Larsson", ContactTitle = "Owner", Address = new Address("kergatan 24", "Brcke", "", "S-844 67", "Sweden", "0695-34 67 21", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "FRANK", CompanyName = "Frankenversand", ContactName = "Peter Franken", ContactTitle = "Marketing Manager", Address = new Address("Berliner Platz 43", "Mnchen", "", "80805", "Germany", "089-0877310", "089-0877451") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "FRANR", CompanyName = "France restauration", ContactName = "Carine Schmitt", ContactTitle = "Marketing Manager", Address = new Address("54, rue Royale", "Nantes", "", "44000", "France", "40.32.21.21", "40.32.21.20") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "FRANS", CompanyName = "Franchi S.p.A.", ContactName = "Paolo Accorti", ContactTitle = "Sales Representative", Address = new Address("Via Monte Bianco 34", "Torino", "", "10100", "Italy", "011-4988260", "011-4988261") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "FURIB", CompanyName = "Furia Bacalhau e Frutos do Mar", ContactName = "Lino Rodriguez", ContactTitle = "Sales Manager", Address = new Address("Jardim das rosas n. 32", "Lisboa", "", "1675", "Portugal", "(1) 354-2534", "(1) 354-2535") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "GALED", CompanyName = "Galera del gastrnomo", ContactName = "Eduardo Saavedra", ContactTitle = "Marketing Manager", Address = new Address("Rambla de Catalua, 23", "Barcelona", "", "08022", "Spain", "(93) 203 4560", "(93) 203 4561") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "GODOS", CompanyName = "Godos Cocina Tpica", ContactName = "Jos Pedro Freyre", ContactTitle = "Sales Manager", Address = new Address("C/ Romero, 33", "Sevilla", "", "41101", "Spain", "(95) 555 82 82", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "GOURL", CompanyName = "Gourmet Lanchonetes", ContactName = "Andr Fonseca", ContactTitle = "Sales Associate", Address = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", "(11) 555-9482", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "GREAL", CompanyName = "Great Lakes Food Market", ContactName = "Howard Snyder", ContactTitle = "Marketing Manager", Address = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", "(503) 555-7555", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "GROSR", CompanyName = "GROSELLA-Restaurante", ContactName = "Manuel Pereira", ContactTitle = "Owner", Address = new Address("5 Ave. Los Palos Grandes", "Caracas", "DF", "1081", "Venezuela", "(2) 283-2951", "(2) 283-3397") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "HANAR", CompanyName = "Hanari Carnes", ContactName = "Mario Pontes", ContactTitle = "Accounting Manager", Address = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", "(21) 555-0091", "(21) 555-8765") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "HILAA", CompanyName = "HILARION-Abastos", ContactName = "Carlos Hernndez", ContactTitle = "Sales Representative", Address = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", "(5) 555-1340", "(5) 555-1948") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "HUNGC", CompanyName = "Hungry Coyote Import Store", ContactName = "Yoshi Latimer", ContactTitle = "Sales Representative", Address = new Address("City Center Plaza 516 Main St.", "Elgin", "OR", "97827", "USA", "(503) 555-6874", "(503) 555-2376") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "HUNGO", CompanyName = "Hungry Owl All-Night Grocers", ContactName = "Patricia McKenna", ContactTitle = "Sales Associate", Address = new Address("8 Johnstown Road", "Cork", "Co. Cork", "", "Ireland", "2967 542", "2967 3333") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "ISLAT", CompanyName = "Island Trading", ContactName = "Helen Bennett", ContactTitle = "Marketing Manager", Address = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", "(198) 555-8888", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "KOENE", CompanyName = "Kniglich Essen", ContactName = "Philip Cramer", ContactTitle = "Sales Associate", Address = new Address("Maubelstr. 90", "Brandenburg", "", "14776", "Germany", "0555-09876", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "LACOR", CompanyName = "La corne d'abondance", ContactName = "Daniel Tonini", ContactTitle = "Sales Representative", Address = new Address("67, avenue de l'Europe", "Versailles", "", "78000", "France", "30.59.84.10", "30.59.85.11") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "LAMAI", CompanyName = "La maison d'Asie", ContactName = "Annette Roulet", ContactTitle = "Sales Manager", Address = new Address("1 rue Alsace-Lorraine", "Toulouse", "", "31000", "France", "61.77.61.10", "61.77.61.11") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "LAUGB", CompanyName = "Laughing Bacchus Wine Cellars", ContactName = "Yoshi Tannamuri", ContactTitle = "Marketing Assistant", Address = new Address("1900 Oak St.", "Vancouver", "BC", "V3F 2K1", "Canada", "(604) 555-3392", "(604) 555-7293") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "LAZYK", CompanyName = "Lazy K Kountry Store", ContactName = "John Steel", ContactTitle = "Marketing Manager", Address = new Address("12 Orchestra Terrace", "Walla Walla", "WA", "99362", "USA", "(509) 555-7969", "(509) 555-6221") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "LEHMS", CompanyName = "Lehmanns Marktstand", ContactName = "Renate Messner", ContactTitle = "Sales Representative", Address = new Address("Magazinweg 7", "Frankfurt a.M.", "", "60528", "Germany", "069-0245984", "069-0245874") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "LETSS", CompanyName = "Let's Stop N Shop", ContactName = "Jaime Yorres", ContactTitle = "Owner", Address = new Address("87 Polk St. Suite 5", "San Francisco", "CA", "94117", "USA", "(415) 555-5938", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "LILAS", CompanyName = "LILA-Supermercado", ContactName = "Carlos Gonzlez", ContactTitle = "Accounting Manager", Address = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", "(9) 331-6954", "(9) 331-7256") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "LINOD", CompanyName = "LINO-Delicateses", ContactName = "Felipe Izquierdo", ContactTitle = "Owner", Address = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", "(8) 34-56-12", "(8) 34-93-93") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "LONEP", CompanyName = "Lonesome Pine Restaurant", ContactName = "Fran Wilson", ContactTitle = "Sales Manager", Address = new Address("89 Chiaroscuro Rd.", "Portland", "OR", "97219", "USA", "(503) 555-9573", "(503) 555-9646") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "MAGAA", CompanyName = "Magazzini Alimentari Riuniti", ContactName = "Giovanni Rovelli", ContactTitle = "Marketing Manager", Address = new Address("Via Ludovico il Moro 22", "Bergamo", "", "24100", "Italy", "035-640230", "035-640231") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "MAISD", CompanyName = "Maison Dewey", ContactName = "Catherine Dewey", ContactTitle = "Sales Agent", Address = new Address("Rue Joseph-Bens 532", "Bruxelles", "", "B-1180", "Belgium", "(02) 201 24 67", "(02) 201 24 68") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "MEREP", CompanyName = "Mre Paillarde", ContactName = "Jean Fresnire", ContactTitle = "Marketing Assistant", Address = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", "(514) 555-8054", "(514) 555-8055") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "MORGK", CompanyName = "Morgenstern Gesundkost", ContactName = "Alexander Feuer", ContactTitle = "Marketing Assistant", Address = new Address("Heerstr. 22", "Leipzig", "", "04179", "Germany", "0342-023176", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "NORTS", CompanyName = "North/South", ContactName = "Simon Crowther", ContactTitle = "Sales Associate", Address = new Address("South House 300 Queensbridge", "London", "", "SW7 1RZ", "UK", "(171) 555-7733", "(171) 555-2530") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "OCEAN", CompanyName = "Ocano Atlntico Ltda.", ContactName = "Yvonne Moncada", ContactTitle = "Sales Agent", Address = new Address("Ing. Gustavo Moncada 8585 Piso 20-A", "Buenos Aires", "", "1010", "Argentina", "(1) 135-5333", "(1) 135-5535") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "OLDWO", CompanyName = "Old World Delicatessen", ContactName = "Rene Phillips", ContactTitle = "Sales Representative", Address = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", "(907) 555-7584", "(907) 555-2880") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "OTTIK", CompanyName = "Ottilies Kseladen", ContactName = "Henriette Pfalzheim", ContactTitle = "Owner", Address = new Address("Mehrheimerstr. 369", "Kln", "", "50739", "Germany", "0221-0644327", "0221-0765721") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "PARIS", CompanyName = "Paris spcialits", ContactName = "Marie Bertrand", ContactTitle = "Owner", Address = new Address("265, boulevard Charonne", "Paris", "", "75012", "France", "(1) 42.34.22.66", "(1) 42.34.22.77") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "PERIC", CompanyName = "Pericles Comidas clsicas", ContactName = "Guillermo Fernndez", ContactTitle = "Sales Representative", Address = new Address("Calle Dr. Jorge Cash 321", "Mxico D.F.", "", "05033", "Mexico", "(5) 552-3745", "(5) 545-3745") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "PICCO", CompanyName = "Piccolo und mehr", ContactName = "Georg Pipps", ContactTitle = "Sales Manager", Address = new Address("Geislweg 14", "Salzburg", "", "5020", "Austria", "6562-9722", "6562-9723") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "PRINI", CompanyName = "Princesa Isabel Vinhos", ContactName = "Isabel de Castro", ContactTitle = "Sales Representative", Address = new Address("Estrada da sade n. 58", "Lisboa", "", "1756", "Portugal", "(1) 356-5634", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "QUEDE", CompanyName = "Que Delcia", ContactName = "Bernardo Batista", ContactTitle = "Accounting Manager", Address = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", "(21) 555-4252", "(21) 555-4545") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "QUEEN", CompanyName = "Queen Cozinha", ContactName = "Lcia Carvalho", ContactTitle = "Marketing Assistant", Address = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", "(11) 555-1189", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "QUICK", CompanyName = "QUICK-Stop", ContactName = "Horst Kloss", ContactTitle = "Accounting Manager", Address = new Address("Taucherstrae 10", "Cunewalde", "", "01307", "Germany", "0372-035188", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "RANCH", CompanyName = "Rancho grande", ContactName = "Sergio Gutirrez", ContactTitle = "Sales Representative", Address = new Address("Av. del Libertador 900", "Buenos Aires", "", "1010", "Argentina", "(1) 123-5555", "(1) 123-5556") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "RATTC", CompanyName = "Rattlesnake Canyon Grocery", ContactName = "Paula Wilson", ContactTitle = "Assistant Sales Representative", Address = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", "(505) 555-5939", "(505) 555-3620") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "REGGC", CompanyName = "Reggiani Caseifici", ContactName = "Maurizio Moroni", ContactTitle = "Sales Associate", Address = new Address("Strada Provinciale 124", "Reggio Emilia", "", "42100", "Italy", "0522-556721", "0522-556722") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "RICAR", CompanyName = "Ricardo Adocicados", ContactName = "Janete Limeira", ContactTitle = "Assistant Sales Agent", Address = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", "(21) 555-3412", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "RICSU", CompanyName = "Richter Supermarkt", ContactName = "Michael Holz", ContactTitle = "Sales Manager", Address = new Address("Grenzacherweg 237", "Genve", "", "1203", "Switzerland", "0897-034214", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "ROMEY", CompanyName = "Romero y tomillo", ContactName = "Alejandra Camino", ContactTitle = "Accounting Manager", Address = new Address("Gran Va, 1", "Madrid", "", "28001", "Spain", "(91) 745 6200", "(91) 745 6210") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "SANTG", CompanyName = "Sant Gourmet", ContactName = "Jonas Bergulfsen", ContactTitle = "Owner", Address = new Address("Erling Skakkes gate 78", "Stavern", "", "4110", "Norway", "07-98 92 35", "07-98 92 47") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "SAVEA", CompanyName = "Save-a-lot Markets", ContactName = "Jose Pavarotti", ContactTitle = "Sales Representative", Address = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", "(208) 555-8097", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "SEVES", CompanyName = "Seven Seas Imports", ContactName = "Hari Kumar", ContactTitle = "Sales Manager", Address = new Address("90 Wadhurst Rd.", "London", "", "OX15 4NB", "UK", "(171) 555-1717", "(171) 555-5646") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "SIMOB", CompanyName = "Simons bistro", ContactName = "Jytte Petersen", ContactTitle = "Owner", Address = new Address("Vinbltet 34", "Kobenhavn", "", "1734", "Denmark", "31 12 34 56", "31 13 35 57") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "SPECD", CompanyName = "Spcialits du monde", ContactName = "Dominique Perrier", ContactTitle = "Marketing Manager", Address = new Address("25, rue Lauriston", "Paris", "", "75016", "France", "(1) 47.55.60.10", "(1) 47.55.60.20") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "SPLIR", CompanyName = "Split Rail Beer & Ale", ContactName = "Art Braunschweiger", ContactTitle = "Sales Manager", Address = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", "(307) 555-4680", "(307) 555-6525") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "SUPRD", CompanyName = "Suprmes dlices", ContactName = "Pascale Cartrain", ContactTitle = "Accounting Manager", Address = new Address("Boulevard Tirou, 255", "Charleroi", "", "B-6000", "Belgium", "(071) 23 67 22 20", "(071) 23 67 22 21") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "THEBI", CompanyName = "The Big Cheese", ContactName = "Liz Nixon", ContactTitle = "Marketing Manager", Address = new Address("89 Jefferson Way Suite 2", "Portland", "OR", "97201", "USA", "(503) 555-3612", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "THECR", CompanyName = "The Cracker Box", ContactName = "Liu Wong", ContactTitle = "Marketing Assistant", Address = new Address("55 Grizzly Peak Rd.", "Butte", "MT", "59801", "USA", "(406) 555-5834", "(406) 555-8083") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "TOMSP", CompanyName = "Toms Spezialitten", ContactName = "Karin Josephs", ContactTitle = "Marketing Manager", Address = new Address("Luisenstr. 48", "Mnster", "", "44087", "Germany", "0251-031259", "0251-035695") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "TORTU", CompanyName = "Tortuga Restaurante", ContactName = "Miguel Angel Paolino", ContactTitle = "Owner", Address = new Address("Avda. Azteca 123", "Mxico D.F.", "", "05033", "Mexico", "(5) 555-2933", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "TRADH", CompanyName = "Tradio Hipermercados", ContactName = "Anabela Domingues", ContactTitle = "Sales Representative", Address = new Address("Av. Ins de Castro, 414", "Sao Paulo", "SP", "05634-030", "Brazil", "(11) 555-2167", "(11) 555-2168") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "TRAIH", CompanyName = "Trail's Head Gourmet Provisioners", ContactName = "Helvetius Nagy", ContactTitle = "Sales Associate", Address = new Address("722 DaVinci Blvd.", "Kirkland", "WA", "98034", "USA", "(206) 555-8257", "(206) 555-2174") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "VAFFE", CompanyName = "Vaffeljernet", ContactName = "Palle Ibsen", ContactTitle = "Sales Manager", Address = new Address("Smagsloget 45", "rhus", "", "8200", "Denmark", "86 21 32 43", "86 22 33 44") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "VICTE", CompanyName = "Victuailles en stock", ContactName = "Mary Saveley", ContactTitle = "Sales Agent", Address = new Address("2, rue du Commerce", "Lyon", "", "69004", "France", "78.32.54.86", "78.32.54.87") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "VINET", CompanyName = "Vins et alcools Chevalier", ContactName = "Paul Henriot", ContactTitle = "Accounting Manager", Address = new Address("59 rue de l'Abbaye", "Reims", "", "51100", "France", "26.47.15.10", "26.47.15.11") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "WANDK", CompanyName = "Die Wandernde Kuh", ContactName = "Rita Mller", ContactTitle = "Sales Representative", Address = new Address("Adenauerallee 900", "Stuttgart", "", "70563", "Germany", "0711-020361", "0711-035428") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "WARTH", CompanyName = "Wartian Herkku", ContactName = "Pirkko Koskitalo", ContactTitle = "Accounting Manager", Address = new Address("Torikatu 38", "Oulu", "", "90110", "Finland", "981-443655", "981-443655") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "WELLI", CompanyName = "Wellington Importadora", ContactName = "Paula Parente", ContactTitle = "Sales Manager", Address = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", "(14) 555-8122", "") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "WHITC", CompanyName = "White Clover Markets", ContactName = "Karl Jablonski", ContactTitle = "Owner", Address = new Address("305 - 14th Ave. S. Suite 3B", "Seattle", "WA", "98128", "USA", "(206) 555-4112", "(206) 555-4115") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "WILMK", CompanyName = "Wilman Kala", ContactName = "Matti Karttunen", ContactTitle = "Owner/Marketing Assistant", Address = new Address("Keskuskatu 45", "Helsinki", "", "21240", "Finland", "90-224 8858", "90-224 8858") }; session.Insert(customer); customers.Add(customer);
			customer = new Customer { CustomerId = "WOLZA", CompanyName = "Wolski  Zajazd", ContactName = "Zbyszek Piestrzeniewicz", ContactTitle = "Owner", Address = new Address("ul. Filtrowa 68", "Warszawa", "", "01-012", "Poland", "(26) 642-7012", "(26) 642-7012") }; session.Insert(customer); customers.Add(customer);

			var products = new List<Product>();
			var id = 1;
			var product = new Product { ProductId = id++, Name = "Chai", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Exotic Liquids").First(), QuantityPerUnit = "10 boxes x 20 bags", UnitPrice = 18.00M, UnitsInStock = 39, UnitsOnOrder = 0, ReorderLevel = 10, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Chang", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Exotic Liquids").First(), QuantityPerUnit = "24 - 12 oz bottles", UnitPrice = 19.00M, UnitsInStock = 17, UnitsOnOrder = 40, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Aniseed Syrup", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "Exotic Liquids").First(), QuantityPerUnit = "12 - 550 ml bottles", UnitPrice = 10.00M, UnitsInStock = 13, UnitsOnOrder = 70, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Chef Anton's Cajun Seasoning", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "New Orleans Cajun Delights").First(), QuantityPerUnit = "48 - 6 oz jars", UnitPrice = 22.00M, UnitsInStock = 53, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Chef Anton's Gumbo Mix", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "New Orleans Cajun Delights").First(), QuantityPerUnit = "36 boxes", UnitPrice = 21.35M, UnitsInStock = 0, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = true }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Grandma's Boysenberry Spread", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "Grandma Kelly's Homestead").First(), QuantityPerUnit = "12 - 8 oz jars", UnitPrice = 25.00M, UnitsInStock = 120, UnitsOnOrder = 0, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Uncle Bob's Organic Dried Pears", Category = categories.Where(c => c.Name == "Produce").First(), Supplier = suppliers.Where(s => s.CompanyName == "Grandma Kelly's Homestead").First(), QuantityPerUnit = "12 - 1 lb pkgs.", UnitPrice = 30.00M, UnitsInStock = 15, UnitsOnOrder = 0, ReorderLevel = 10, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Northwoods Cranberry Sauce", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "Grandma Kelly's Homestead").First(), QuantityPerUnit = "12 - 12 oz jars", UnitPrice = 40.00M, UnitsInStock = 6, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Mishi Kobe Niku", Category = categories.Where(c => c.Name == "Meat/Poultry").First(), Supplier = suppliers.Where(s => s.CompanyName == "Tokyo Traders").First(), QuantityPerUnit = "18 - 500 g pkgs.", UnitPrice = 97.00M, UnitsInStock = 29, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = true }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Ikura", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Tokyo Traders").First(), QuantityPerUnit = "12 - 200 ml jars", UnitPrice = 31.00M, UnitsInStock = 31, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Queso Cabrales", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Cooperativa de Quesos 'Las Cabras'").First(), QuantityPerUnit = "1 kg pkg.", UnitPrice = 21.00M, UnitsInStock = 22, UnitsOnOrder = 30, ReorderLevel = 30, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Queso Manchego La Pastora", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Cooperativa de Quesos 'Las Cabras'").First(), QuantityPerUnit = "10 - 500 g pkgs.", UnitPrice = 38.00M, UnitsInStock = 86, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Konbu", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Mayumi's").First(), QuantityPerUnit = "2 kg box", UnitPrice = 6.00M, UnitsInStock = 24, UnitsOnOrder = 0, ReorderLevel = 5, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Tofu", Category = categories.Where(c => c.Name == "Produce").First(), Supplier = suppliers.Where(s => s.CompanyName == "Mayumi's").First(), QuantityPerUnit = "40 - 100 g pkgs.", UnitPrice = 23.25M, UnitsInStock = 35, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Genen Shouyu", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "Mayumi's").First(), QuantityPerUnit = "24 - 250 ml bottles", UnitPrice = 15.50M, UnitsInStock = 39, UnitsOnOrder = 0, ReorderLevel = 5, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Pavlova", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Pavlova, Ltd.").First(), QuantityPerUnit = "32 - 500 g boxes", UnitPrice = 17.45M, UnitsInStock = 29, UnitsOnOrder = 0, ReorderLevel = 10, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Alice Mutton", Category = categories.Where(c => c.Name == "Meat/Poultry").First(), Supplier = suppliers.Where(s => s.CompanyName == "Pavlova, Ltd.").First(), QuantityPerUnit = "20 - 1 kg tins", UnitPrice = 39.00M, UnitsInStock = 0, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = true }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Carnarvon Tigers", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Pavlova, Ltd.").First(), QuantityPerUnit = "16 kg pkg.", UnitPrice = 62.50M, UnitsInStock = 42, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Teatime Chocolate Biscuits", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Specialty Biscuits, Ltd.").First(), QuantityPerUnit = "10 boxes x 12 pieces", UnitPrice = 9.20M, UnitsInStock = 25, UnitsOnOrder = 0, ReorderLevel = 5, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Sir Rodney's Marmalade", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Specialty Biscuits, Ltd.").First(), QuantityPerUnit = "30 gift boxes", UnitPrice = 81.00M, UnitsInStock = 40, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Sir Rodney's Scones", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Specialty Biscuits, Ltd.").First(), QuantityPerUnit = "24 pkgs. x 4 pieces", UnitPrice = 10.00M, UnitsInStock = 3, UnitsOnOrder = 40, ReorderLevel = 5, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Gustaf's Knckebrd", Category = categories.Where(c => c.Name == "Grains/Cereals").First(), Supplier = suppliers.Where(s => s.CompanyName == "PB Knckebrd AB").First(), QuantityPerUnit = "24 - 500 g pkgs.", UnitPrice = 21.00M, UnitsInStock = 104, UnitsOnOrder = 0, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Tunnbrd", Category = categories.Where(c => c.Name == "Grains/Cereals").First(), Supplier = suppliers.Where(s => s.CompanyName == "PB Knckebrd AB").First(), QuantityPerUnit = "12 - 250 g pkgs.", UnitPrice = 9.00M, UnitsInStock = 61, UnitsOnOrder = 0, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Guaran Fantstica", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Refrescos Americanas LTDA").First(), QuantityPerUnit = "12 - 355 ml cans", UnitPrice = 4.50M, UnitsInStock = 20, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = true }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "NuNuCa Nu-Nougat-Creme", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Heli Swaren GmbH & Co. KG").First(), QuantityPerUnit = "20 - 450 g glasses", UnitPrice = 14.00M, UnitsInStock = 76, UnitsOnOrder = 0, ReorderLevel = 30, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Gumbr Gummibrchen", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Heli Swaren GmbH & Co. KG").First(), QuantityPerUnit = "100 - 250 g bags", UnitPrice = 31.23M, UnitsInStock = 15, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Schoggi Schokolade", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Heli Swaren GmbH & Co. KG").First(), QuantityPerUnit = "100 - 100 g pieces", UnitPrice = 43.90M, UnitsInStock = 49, UnitsOnOrder = 0, ReorderLevel = 30, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Rssle Sauerkraut", Category = categories.Where(c => c.Name == "Produce").First(), Supplier = suppliers.Where(s => s.CompanyName == "Plutzer Lebensmittelgromrkte AG").First(), QuantityPerUnit = "25 - 825 g cans", UnitPrice = 45.60M, UnitsInStock = 26, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = true }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Thringer Rostbratwurst", Category = categories.Where(c => c.Name == "Meat/Poultry").First(), Supplier = suppliers.Where(s => s.CompanyName == "Plutzer Lebensmittelgromrkte AG").First(), QuantityPerUnit = "50 bags x 30 sausgs.", UnitPrice = 123.79M, UnitsInStock = 0, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = true }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Nord-Ost Matjeshering", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Nord-Ost-Fisch Handelsgesellschaft mbH").First(), QuantityPerUnit = "10 - 200 g glasses", UnitPrice = 25.89M, UnitsInStock = 10, UnitsOnOrder = 0, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Gorgonzola Telino", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Formaggi Fortini s.r.l.").First(), QuantityPerUnit = "12 - 100 g pkgs", UnitPrice = 12.50M, UnitsInStock = 0, UnitsOnOrder = 70, ReorderLevel = 20, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Mascarpone Fabioli", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Formaggi Fortini s.r.l.").First(), QuantityPerUnit = "24 - 200 g pkgs.", UnitPrice = 32.00M, UnitsInStock = 9, UnitsOnOrder = 40, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Geitost", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Norske Meierier").First(), QuantityPerUnit = "500 g", UnitPrice = 2.50M, UnitsInStock = 112, UnitsOnOrder = 0, ReorderLevel = 20, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Sasquatch Ale", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Bigfoot Breweries").First(), QuantityPerUnit = "24 - 12 oz bottles", UnitPrice = 14.00M, UnitsInStock = 111, UnitsOnOrder = 0, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Steeleye Stout", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Bigfoot Breweries").First(), QuantityPerUnit = "24 - 12 oz bottles", UnitPrice = 18.00M, UnitsInStock = 20, UnitsOnOrder = 0, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Inlagd Sill", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Svensk Sjfda AB").First(), QuantityPerUnit = "24 - 250 g  jars", UnitPrice = 19.00M, UnitsInStock = 112, UnitsOnOrder = 0, ReorderLevel = 20, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Gravad lax", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Svensk Sjfda AB").First(), QuantityPerUnit = "12 - 500 g pkgs.", UnitPrice = 26.00M, UnitsInStock = 11, UnitsOnOrder = 50, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Cte de Blaye", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Aux joyeux ecclsiastiques").First(), QuantityPerUnit = "12 - 75 cl bottles", UnitPrice = 263.50M, UnitsInStock = 17, UnitsOnOrder = 0, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Chartreuse verte", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Aux joyeux ecclsiastiques").First(), QuantityPerUnit = "750 cc per bottle", UnitPrice = 18.00M, UnitsInStock = 69, UnitsOnOrder = 0, ReorderLevel = 5, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Boston Crab Meat", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "New England Seafood Cannery").First(), QuantityPerUnit = "24 - 4 oz tins", UnitPrice = 18.40M, UnitsInStock = 123, UnitsOnOrder = 0, ReorderLevel = 30, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Jack's New England Clam Chowder", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "New England Seafood Cannery").First(), QuantityPerUnit = "12 - 12 oz cans", UnitPrice = 9.65M, UnitsInStock = 85, UnitsOnOrder = 0, ReorderLevel = 10, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Singaporean Hokkien Fried Mee", Category = categories.Where(c => c.Name == "Grains/Cereals").First(), Supplier = suppliers.Where(s => s.CompanyName == "Leka Trading").First(), QuantityPerUnit = "32 - 1 kg pkgs.", UnitPrice = 14.00M, UnitsInStock = 26, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = true }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Ipoh Coffee", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Leka Trading").First(), QuantityPerUnit = "16 - 500 g tins", UnitPrice = 46.00M, UnitsInStock = 17, UnitsOnOrder = 10, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Gula Malacca", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "Leka Trading").First(), QuantityPerUnit = "20 - 2 kg bags", UnitPrice = 19.45M, UnitsInStock = 27, UnitsOnOrder = 0, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Rogede sild", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Lyngbysild").First(), QuantityPerUnit = "1k pkg.", UnitPrice = 9.50M, UnitsInStock = 5, UnitsOnOrder = 70, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Spegesild", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Lyngbysild").First(), QuantityPerUnit = "4 - 450 g glasses", UnitPrice = 12.00M, UnitsInStock = 95, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Zaanse koeken", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Zaanse Snoepfabriek").First(), QuantityPerUnit = "10 - 4 oz boxes", UnitPrice = 9.50M, UnitsInStock = 36, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Chocolade", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Zaanse Snoepfabriek").First(), QuantityPerUnit = "10 pkgs.", UnitPrice = 12.75M, UnitsInStock = 15, UnitsOnOrder = 70, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Maxilaku", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Karkki Oy").First(), QuantityPerUnit = "24 - 50 g pkgs.", UnitPrice = 20.00M, UnitsInStock = 10, UnitsOnOrder = 60, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Valkoinen suklaa", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Karkki Oy").First(), QuantityPerUnit = "12 - 100 g bars", UnitPrice = 16.25M, UnitsInStock = 65, UnitsOnOrder = 0, ReorderLevel = 30, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Manjimup Dried Apples", Category = categories.Where(c => c.Name == "Produce").First(), Supplier = suppliers.Where(s => s.CompanyName == "G'day, Mate").First(), QuantityPerUnit = "50 - 300 g pkgs.", UnitPrice = 53.00M, UnitsInStock = 20, UnitsOnOrder = 0, ReorderLevel = 10, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Filo Mix", Category = categories.Where(c => c.Name == "Grains/Cereals").First(), Supplier = suppliers.Where(s => s.CompanyName == "G'day, Mate").First(), QuantityPerUnit = "16 - 2 kg boxes", UnitPrice = 7.00M, UnitsInStock = 38, UnitsOnOrder = 0, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Perth Pasties", Category = categories.Where(c => c.Name == "Meat/Poultry").First(), Supplier = suppliers.Where(s => s.CompanyName == "G'day, Mate").First(), QuantityPerUnit = "48 pieces", UnitPrice = 32.80M, UnitsInStock = 0, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = true }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Tourtire", Category = categories.Where(c => c.Name == "Meat/Poultry").First(), Supplier = suppliers.Where(s => s.CompanyName == "Ma Maison").First(), QuantityPerUnit = "16 pies", UnitPrice = 7.45M, UnitsInStock = 21, UnitsOnOrder = 0, ReorderLevel = 10, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Pt chinois", Category = categories.Where(c => c.Name == "Meat/Poultry").First(), Supplier = suppliers.Where(s => s.CompanyName == "Ma Maison").First(), QuantityPerUnit = "24 boxes x 2 pies", UnitPrice = 24.00M, UnitsInStock = 115, UnitsOnOrder = 0, ReorderLevel = 20, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Gnocchi di nonna Alice", Category = categories.Where(c => c.Name == "Grains/Cereals").First(), Supplier = suppliers.Where(s => s.CompanyName == "Pasta Buttini s.r.l.").First(), QuantityPerUnit = "24 - 250 g pkgs.", UnitPrice = 38.00M, UnitsInStock = 21, UnitsOnOrder = 10, ReorderLevel = 30, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Ravioli Angelo", Category = categories.Where(c => c.Name == "Grains/Cereals").First(), Supplier = suppliers.Where(s => s.CompanyName == "Pasta Buttini s.r.l.").First(), QuantityPerUnit = "24 - 250 g pkgs.", UnitPrice = 19.50M, UnitsInStock = 36, UnitsOnOrder = 0, ReorderLevel = 20, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Escargots de Bourgogne", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Escargots Nouveaux").First(), QuantityPerUnit = "24 pieces", UnitPrice = 13.25M, UnitsInStock = 62, UnitsOnOrder = 0, ReorderLevel = 20, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Raclette Courdavault", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Gai pturage").First(), QuantityPerUnit = "5 kg pkg.", UnitPrice = 55.00M, UnitsInStock = 79, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Camembert Pierrot", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Gai pturage").First(), QuantityPerUnit = "15 - 300 g rounds", UnitPrice = 34.00M, UnitsInStock = 19, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Sirop d'rable", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "Forts d'rables").First(), QuantityPerUnit = "24 - 500 ml bottles", UnitPrice = 28.50M, UnitsInStock = 113, UnitsOnOrder = 0, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Tarte au sucre", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Forts d'rables").First(), QuantityPerUnit = "48 pies", UnitPrice = 49.30M, UnitsInStock = 17, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Vegie-spread", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "Pavlova, Ltd.").First(), QuantityPerUnit = "15 - 625 g jars", UnitPrice = 43.90M, UnitsInStock = 24, UnitsOnOrder = 0, ReorderLevel = 5, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Wimmers gute Semmelkndel", Category = categories.Where(c => c.Name == "Grains/Cereals").First(), Supplier = suppliers.Where(s => s.CompanyName == "Plutzer Lebensmittelgromrkte AG").First(), QuantityPerUnit = "20 bags x 4 pieces", UnitPrice = 33.25M, UnitsInStock = 22, UnitsOnOrder = 80, ReorderLevel = 30, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Louisiana Fiery Hot Pepper Sauce", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "New Orleans Cajun Delights").First(), QuantityPerUnit = "32 - 8 oz bottles", UnitPrice = 21.05M, UnitsInStock = 76, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Louisiana Hot Spiced Okra", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "New Orleans Cajun Delights").First(), QuantityPerUnit = "24 - 8 oz jars", UnitPrice = 17.00M, UnitsInStock = 4, UnitsOnOrder = 100, ReorderLevel = 20, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Laughing Lumberjack Lager", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Bigfoot Breweries").First(), QuantityPerUnit = "24 - 12 oz bottles", UnitPrice = 14.00M, UnitsInStock = 52, UnitsOnOrder = 0, ReorderLevel = 10, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Scottish Longbreads", Category = categories.Where(c => c.Name == "Confections").First(), Supplier = suppliers.Where(s => s.CompanyName == "Specialty Biscuits, Ltd.").First(), QuantityPerUnit = "10 boxes x 8 pieces", UnitPrice = 12.50M, UnitsInStock = 6, UnitsOnOrder = 10, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Gudbrandsdalsost", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Norske Meierier").First(), QuantityPerUnit = "10 kg pkg.", UnitPrice = 36.00M, UnitsInStock = 26, UnitsOnOrder = 0, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Outback Lager", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Pavlova, Ltd.").First(), QuantityPerUnit = "24 - 355 ml bottles", UnitPrice = 15.00M, UnitsInStock = 15, UnitsOnOrder = 10, ReorderLevel = 30, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Flotemysost", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Norske Meierier").First(), QuantityPerUnit = "10 - 500 g pkgs.", UnitPrice = 21.50M, UnitsInStock = 26, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Mozzarella di Giovanni", Category = categories.Where(c => c.Name == "Dairy Products").First(), Supplier = suppliers.Where(s => s.CompanyName == "Formaggi Fortini s.r.l.").First(), QuantityPerUnit = "24 - 200 g pkgs.", UnitPrice = 34.80M, UnitsInStock = 14, UnitsOnOrder = 0, ReorderLevel = 0, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Rd Kaviar", Category = categories.Where(c => c.Name == "Seafood").First(), Supplier = suppliers.Where(s => s.CompanyName == "Svensk Sjfda AB").First(), QuantityPerUnit = "24 - 150 g jars", UnitPrice = 15.00M, UnitsInStock = 101, UnitsOnOrder = 0, ReorderLevel = 5, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Longlife Tofu", Category = categories.Where(c => c.Name == "Produce").First(), Supplier = suppliers.Where(s => s.CompanyName == "Tokyo Traders").First(), QuantityPerUnit = "5 kg pkg.", UnitPrice = 10.00M, UnitsInStock = 4, UnitsOnOrder = 20, ReorderLevel = 5, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Rhnbru Klosterbier", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Plutzer Lebensmittelgromrkte AG").First(), QuantityPerUnit = "24 - 0.5 l bottles", UnitPrice = 7.75M, UnitsInStock = 125, UnitsOnOrder = 0, ReorderLevel = 25, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Lakkalikri", Category = categories.Where(c => c.Name == "Beverages").First(), Supplier = suppliers.Where(s => s.CompanyName == "Karkki Oy").First(), QuantityPerUnit = "500 ml", UnitPrice = 18.00M, UnitsInStock = 57, UnitsOnOrder = 0, ReorderLevel = 20, Discontinued = false }; session.Insert(product); products.Add(product);
			product = new Product { ProductId = id++, Name = "Original Frankfurter grne Soe", Category = categories.Where(c => c.Name == "Condiments").First(), Supplier = suppliers.Where(s => s.CompanyName == "Plutzer Lebensmittelgromrkte AG").First(), QuantityPerUnit = "12 boxes", UnitPrice = 13.00M, UnitsInStock = 32, UnitsOnOrder = 0, ReorderLevel = 15, Discontinued = false }; session.Insert(product); products.Add(product);

			var orders = new List<Order>();
			var order = new Order { OrderId = 10248, Customer = customers.Where(c => c.CompanyName == "Vins et alcools Chevalier").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jul  4 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug  1 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 16 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 32.38M, ShippedTo = "Vins et alcools Chevalier", ShippingAddress = new Address("59 rue de l'Abbaye", "Reims", "null", "51100", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10249, Customer = customers.Where(c => c.CompanyName == "Toms Spezialitten").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jul  5 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 16 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 10 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 11.61M, ShippedTo = "Toms Spezialitten", ShippingAddress = new Address("Luisenstr. 48", "Mnster", "null", "44087", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10250, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul  8 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug  5 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 12 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 65.83M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10251, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jul  8 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug  5 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 15 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 41.34M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10252, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul  9 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug  6 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 11 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 51.30M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10253, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jul 10 1996 12:00AM"), RequiredDate = DateTime.Parse("Jul 24 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 16 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 58.17M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10254, Customer = customers.Where(c => c.CompanyName == "Chop-suey Chinese").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jul 11 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug  8 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 23 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 22.98M, ShippedTo = "Chop-suey Chinese", ShippingAddress = new Address("Hauptstr. 31", "Bern", "null", "3012", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10255, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jul 12 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug  9 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 15 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 148.33M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10256, Customer = customers.Where(c => c.CompanyName == "Wellington Importadora").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jul 15 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 12 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 17 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 13.97M, ShippedTo = "Wellington Importadora", ShippingAddress = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10257, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 16 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 13 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 22 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 81.91M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10258, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jul 17 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 14 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 23 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 140.51M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10259, Customer = customers.Where(c => c.CompanyName == "Centro comercial Moctezuma").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 18 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 15 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 25 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 3.25M, ShippedTo = "Centro comercial Moctezuma", ShippingAddress = new Address("Sierras de Granada 9993", "Mxico D.F.", "null", "05022", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10260, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 19 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 16 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 29 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 55.09M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10261, Customer = customers.Where(c => c.CompanyName == "Que Delcia").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 19 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 16 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 30 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.05M, ShippedTo = "Que Delcia", ShippingAddress = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10262, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jul 22 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 19 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 25 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 48.29M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10263, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jul 23 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 20 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 31 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 146.06M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10264, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jul 24 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 21 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 23 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 3.67M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10265, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jul 25 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 22 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 12 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 55.28M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10266, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jul 26 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep  6 1996 12:00AM"), ShippingDate = DateTime.Parse("Jul 31 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 25.73M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10267, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 29 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 26 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug  6 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 208.58M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10268, Customer = customers.Where(c => c.CompanyName == "GROSELLA-Restaurante").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jul 30 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 27 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug  2 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 66.29M, ShippedTo = "GROSELLA-Restaurante", ShippingAddress = new Address("5 Ave. Los Palos Grandes", "Caracas", "DF", "1081", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10269, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jul 31 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 14 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug  9 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 4.56M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10270, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Aug  1 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 29 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug  2 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 136.54M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10271, Customer = customers.Where(c => c.CompanyName == "Split Rail Beer & Ale").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Aug  1 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 29 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 30 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.54M, ShippedTo = "Split Rail Beer & Ale", ShippingAddress = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10272, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Aug  2 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 30 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug  6 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 98.03M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10273, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Aug  5 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep  2 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 12 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 76.07M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10274, Customer = customers.Where(c => c.CompanyName == "Vins et alcools Chevalier").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Aug  6 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep  3 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 16 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 6.01M, ShippedTo = "Vins et alcools Chevalier", ShippingAddress = new Address("59 rue de l'Abbaye", "Reims", "null", "51100", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10275, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Aug  7 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep  4 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug  9 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 26.93M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10276, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug  8 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 22 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 14 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 13.84M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10277, Customer = customers.Where(c => c.CompanyName == "Morgenstern Gesundkost").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Aug  9 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep  6 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 13 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 125.77M, ShippedTo = "Morgenstern Gesundkost", ShippingAddress = new Address("Heerstr. 22", "Leipzig", "null", "04179", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10278, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug 12 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep  9 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 16 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 92.69M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10279, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug 13 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 10 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 16 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 25.83M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10280, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Aug 14 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 11 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 12 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 8.98M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10281, Customer = customers.Where(c => c.CompanyName == "Romero y tomillo").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 14 1996 12:00AM"), RequiredDate = DateTime.Parse("Aug 28 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 21 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 2.94M, ShippedTo = "Romero y tomillo", ShippingAddress = new Address("Gran Va, 1", "Madrid", "null", "28001", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10282, Customer = customers.Where(c => c.CompanyName == "Romero y tomillo").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 15 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 12 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 21 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 12.69M, ShippedTo = "Romero y tomillo", ShippingAddress = new Address("Gran Va, 1", "Madrid", "null", "28001", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10283, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Aug 16 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 13 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 23 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 84.81M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10284, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 19 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 16 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 27 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 76.56M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10285, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Aug 20 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 17 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 26 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 76.83M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10286, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug 21 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 18 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 30 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 229.24M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10287, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug 22 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 19 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 28 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 12.76M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10288, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 23 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 20 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep  3 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 7.45M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10289, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Aug 26 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 23 1996 12:00AM"), ShippingDate = DateTime.Parse("Aug 28 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 22.77M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10290, Customer = customers.Where(c => c.CompanyName == "Comrcio Mineiro").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug 27 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 24 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep  3 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 79.70M, ShippedTo = "Comrcio Mineiro", ShippingAddress = new Address("Av. dos Lusadas, 23", "Sao Paulo", "SP", "05432-043", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10291, Customer = customers.Where(c => c.CompanyName == "Que Delcia").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Aug 27 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 24 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep  4 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 6.40M, ShippedTo = "Que Delcia", ShippingAddress = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10292, Customer = customers.Where(c => c.CompanyName == "Tradio Hipermercados").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Aug 28 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 25 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep  2 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.35M, ShippedTo = "Tradiao Hipermercados", ShippingAddress = new Address("Av. Ins de Castro, 414", "Sao Paulo", "SP", "05634-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10293, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Aug 29 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 26 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 11 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 21.18M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10294, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 30 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 27 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep  5 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 147.26M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10295, Customer = customers.Where(c => c.CompanyName == "Vins et alcools Chevalier").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep  2 1996 12:00AM"), RequiredDate = DateTime.Parse("Sep 30 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 10 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.15M, ShippedTo = "Vins et alcools Chevalier", ShippingAddress = new Address("59 rue de l'Abbaye", "Reims", "null", "51100", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10296, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Sep  3 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct  1 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 11 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.12M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10297, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Sep  4 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 16 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 10 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 5.74M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10298, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Sep  5 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct  3 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 11 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 168.22M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10299, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Sep  6 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct  4 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 13 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 29.76M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10300, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep  9 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct  7 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 18 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 17.68M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10301, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Sep  9 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct  7 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 17 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 45.08M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10302, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Sep 10 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct  8 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct  9 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 6.27M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10303, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Sep 11 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct  9 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 18 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 107.83M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10304, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 12 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 10 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 17 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 63.79M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10305, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Sep 13 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 11 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct  9 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 257.62M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10306, Customer = customers.Where(c => c.CompanyName == "Romero y tomillo").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 16 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 14 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 23 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 7.56M, ShippedTo = "Romero y tomillo", ShippingAddress = new Address("Gran Va, 1", "Madrid", "null", "28001", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10307, Customer = customers.Where(c => c.CompanyName == "Lonesome Pine Restaurant").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep 17 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 15 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 25 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 0.56M, ShippedTo = "Lonesome Pine Restaurant", ShippingAddress = new Address("89 Chiaroscuro Rd.", "Portland", "OR", "97219", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10308, Customer = customers.Where(c => c.CompanyName == "Ana Trujillo Emparedados y helados").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Sep 18 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 16 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 24 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.61M, ShippedTo = "Ana Trujillo Emparedados y helados", ShippingAddress = new Address("Avda. de la Constitucin 2222", "Mxico D.F.", "null", "05021", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10309, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Sep 19 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 17 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 23 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 47.30M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10310, Customer = customers.Where(c => c.CompanyName == "The Big Cheese").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Sep 20 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 18 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 27 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 17.52M, ShippedTo = "The Big Cheese", ShippingAddress = new Address("89 Jefferson Way Suite 2", "Portland", "OR", "97201", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10311, Customer = customers.Where(c => c.CompanyName == "Du monde entier").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 20 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct  4 1996 12:00AM"), ShippingDate = DateTime.Parse("Sep 26 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 24.69M, ShippedTo = "Du monde entier", ShippingAddress = new Address("67, rue des Cinquante Otages", "Nantes", "null", "44000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10312, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep 23 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 21 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct  3 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 40.26M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10313, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep 24 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 22 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct  4 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.96M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10314, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 25 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 23 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct  4 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 74.16M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10315, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Sep 26 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 24 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct  3 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 41.76M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10316, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 27 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 25 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct  8 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 150.15M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10317, Customer = customers.Where(c => c.CompanyName == "Lonesome Pine Restaurant").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Sep 30 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 28 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 10 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 12.69M, ShippedTo = "Lonesome Pine Restaurant", ShippingAddress = new Address("89 Chiaroscuro Rd.", "Portland", "OR", "97219", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10318, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Oct  1 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 29 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct  4 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.73M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10319, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Oct  2 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 30 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 11 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 64.50M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10320, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Oct  3 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 17 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 18 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 34.57M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10321, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct  3 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 31 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 11 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.43M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10322, Customer = customers.Where(c => c.CompanyName == "Pericles Comidas clsicas").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Oct  4 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov  1 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 23 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 0.40M, ShippedTo = "Pericles Comidas clsicas", ShippingAddress = new Address("Calle Dr. Jorge Cash 321", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10323, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct  7 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov  4 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 14 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 4.88M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10324, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Oct  8 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov  5 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 10 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 214.27M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10325, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Oct  9 1996 12:00AM"), RequiredDate = DateTime.Parse("Oct 23 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 14 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 64.86M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10326, Customer = customers.Where(c => c.CompanyName == "Blido Comidas preparadas").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 10 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov  7 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 14 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 77.92M, ShippedTo = "Blido Comidas preparadas", ShippingAddress = new Address("C/ Araquil, 67", "Madrid", "null", "28023", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10327, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Oct 11 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov  8 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 14 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 63.36M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10328, Customer = customers.Where(c => c.CompanyName == "Furia Bacalhau e Frutos do Mar").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 14 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 11 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 17 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 87.03M, ShippedTo = "Furia Bacalhau e Frutos do Mar", ShippingAddress = new Address("Jardim das rosas n. 32", "Lisboa", "null", "1675", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10329, Customer = customers.Where(c => c.CompanyName == "Split Rail Beer & Ale").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 15 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 26 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 23 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 191.67M, ShippedTo = "Split Rail Beer & Ale", ShippingAddress = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10330, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct 16 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 13 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 28 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 12.75M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10331, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Oct 16 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 27 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 21 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 10.19M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10332, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct 17 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 28 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 21 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 52.84M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10333, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Oct 18 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 15 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 25 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 0.59M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10334, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Oct 21 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 18 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 28 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 8.56M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10335, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Oct 22 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 19 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 24 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 42.11M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10336, Customer = customers.Where(c => c.CompanyName == "Princesa Isabel Vinhos").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Oct 23 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 20 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 25 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 15.51M, ShippedTo = "Princesa Isabel Vinhos", ShippingAddress = new Address("Estrada da sade n. 58", "Lisboa", "null", "1756", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10337, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 24 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 21 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 29 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 108.26M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10338, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 25 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 22 1996 12:00AM"), ShippingDate = DateTime.Parse("Oct 29 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 84.21M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10339, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Oct 28 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 25 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov  4 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 15.66M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10340, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Oct 29 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 26 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov  8 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 166.31M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10341, Customer = customers.Where(c => c.CompanyName == "Simons bistro").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Oct 29 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 26 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov  5 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 26.78M, ShippedTo = "Simons bistro", ShippingAddress = new Address("Vinbltet 34", "Kobenhavn", "null", "1734", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10342, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 30 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 13 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov  4 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 54.83M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10343, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 31 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 28 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov  6 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 110.37M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10344, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov  1 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 29 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov  5 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 23.29M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10345, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Nov  4 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec  2 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 11 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 249.06M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10346, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov  5 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 17 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov  8 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 142.08M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10347, Customer = customers.Where(c => c.CompanyName == "Familia Arquibaldo").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov  6 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec  4 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov  8 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 3.10M, ShippedTo = "Familia Arquibaldo", ShippingAddress = new Address("Rua Ors, 92", "Sao Paulo", "SP", "05442-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10348, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov  7 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec  5 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 15 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 0.78M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10349, Customer = customers.Where(c => c.CompanyName == "Split Rail Beer & Ale").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Nov  8 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec  6 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 15 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 8.63M, ShippedTo = "Split Rail Beer & Ale", ShippingAddress = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10350, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Nov 11 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec  9 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec  3 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 64.19M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10351, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Nov 11 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec  9 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 20 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 162.33M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10352, Customer = customers.Where(c => c.CompanyName == "Furia Bacalhau e Frutos do Mar").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 12 1996 12:00AM"), RequiredDate = DateTime.Parse("Nov 26 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 18 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.30M, ShippedTo = "Furia Bacalhau e Frutos do Mar", ShippingAddress = new Address("Jardim das rosas n. 32", "Lisboa", "null", "1675", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10353, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Nov 13 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 11 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 25 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 360.63M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10354, Customer = customers.Where(c => c.CompanyName == "Pericles Comidas clsicas").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Nov 14 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 12 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 20 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 53.80M, ShippedTo = "Pericles Comidas clsicas", ShippingAddress = new Address("Calle Dr. Jorge Cash 321", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10355, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Nov 15 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 13 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 20 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 41.95M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10356, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Nov 18 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 16 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 27 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 36.71M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10357, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Nov 19 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 17 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec  2 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 34.88M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10358, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Nov 20 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 18 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 27 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 19.64M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10359, Customer = customers.Where(c => c.CompanyName == "Seven Seas Imports").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Nov 21 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 19 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 26 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 288.43M, ShippedTo = "Seven Seas Imports", ShippingAddress = new Address("90 Wadhurst Rd.", "London", "null", "OX15 4NB", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10360, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov 22 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 20 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec  2 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 131.70M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10361, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Nov 22 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 20 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec  3 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 183.17M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10362, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 25 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 23 1996 12:00AM"), ShippingDate = DateTime.Parse("Nov 28 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 96.04M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10363, Customer = customers.Where(c => c.CompanyName == "Drachenblut Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov 26 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 24 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec  4 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 30.54M, ShippedTo = "Drachenblut Delikatessen", ShippingAddress = new Address("Walserweg 21", "Aachen", "null", "52066", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10364, Customer = customers.Where(c => c.CompanyName == "Eastern Connection").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Nov 26 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  4 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 71.97M, ShippedTo = "Eastern Connection", ShippingAddress = new Address("35 King George", "London", "null", "WX3 6FW", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10365, Customer = customers.Where(c => c.CompanyName == "Antonio Moreno Taquera").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 27 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 25 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec  2 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 22.00M, ShippedTo = "Antonio Moreno Taquera", ShippingAddress = new Address("Mataderos  2312", "Mxico D.F.", "null", "05023", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10366, Customer = customers.Where(c => c.CompanyName == "Galera del gastrnomo").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Nov 28 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 30 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 10.14M, ShippedTo = "Galera del gastronmo", ShippingAddress = new Address("Rambla de Catalua, 23", "Barcelona", "null", "8022", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10367, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Nov 28 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 26 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec  2 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 13.55M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10368, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Nov 29 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 27 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec  2 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 101.95M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10369, Customer = customers.Where(c => c.CompanyName == "Split Rail Beer & Ale").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Dec  2 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 30 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec  9 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 195.68M, ShippedTo = "Split Rail Beer & Ale", ShippingAddress = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10370, Customer = customers.Where(c => c.CompanyName == "Chop-suey Chinese").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Dec  3 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 31 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec 27 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.17M, ShippedTo = "Chop-suey Chinese", ShippingAddress = new Address("Hauptstr. 31", "Bern", "null", "3012", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10371, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec  3 1996 12:00AM"), RequiredDate = DateTime.Parse("Dec 31 1996 12:00AM"), ShippingDate = DateTime.Parse("Dec 24 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.45M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10372, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Dec  4 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  9 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 890.78M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10373, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec  5 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 11 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 124.12M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10374, Customer = customers.Where(c => c.CompanyName == "Wolski  Zajazd").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec  5 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  9 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 3.94M, ShippedTo = "Wolski Zajazd", ShippingAddress = new Address("ul. Filtrowa 68", "Warszawa", "null", "01-012", "Poland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10375, Customer = customers.Where(c => c.CompanyName == "Hungry Coyote Import Store").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec  6 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  9 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 20.12M, ShippedTo = "Hungry Coyote Import Store", ShippingAddress = new Address("City Center Plaza 516 Main St.", "Elgin", "OR", "97827", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10376, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec  9 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 13 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 20.39M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10377, Customer = customers.Where(c => c.CompanyName == "Seven Seas Imports").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec  9 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 13 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 22.21M, ShippedTo = "Seven Seas Imports", ShippingAddress = new Address("90 Wadhurst Rd.", "London", "null", "OX15 4NB", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10378, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Dec 10 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 19 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 5.44M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10379, Customer = customers.Where(c => c.CompanyName == "Que Delcia").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Dec 11 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 13 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 45.03M, ShippedTo = "Que Delcia", ShippingAddress = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10380, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Dec 12 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 35.03M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10381, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec 12 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 13 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 7.99M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10382, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec 13 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 16 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 94.77M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10383, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Dec 16 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 18 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 34.24M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10384, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec 16 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 20 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 168.64M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10385, Customer = customers.Where(c => c.CompanyName == "Split Rail Beer & Ale").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 17 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 23 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 30.96M, ShippedTo = "Split Rail Beer & Ale", ShippingAddress = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10386, Customer = customers.Where(c => c.CompanyName == "Familia Arquibaldo").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Dec 18 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 25 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 13.99M, ShippedTo = "Familia Arquibaldo", ShippingAddress = new Address("Rua Ors, 92", "Sao Paulo", "SP", "05442-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10387, Customer = customers.Where(c => c.CompanyName == "Sant Gourmet").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 18 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 20 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 93.63M, ShippedTo = "Sant Gourmet", ShippingAddress = new Address("Erling Skakkes gate 78", "Stavern", "null", "4110", "Norway", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10388, Customer = customers.Where(c => c.CompanyName == "Seven Seas Imports").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Dec 19 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 20 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 34.86M, ShippedTo = "Seven Seas Imports", ShippingAddress = new Address("90 Wadhurst Rd.", "London", "null", "OX15 4NB", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10389, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec 20 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 24 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 47.42M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10390, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Dec 23 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 20 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 26 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 126.38M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10391, Customer = customers.Where(c => c.CompanyName == "Drachenblut Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec 23 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 20 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 31 1996 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 5.45M, ShippedTo = "Drachenblut Delikatessen", ShippingAddress = new Address("Walserweg 21", "Aachen", "null", "52066", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10392, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Dec 24 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 122.46M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10393, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 25 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 126.56M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10394, Customer = customers.Where(c => c.CompanyName == "Hungry Coyote Import Store").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 25 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 30.34M, ShippedTo = "Hungry Coyote Import Store", ShippingAddress = new Address("City Center Plaza 516 Main St.", "Elgin", "OR", "97827", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10395, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Dec 26 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 184.41M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10396, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 27 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 135.35M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10397, Customer = customers.Where(c => c.CompanyName == "Princesa Isabel Vinhos").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Dec 27 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 60.26M, ShippedTo = "Princesa Isabel Vinhos", ShippingAddress = new Address("Estrada da sade n. 58", "Lisboa", "null", "1756", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10398, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Dec 30 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 27 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 89.16M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10399, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Dec 31 1996 12:00AM"), RequiredDate = DateTime.Parse("Jan 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 27.36M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10400, Customer = customers.Where(c => c.CompanyName == "Eastern Connection").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 83.93M, ShippedTo = "Eastern Connection", ShippingAddress = new Address("35 King George", "London", "null", "WX3 6FW", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10401, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 12.51M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10402, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 67.88M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10403, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 31 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 73.79M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10404, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 31 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 155.97M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10405, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 34.82M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10406, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jan  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 108.04M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10407, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 91.48M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10408, Customer = customers.Where(c => c.CompanyName == "Folies gourmandes").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 11.26M, ShippedTo = "Folies gourmandes", ShippingAddress = new Address("184, chausse de Tournai", "Lille", "null", "59000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10409, Customer = customers.Where(c => c.CompanyName == "Ocano Atlntico Ltda.").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 29.83M, ShippedTo = "Ocano Atlntico Ltda.", ShippingAddress = new Address("Ing. Gustavo Moncada 8585 Piso 20-A", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10410, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 2.40M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10411, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jan 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 23.65M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10412, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.77M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10413, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 95.66M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10414, Customer = customers.Where(c => c.CompanyName == "Familia Arquibaldo").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 17 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 21.48M, ShippedTo = "Familia Arquibaldo", ShippingAddress = new Address("Rua Ors, 92", "Sao Paulo", "SP", "05442-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10415, Customer = customers.Where(c => c.CompanyName == "Hungry Coyote Import Store").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.20M, ShippedTo = "Hungry Coyote Import Store", ShippingAddress = new Address("City Center Plaza 516 Main St.", "Elgin", "OR", "97827", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10416, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 22.72M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10417, Customer = customers.Where(c => c.CompanyName == "Simons bistro").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 70.29M, ShippedTo = "Simons bistro", ShippingAddress = new Address("Vinbltet 34", "Kobenhavn", "null", "1734", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10418, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 17.55M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10419, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 137.35M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10420, Customer = customers.Where(c => c.CompanyName == "Wellington Importadora").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 44.12M, ShippedTo = "Wellington Importadora", ShippingAddress = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10421, Customer = customers.Where(c => c.CompanyName == "Que Delcia").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 99.23M, ShippedTo = "Que Delcia", ShippingAddress = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10422, Customer = customers.Where(c => c.CompanyName == "Franchi S.p.A.").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 31 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 3.02M, ShippedTo = "Franchi S.p.A.", ShippingAddress = new Address("Via Monte Bianco 34", "Torino", "null", "10100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10423, Customer = customers.Where(c => c.CompanyName == "Gourmet Lanchonetes").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jan 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 24.50M, ShippedTo = "Gourmet Lanchonetes", ShippingAddress = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10424, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jan 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 20 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 370.61M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10425, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jan 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 7.93M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10426, Customer = customers.Where(c => c.CompanyName == "Galera del gastrnomo").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 18.69M, ShippedTo = "Galera del gastronmo", ShippingAddress = new Address("Rambla de Catalua, 23", "Barcelona", "null", "8022", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10427, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 31.29M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10428, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jan 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 11.09M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10429, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 56.63M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10430, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 458.78M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10431, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 44.17M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10432, Customer = customers.Where(c => c.CompanyName == "Split Rail Beer & Ale").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 31 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.34M, ShippedTo = "Split Rail Beer & Ale", ShippingAddress = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10433, Customer = customers.Where(c => c.CompanyName == "Princesa Isabel Vinhos").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 73.83M, ShippedTo = "Princesa Isabel Vinhos", ShippingAddress = new Address("Estrada da sade n. 58", "Lisboa", "null", "1756", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10434, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 17.92M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10435, Customer = customers.Where(c => c.CompanyName == "Consolidated Holdings").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 9.21M, ShippedTo = "Consolidated Holdings", ShippingAddress = new Address("Berkeley Gardens 12  Brewery", "London", "null", "WX1 6LT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10436, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 156.66M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10437, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 19.97M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10438, Customer = customers.Where(c => c.CompanyName == "Toms Spezialitten").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 8.24M, ShippedTo = "Toms Spezialitten", ShippingAddress = new Address("Luisenstr. 48", "Mnster", "null", "44087", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10439, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Feb  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.07M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10440, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 86.53M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10441, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 73.02M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10442, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 47.94M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10443, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 13.95M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10444, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 3.50M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10445, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 20 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 9.30M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10446, Customer = customers.Where(c => c.CompanyName == "Toms Spezialitten").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Feb 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 14.68M, ShippedTo = "Toms Spezialitten", ShippingAddress = new Address("Luisenstr. 48", "Mnster", "null", "44087", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10447, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 68.66M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10448, Customer = customers.Where(c => c.CompanyName == "Rancho grande").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 38.82M, ShippedTo = "Rancho grande", ShippingAddress = new Address("Av. del Libertador 900", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10449, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 53.30M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10450, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 7.23M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10451, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 189.09M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10452, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 20 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 140.26M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10453, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 25.36M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10454, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 2.74M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10455, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 180.45M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10456, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 8.12M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10457, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Feb 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 11.57M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10458, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Feb 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 26 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 147.06M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10459, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 27 1997 12:00AM"), ShippingDate = DateTime.Parse("Feb 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 25.09M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10460, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 16.27M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10461, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 148.61M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10462, Customer = customers.Where(c => c.CompanyName == "Consolidated Holdings").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 31 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 6.17M, ShippedTo = "Consolidated Holdings", ShippingAddress = new Address("Berkeley Gardens 12  Brewery", "London", "null", "WX1 6LT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10463, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Mar  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 14.78M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10464, Customer = customers.Where(c => c.CompanyName == "Furia Bacalhau e Frutos do Mar").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 89.00M, ShippedTo = "Furia Bacalhau e Frutos do Mar", ShippingAddress = new Address("Jardim das rosas n. 32", "Lisboa", "null", "1675", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10465, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 145.04M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10466, Customer = customers.Where(c => c.CompanyName == "Comrcio Mineiro").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 11.93M, ShippedTo = "Comrcio Mineiro", ShippingAddress = new Address("Av. dos Lusadas, 23", "Sao Paulo", "SP", "05432-043", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10467, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.93M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10468, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 44.12M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10469, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 60.18M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10470, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 64.56M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10471, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 45.59M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10472, Customer = customers.Where(c => c.CompanyName == "Seven Seas Imports").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 4.20M, ShippedTo = "Seven Seas Imports", ShippingAddress = new Address("90 Wadhurst Rd.", "London", "null", "OX15 4NB", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10473, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Mar 27 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 16.37M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10474, Customer = customers.Where(c => c.CompanyName == "Pericles Comidas clsicas").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Mar 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 83.49M, ShippedTo = "Pericles Comidas clsicas", ShippingAddress = new Address("Calle Dr. Jorge Cash 321", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10475, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Mar 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 68.52M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10476, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.41M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10477, Customer = customers.Where(c => c.CompanyName == "Princesa Isabel Vinhos").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Mar 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 13.02M, ShippedTo = "Princesa Isabel Vinhos", ShippingAddress = new Address("Estrada da sade n. 58", "Lisboa", "null", "1756", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10478, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.81M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10479, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 708.95M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10480, Customer = customers.Where(c => c.CompanyName == "Folies gourmandes").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Mar 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.35M, ShippedTo = "Folies gourmandes", ShippingAddress = new Address("184, chausse de Tournai", "Lille", "null", "59000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10481, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 64.33M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10482, Customer = customers.Where(c => c.CompanyName == "Lazy K Kountry Store").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 7.48M, ShippedTo = "Lazy K Kountry Store", ShippingAddress = new Address("12 Orchestra Terrace", "Walla Walla", "WA", "99362", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10483, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Mar 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 15.28M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10484, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 6.88M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10485, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 31 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 64.45M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10486, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 30.53M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10487, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Mar 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 71.07M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10488, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.93M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10489, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Mar 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 5.29M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10490, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Mar 31 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 210.19M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10491, Customer = customers.Where(c => c.CompanyName == "Furia Bacalhau e Frutos do Mar").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 31 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 16.96M, ShippedTo = "Furia Bacalhau e Frutos do Mar", ShippingAddress = new Address("Jardim das rosas n. 32", "Lisboa", "null", "1675", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10492, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 62.89M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10493, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 10.64M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10494, Customer = customers.Where(c => c.CompanyName == "Comrcio Mineiro").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Apr 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 65.99M, ShippedTo = "Comrcio Mineiro", ShippingAddress = new Address("Av. dos Lusadas, 23", "Sao Paulo", "SP", "05432-043", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10495, Customer = customers.Where(c => c.CompanyName == "Laughing Bacchus Wine Cellars").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr  3 1997 12:00AM"), RequiredDate = DateTime.Parse("May  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.65M, ShippedTo = "Laughing Bacchus Wine Cellars", ShippingAddress = new Address("2319 Elm St.", "Vancouver", "BC", "V3F 2K1", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10496, Customer = customers.Where(c => c.CompanyName == "Tradio Hipermercados").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr  4 1997 12:00AM"), RequiredDate = DateTime.Parse("May  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 46.77M, ShippedTo = "Tradiao Hipermercados", ShippingAddress = new Address("Av. Ins de Castro, 414", "Sao Paulo", "SP", "05634-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10497, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr  4 1997 12:00AM"), RequiredDate = DateTime.Parse("May  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 36.21M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10498, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr  7 1997 12:00AM"), RequiredDate = DateTime.Parse("May  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 29.75M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10499, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr  8 1997 12:00AM"), RequiredDate = DateTime.Parse("May  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 102.02M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10500, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Apr  9 1997 12:00AM"), RequiredDate = DateTime.Parse("May  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 17 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 42.68M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10501, Customer = customers.Where(c => c.CompanyName == "Blauer See Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Apr  9 1997 12:00AM"), RequiredDate = DateTime.Parse("May  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 8.85M, ShippedTo = "Blauer See Delikatessen", ShippingAddress = new Address("Forsterstr. 57", "Mannheim", "null", "68306", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10502, Customer = customers.Where(c => c.CompanyName == "Pericles Comidas clsicas").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 10 1997 12:00AM"), RequiredDate = DateTime.Parse("May  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 69.32M, ShippedTo = "Pericles Comidas clsicas", ShippingAddress = new Address("Calle Dr. Jorge Cash 321", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10503, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Apr 11 1997 12:00AM"), RequiredDate = DateTime.Parse("May  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 16.74M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10504, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 11 1997 12:00AM"), RequiredDate = DateTime.Parse("May  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 59.13M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10505, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr 14 1997 12:00AM"), RequiredDate = DateTime.Parse("May 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 7.13M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10506, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Apr 15 1997 12:00AM"), RequiredDate = DateTime.Parse("May 13 1997 12:00AM"), ShippingDate = DateTime.Parse("May  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 21.19M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10507, Customer = customers.Where(c => c.CompanyName == "Antonio Moreno Taquera").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 15 1997 12:00AM"), RequiredDate = DateTime.Parse("May 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 47.45M, ShippedTo = "Antonio Moreno Taquera", ShippingAddress = new Address("Mataderos  2312", "Mxico D.F.", "null", "05023", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10508, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Apr 16 1997 12:00AM"), RequiredDate = DateTime.Parse("May 14 1997 12:00AM"), ShippingDate = DateTime.Parse("May 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.99M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10509, Customer = customers.Where(c => c.CompanyName == "Blauer See Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 17 1997 12:00AM"), RequiredDate = DateTime.Parse("May 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.15M, ShippedTo = "Blauer See Delikatessen", ShippingAddress = new Address("Forsterstr. 57", "Mannheim", "null", "68306", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10510, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Apr 18 1997 12:00AM"), RequiredDate = DateTime.Parse("May 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 367.63M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10511, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 18 1997 12:00AM"), RequiredDate = DateTime.Parse("May 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 350.64M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10512, Customer = customers.Where(c => c.CompanyName == "Familia Arquibaldo").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 21 1997 12:00AM"), RequiredDate = DateTime.Parse("May 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.53M, ShippedTo = "Familia Arquibaldo", ShippingAddress = new Address("Rua Ors, 92", "Sao Paulo", "SP", "05442-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10513, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 105.65M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10514, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr 22 1997 12:00AM"), RequiredDate = DateTime.Parse("May 20 1997 12:00AM"), ShippingDate = DateTime.Parse("May 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 789.95M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10515, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 23 1997 12:00AM"), RequiredDate = DateTime.Parse("May  7 1997 12:00AM"), ShippingDate = DateTime.Parse("May 23 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 204.47M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10516, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 24 1997 12:00AM"), RequiredDate = DateTime.Parse("May 22 1997 12:00AM"), ShippingDate = DateTime.Parse("May  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 62.78M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10517, Customer = customers.Where(c => c.CompanyName == "North/South").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr 24 1997 12:00AM"), RequiredDate = DateTime.Parse("May 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Apr 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 32.07M, ShippedTo = "North/South", ShippingAddress = new Address("South House 300 Queensbridge", "London", "null", "SW7 1RZ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10518, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 25 1997 12:00AM"), RequiredDate = DateTime.Parse("May  9 1997 12:00AM"), ShippingDate = DateTime.Parse("May  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 218.15M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10519, Customer = customers.Where(c => c.CompanyName == "Chop-suey Chinese").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Apr 28 1997 12:00AM"), RequiredDate = DateTime.Parse("May 26 1997 12:00AM"), ShippingDate = DateTime.Parse("May  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 91.76M, ShippedTo = "Chop-suey Chinese", ShippingAddress = new Address("Hauptstr. 31", "Bern", "null", "3012", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10520, Customer = customers.Where(c => c.CompanyName == "Sant Gourmet").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 29 1997 12:00AM"), RequiredDate = DateTime.Parse("May 27 1997 12:00AM"), ShippingDate = DateTime.Parse("May  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 13.37M, ShippedTo = "Sant Gourmet", ShippingAddress = new Address("Erling Skakkes gate 78", "Stavern", "null", "4110", "Norway", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10521, Customer = customers.Where(c => c.CompanyName == "Cactus Comidas para llevar").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr 29 1997 12:00AM"), RequiredDate = DateTime.Parse("May 27 1997 12:00AM"), ShippingDate = DateTime.Parse("May  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 17.22M, ShippedTo = "Cactus Comidas para llevar", ShippingAddress = new Address("Cerrito 333", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10522, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 30 1997 12:00AM"), RequiredDate = DateTime.Parse("May 28 1997 12:00AM"), ShippingDate = DateTime.Parse("May  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 45.33M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10523, Customer = customers.Where(c => c.CompanyName == "Seven Seas Imports").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("May  1 1997 12:00AM"), RequiredDate = DateTime.Parse("May 29 1997 12:00AM"), ShippingDate = DateTime.Parse("May 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 77.63M, ShippedTo = "Seven Seas Imports", ShippingAddress = new Address("90 Wadhurst Rd.", "London", "null", "OX15 4NB", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10524, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May  1 1997 12:00AM"), RequiredDate = DateTime.Parse("May 29 1997 12:00AM"), ShippingDate = DateTime.Parse("May  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 244.79M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10525, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May  2 1997 12:00AM"), RequiredDate = DateTime.Parse("May 30 1997 12:00AM"), ShippingDate = DateTime.Parse("May 23 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 11.06M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10526, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("May  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun  2 1997 12:00AM"), ShippingDate = DateTime.Parse("May 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 58.59M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10527, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("May  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun  2 1997 12:00AM"), ShippingDate = DateTime.Parse("May  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 41.90M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10528, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("May  6 1997 12:00AM"), RequiredDate = DateTime.Parse("May 20 1997 12:00AM"), ShippingDate = DateTime.Parse("May  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.35M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10529, Customer = customers.Where(c => c.CompanyName == "Maison Dewey").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("May  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun  4 1997 12:00AM"), ShippingDate = DateTime.Parse("May  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 66.69M, ShippedTo = "Maison Dewey", ShippingAddress = new Address("Rue Joseph-Bens 532", "Bruxelles", "null", "B-1180", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10530, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("May  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun  5 1997 12:00AM"), ShippingDate = DateTime.Parse("May 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 339.22M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10531, Customer = customers.Where(c => c.CompanyName == "Ocano Atlntico Ltda.").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("May  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun  5 1997 12:00AM"), ShippingDate = DateTime.Parse("May 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 8.12M, ShippedTo = "Ocano Atlntico Ltda.", ShippingAddress = new Address("Ing. Gustavo Moncada 8585 Piso 20-A", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10532, Customer = customers.Where(c => c.CompanyName == "Eastern Connection").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("May  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun  6 1997 12:00AM"), ShippingDate = DateTime.Parse("May 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 74.46M, ShippedTo = "Eastern Connection", ShippingAddress = new Address("35 King George", "London", "null", "WX3 6FW", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10533, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("May 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun  9 1997 12:00AM"), ShippingDate = DateTime.Parse("May 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 188.04M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10534, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("May 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun  9 1997 12:00AM"), ShippingDate = DateTime.Parse("May 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 27.94M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10535, Customer = customers.Where(c => c.CompanyName == "Antonio Moreno Taquera").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("May 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 10 1997 12:00AM"), ShippingDate = DateTime.Parse("May 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 15.64M, ShippedTo = "Antonio Moreno Taquera", ShippingAddress = new Address("Mataderos  2312", "Mxico D.F.", "null", "05023", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10536, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("May 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 58.88M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10537, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May 14 1997 12:00AM"), RequiredDate = DateTime.Parse("May 28 1997 12:00AM"), ShippingDate = DateTime.Parse("May 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 78.85M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10538, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("May 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 12 1997 12:00AM"), ShippingDate = DateTime.Parse("May 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.87M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10539, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("May 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 13 1997 12:00AM"), ShippingDate = DateTime.Parse("May 23 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 12.36M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10540, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("May 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1007.64M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10541, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("May 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 16 1997 12:00AM"), ShippingDate = DateTime.Parse("May 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 68.65M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10542, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 17 1997 12:00AM"), ShippingDate = DateTime.Parse("May 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 10.95M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10543, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("May 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 18 1997 12:00AM"), ShippingDate = DateTime.Parse("May 23 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 48.17M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10544, Customer = customers.Where(c => c.CompanyName == "Lonesome Pine Restaurant").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("May 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 18 1997 12:00AM"), ShippingDate = DateTime.Parse("May 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 24.91M, ShippedTo = "Lonesome Pine Restaurant", ShippingAddress = new Address("89 Chiaroscuro Rd.", "Portland", "OR", "97219", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10545, Customer = customers.Where(c => c.CompanyName == "Lazy K Kountry Store").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("May 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 11.92M, ShippedTo = "Lazy K Kountry Store", ShippingAddress = new Address("12 Orchestra Terrace", "Walla Walla", "WA", "99362", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10546, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 20 1997 12:00AM"), ShippingDate = DateTime.Parse("May 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 194.72M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10547, Customer = customers.Where(c => c.CompanyName == "Seven Seas Imports").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("May 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 20 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 178.43M, ShippedTo = "Seven Seas Imports", ShippingAddress = new Address("90 Wadhurst Rd.", "London", "null", "OX15 4NB", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10548, Customer = customers.Where(c => c.CompanyName == "Toms Spezialitten").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("May 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.43M, ShippedTo = "Toms Spezialitten", ShippingAddress = new Address("Luisenstr. 48", "Mnster", "null", "44087", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10549, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("May 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 10 1997 12:00AM"), ShippingDate = DateTime.Parse("May 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 171.24M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10550, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("May 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.32M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10551, Customer = customers.Where(c => c.CompanyName == "Furia Bacalhau e Frutos do Mar").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("May 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 72.95M, ShippedTo = "Furia Bacalhau e Frutos do Mar", ShippingAddress = new Address("Jardim das rosas n. 32", "Lisboa", "null", "1675", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10552, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("May 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 26 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 83.22M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10553, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("May 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 27 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 149.49M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10554, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("May 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 27 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 120.97M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10555, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jun  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 252.49M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10556, Customer = customers.Where(c => c.CompanyName == "Simons bistro").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jun  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 9.80M, ShippedTo = "Simons bistro", ShippingAddress = new Address("Vinbltet 34", "Kobenhavn", "null", "1734", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10557, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jun  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Jun 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 96.72M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10558, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jun  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 72.97M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10559, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jun  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 8.05M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10560, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jun  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 36.65M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10561, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jun  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 242.21M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10562, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jun  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 22.95M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10563, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jun 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 60.43M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10564, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jun 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 13.75M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10565, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jun 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 7.15M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10566, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jun 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 88.40M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10567, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jun 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 17 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 33.97M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10568, Customer = customers.Where(c => c.CompanyName == "Galera del gastrnomo").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jun 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 6.54M, ShippedTo = "Galera del gastronmo", ShippingAddress = new Address("Rambla de Catalua, 23", "Barcelona", "null", "8022", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10569, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jun 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 58.98M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10570, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jun 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 188.99M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10571, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jun 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 26.06M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10572, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jun 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 116.43M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10573, Customer = customers.Where(c => c.CompanyName == "Antonio Moreno Taquera").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jun 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 20 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 84.84M, ShippedTo = "Antonio Moreno Taquera", ShippingAddress = new Address("Mataderos  2312", "Mxico D.F.", "null", "05023", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10574, Customer = customers.Where(c => c.CompanyName == "Trail's Head Gourmet Provisioners").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jun 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 37.60M, ShippedTo = "Trail's Head Gourmet Provisioners", ShippingAddress = new Address("722 DaVinci Blvd.", "Kirkland", "WA", "98034", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10575, Customer = customers.Where(c => c.CompanyName == "Morgenstern Gesundkost").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jun 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 127.34M, ShippedTo = "Morgenstern Gesundkost", ShippingAddress = new Address("Heerstr. 22", "Leipzig", "null", "04179", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10576, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jun 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 18.56M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10577, Customer = customers.Where(c => c.CompanyName == "Trail's Head Gourmet Provisioners").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jun 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Jun 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 25.41M, ShippedTo = "Trail's Head Gourmet Provisioners", ShippingAddress = new Address("722 DaVinci Blvd.", "Kirkland", "WA", "98034", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10578, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jun 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 29.60M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10579, Customer = customers.Where(c => c.CompanyName == "Let's Stop N Shop").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jun 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 13.73M, ShippedTo = "Let's Stop N Shop", ShippingAddress = new Address("87 Polk St. Suite 5", "San Francisco", "CA", "94117", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10580, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jun 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 75.89M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10581, Customer = customers.Where(c => c.CompanyName == "Familia Arquibaldo").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jun 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 3.01M, ShippedTo = "Familia Arquibaldo", ShippingAddress = new Address("Rua Ors, 92", "Sao Paulo", "SP", "05442-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10582, Customer = customers.Where(c => c.CompanyName == "Blauer See Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jun 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 27.71M, ShippedTo = "Blauer See Delikatessen", ShippingAddress = new Address("Forsterstr. 57", "Mannheim", "null", "68306", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10583, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jun 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 7.28M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10584, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jun 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 59.14M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10585, Customer = customers.Where(c => c.CompanyName == "Wellington Importadora").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jul  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 13.41M, ShippedTo = "Wellington Importadora", ShippingAddress = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10586, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jul  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.48M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10587, Customer = customers.Where(c => c.CompanyName == "Que Delcia").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jul  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 62.52M, ShippedTo = "Que Delcia", ShippingAddress = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10588, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jul  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 31 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 194.67M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10589, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jul  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.42M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10590, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 44.77M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10591, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jul  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Jul 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 55.92M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10592, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jul  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 32.10M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10593, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jul  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 174.20M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10594, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jul  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 5.24M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10595, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jul 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 96.78M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10596, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jul 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 16.34M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10597, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jul 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 35.12M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10598, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jul 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 44.42M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10599, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jul 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 26 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 29.98M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10600, Customer = customers.Where(c => c.CompanyName == "Hungry Coyote Import Store").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 45.13M, ShippedTo = "Hungry Coyote Import Store", ShippingAddress = new Address("City Center Plaza 516 Main St.", "Elgin", "OR", "97827", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10601, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jul 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 27 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 58.30M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10602, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jul 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 2.92M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10603, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jul 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 48.77M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10604, Customer = customers.Where(c => c.CompanyName == "Furia Bacalhau e Frutos do Mar").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jul 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 7.46M, ShippedTo = "Furia Bacalhau e Frutos do Mar", ShippingAddress = new Address("Jardim das rosas n. 32", "Lisboa", "null", "1675", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10605, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jul 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 379.13M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10606, Customer = customers.Where(c => c.CompanyName == "Tradio Hipermercados").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 31 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 79.40M, ShippedTo = "Tradiao Hipermercados", ShippingAddress = new Address("Av. Ins de Castro, 414", "Sao Paulo", "SP", "05634-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10607, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jul 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 200.24M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10608, Customer = customers.Where(c => c.CompanyName == "Toms Spezialitten").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 20 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 27.79M, ShippedTo = "Toms Spezialitten", ShippingAddress = new Address("Luisenstr. 48", "Mnster", "null", "44087", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10609, Customer = customers.Where(c => c.CompanyName == "Du monde entier").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jul 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Jul 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.85M, ShippedTo = "Du monde entier", ShippingAddress = new Address("67, rue des Cinquante Otages", "Nantes", "null", "44000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10610, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jul 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 26.78M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10611, Customer = customers.Where(c => c.CompanyName == "Wolski  Zajazd").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jul 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 80.65M, ShippedTo = "Wolski Zajazd", ShippingAddress = new Address("ul. Filtrowa 68", "Warszawa", "null", "01-012", "Poland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10612, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jul 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 544.08M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10613, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 26 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 8.11M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10614, Customer = customers.Where(c => c.CompanyName == "Blauer See Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jul 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 26 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.93M, ShippedTo = "Blauer See Delikatessen", ShippingAddress = new Address("Forsterstr. 57", "Mannheim", "null", "68306", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10615, Customer = customers.Where(c => c.CompanyName == "Wilman Kala").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jul 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 27 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  6 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 0.75M, ShippedTo = "Wilman Kala", ShippingAddress = new Address("Keskuskatu 45", "Helsinki", "null", "21240", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10616, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jul 31 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 116.53M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10617, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jul 31 1997 12:00AM"), RequiredDate = DateTime.Parse("Aug 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 18.53M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10618, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Aug  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 154.68M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10619, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Aug  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 91.05M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10620, Customer = customers.Where(c => c.CompanyName == "Laughing Bacchus Wine Cellars").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Aug  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 0.94M, ShippedTo = "Laughing Bacchus Wine Cellars", ShippingAddress = new Address("2319 Elm St.", "Vancouver", "BC", "V3F 2K1", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10621, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 23.73M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10622, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 50.97M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10623, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 97.18M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10624, Customer = customers.Where(c => c.CompanyName == "The Cracker Box").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 94.80M, ShippedTo = "The Cracker Box", ShippingAddress = new Address("55 Grizzly Peak Rd.", "Butte", "MT", "59801", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10625, Customer = customers.Where(c => c.CompanyName == "Ana Trujillo Emparedados y helados").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Aug  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 43.90M, ShippedTo = "Ana Trujillo Emparedados y helados", ShippingAddress = new Address("Avda. de la Constitucin 2222", "Mxico D.F.", "null", "05021", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10626, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Aug 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 20 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 138.69M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10627, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 107.46M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10628, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 20 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 30.36M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10629, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 20 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 85.46M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10630, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Aug 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 32.35M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10631, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.87M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10632, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 41.38M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10633, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Aug 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 477.90M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10634, Customer = customers.Where(c => c.CompanyName == "Folies gourmandes").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 487.38M, ShippedTo = "Folies gourmandes", ShippingAddress = new Address("184, chausse de Tournai", "Lille", "null", "59000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10635, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Aug 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 47.46M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10636, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 1.15M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10637, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Aug 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 201.29M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10638, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Aug 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 158.44M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10639, Customer = customers.Where(c => c.CompanyName == "Sant Gourmet").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Aug 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 38.64M, ShippedTo = "Sant Gourmet", ShippingAddress = new Address("Erling Skakkes gate 78", "Stavern", "null", "4110", "Norway", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10640, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 23.55M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10641, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 179.61M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10642, Customer = customers.Where(c => c.CompanyName == "Simons bistro").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Aug 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 41.89M, ShippedTo = "Simons bistro", ShippingAddress = new Address("Vinbltet 34", "Kobenhavn", "null", "1734", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10643, Customer = customers.Where(c => c.CompanyName == "Alfreds Futterkiste").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Aug 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 29.46M, ShippedTo = "Alfreds Futterkiste", ShippingAddress = new Address("Obere Str. 57", "Berlin", "null", "12209", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10644, Customer = customers.Where(c => c.CompanyName == "Wellington Importadora").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Aug 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 0.14M, ShippedTo = "Wellington Importadora", ShippingAddress = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10645, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 12.41M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10646, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Aug 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 142.33M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10647, Customer = customers.Where(c => c.CompanyName == "Que Delcia").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Aug 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 45.54M, ShippedTo = "Que Delcia", ShippingAddress = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10648, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Aug 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 14.25M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10649, Customer = customers.Where(c => c.CompanyName == "Maison Dewey").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Aug 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Aug 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 6.20M, ShippedTo = "Maison Dewey", ShippingAddress = new Address("Rue Joseph-Bens 532", "Bruxelles", "null", "B-1180", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10650, Customer = customers.Where(c => c.CompanyName == "Familia Arquibaldo").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Aug 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 26 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 176.81M, ShippedTo = "Familia Arquibaldo", ShippingAddress = new Address("Rua Ors, 92", "Sao Paulo", "SP", "05442-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10651, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Sep  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 20.60M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10652, Customer = customers.Where(c => c.CompanyName == "Gourmet Lanchonetes").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Sep  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 7.14M, ShippedTo = "Gourmet Lanchonetes", ShippingAddress = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10653, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 93.25M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10654, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Sep  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 55.26M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10655, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.41M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10656, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Sep  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 57.15M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10657, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 352.69M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10658, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Sep  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 364.15M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10659, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Sep  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 105.81M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10660, Customer = customers.Where(c => c.CompanyName == "Hungry Coyote Import Store").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Sep  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 111.29M, ShippedTo = "Hungry Coyote Import Store", ShippingAddress = new Address("City Center Plaza 516 Main St.", "Elgin", "OR", "97827", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10661, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Sep  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 17.55M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10662, Customer = customers.Where(c => c.CompanyName == "Lonesome Pine Restaurant").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Sep  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.28M, ShippedTo = "Lonesome Pine Restaurant", ShippingAddress = new Address("89 Chiaroscuro Rd.", "Portland", "OR", "97219", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10663, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Sep 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 113.15M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10664, Customer = customers.Where(c => c.CompanyName == "Furia Bacalhau e Frutos do Mar").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.27M, ShippedTo = "Furia Bacalhau e Frutos do Mar", ShippingAddress = new Address("Jardim das rosas n. 32", "Lisboa", "null", "1675", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10665, Customer = customers.Where(c => c.CompanyName == "Lonesome Pine Restaurant").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 17 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 26.31M, ShippedTo = "Lonesome Pine Restaurant", ShippingAddress = new Address("89 Chiaroscuro Rd.", "Portland", "OR", "97219", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10666, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Sep 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 232.42M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10667, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Sep 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 78.09M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10668, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 23 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 47.22M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10669, Customer = customers.Where(c => c.CompanyName == "Simons bistro").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 24.39M, ShippedTo = "Simons bistro", ShippingAddress = new Address("Vinbltet 34", "Kobenhavn", "null", "1734", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10670, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Sep 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 203.48M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10671, Customer = customers.Where(c => c.CompanyName == "France restauration").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 30.34M, ShippedTo = "France restauration", ShippingAddress = new Address("54, rue Royale", "Nantes", "null", "44000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10672, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Sep 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 95.75M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10673, Customer = customers.Where(c => c.CompanyName == "Wilman Kala").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 22.76M, ShippedTo = "Wilman Kala", ShippingAddress = new Address("Keskuskatu 45", "Helsinki", "null", "21240", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10674, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Sep 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 0.90M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10675, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Sep 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 23 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 31.85M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10676, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 20 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 2.01M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10677, Customer = customers.Where(c => c.CompanyName == "Antonio Moreno Taquera").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 20 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.03M, ShippedTo = "Antonio Moreno Taquera", ShippingAddress = new Address("Mataderos  2312", "Mxico D.F.", "null", "05023", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10678, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Sep 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 388.98M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10679, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Sep 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 27.94M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10680, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Sep 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 26.61M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10681, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Sep 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 76.13M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10682, Customer = customers.Where(c => c.CompanyName == "Antonio Moreno Taquera").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Sep 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 36.13M, ShippedTo = "Antonio Moreno Taquera", ShippingAddress = new Address("Mataderos  2312", "Mxico D.F.", "null", "05023", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10683, Customer = customers.Where(c => c.CompanyName == "Du monde entier").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct  1 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 4.40M, ShippedTo = "Du monde entier", ShippingAddress = new Address("67, rue des Cinquante Otages", "Nantes", "null", "44000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10684, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Sep 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Sep 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 145.63M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10685, Customer = customers.Where(c => c.CompanyName == "Gourmet Lanchonetes").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Sep 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 33.75M, ShippedTo = "Gourmet Lanchonetes", ShippingAddress = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10686, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Sep 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 96.50M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10687, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Sep 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 30 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 296.43M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10688, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 299.09M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10689, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Oct  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 13.42M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10690, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Oct  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 15.80M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10691, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Oct  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 810.05M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10692, Customer = customers.Where(c => c.CompanyName == "Alfreds Futterkiste").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 31 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 61.02M, ShippedTo = "Alfred's Futterkiste", ShippingAddress = new Address("Obere Str. 57", "Berlin", "null", "12209", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10693, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 20 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 139.34M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10694, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Oct  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 398.36M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10695, Customer = customers.Where(c => c.CompanyName == "Wilman Kala").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Oct  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 16.72M, ShippedTo = "Wilman Kala", ShippingAddress = new Address("Keskuskatu 45", "Helsinki", "null", "21240", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10696, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Oct  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 102.55M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10697, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 45.52M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10698, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 17 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 272.47M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10699, Customer = customers.Where(c => c.CompanyName == "Morgenstern Gesundkost").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 13 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 0.58M, ShippedTo = "Morgenstern Gesundkost", ShippingAddress = new Address("Heerstr. 22", "Leipzig", "null", "04179", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10700, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov  7 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 65.10M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10701, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Oct 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 27 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 220.31M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10702, Customer = customers.Where(c => c.CompanyName == "Alfreds Futterkiste").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 23.94M, ShippedTo = "Alfred's Futterkiste", ShippingAddress = new Address("Obere Str. 57", "Berlin", "null", "12209", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10703, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Oct 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 20 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 152.30M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10704, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Oct 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 4.78M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10705, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Oct 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.52M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10706, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Oct 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 13 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 135.63M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10707, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Oct 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 23 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 21.74M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10708, Customer = customers.Where(c => c.CompanyName == "The Big Cheese").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Oct 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 2.96M, ShippedTo = "The Big Cheese", ShippingAddress = new Address("89 Jefferson Way Suite 2", "Portland", "OR", "97201", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10709, Customer = customers.Where(c => c.CompanyName == "Gourmet Lanchonetes").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Oct 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 14 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 20 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 210.80M, ShippedTo = "Gourmet Lanchonetes", ShippingAddress = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10710, Customer = customers.Where(c => c.CompanyName == "Franchi S.p.A.").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Oct 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 23 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 4.98M, ShippedTo = "Franchi S.p.A.", ShippingAddress = new Address("Via Monte Bianco 34", "Torino", "null", "10100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10711, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Oct 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 52.41M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10712, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 31 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 89.93M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10713, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Oct 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 167.05M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10714, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Oct 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 24.49M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10715, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov  6 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 63.20M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10716, Customer = customers.Where(c => c.CompanyName == "Rancho grande").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 22.57M, ShippedTo = "Rancho grande", ShippingAddress = new Address("Av. del Libertador 900", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10717, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Oct 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 21 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 59.25M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10718, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Oct 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 29 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 170.88M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10719, Customer = customers.Where(c => c.CompanyName == "Let's Stop N Shop").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Oct 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 51.44M, ShippedTo = "Let's Stop N Shop", ShippingAddress = new Address("87 Polk St. Suite 5", "San Francisco", "CA", "94117", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10720, Customer = customers.Where(c => c.CompanyName == "Que Delcia").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Oct 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 9.53M, ShippedTo = "Que Delcia", ShippingAddress = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10721, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Oct 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 26 1997 12:00AM"), ShippingDate = DateTime.Parse("Oct 31 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 48.92M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10722, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Oct 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 74.58M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10723, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Oct 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 27 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 21.72M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10724, Customer = customers.Where(c => c.CompanyName == "Mre Paillarde").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Oct 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 57.75M, ShippedTo = "Mre Paillarde", ShippingAddress = new Address("43 rue St. Laurent", "Montral", "Qubec", "H1J 1C3", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10725, Customer = customers.Where(c => c.CompanyName == "Familia Arquibaldo").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Oct 31 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 10.83M, ShippedTo = "Familia Arquibaldo", ShippingAddress = new Address("Rua Ors, 92", "Sao Paulo", "SP", "05442-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10726, Customer = customers.Where(c => c.CompanyName == "Eastern Connection").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 16.56M, ShippedTo = "Eastern Connection", ShippingAddress = new Address("35 King George", "London", "null", "WX3 6FW", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10727, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Nov  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  1 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  5 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 89.90M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10728, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  2 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 11 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 58.33M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10729, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Nov  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 141.06M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10730, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Nov  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  3 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 20.12M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10731, Customer = customers.Where(c => c.CompanyName == "Chop-suey Chinese").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Nov  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 14 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 96.65M, ShippedTo = "Chop-suey Chinese", ShippingAddress = new Address("Hauptstr. 31", "Bern", "null", "3012", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10732, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov  6 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  4 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov  7 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 16.97M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10733, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Nov  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 110.11M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10734, Customer = customers.Where(c => c.CompanyName == "Gourmet Lanchonetes").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Nov  7 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  5 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.63M, ShippedTo = "Gourmet Lanchonetes", ShippingAddress = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10735, Customer = customers.Where(c => c.CompanyName == "Let's Stop N Shop").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Nov 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  8 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 45.97M, ShippedTo = "Let's Stop N Shop", ShippingAddress = new Address("87 Polk St. Suite 5", "San Francisco", "CA", "94117", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10736, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Nov 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 44.10M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10737, Customer = customers.Where(c => c.CompanyName == "Vins et alcools Chevalier").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Nov 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec  9 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 7.79M, ShippedTo = "Vins et alcools Chevalier", ShippingAddress = new Address("59 rue de l'Abbaye", "Reims", "null", "51100", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10738, Customer = customers.Where(c => c.CompanyName == "Spcialits du monde").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Nov 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 2.91M, ShippedTo = "Spcialits du monde", ShippingAddress = new Address("25, rue Lauriston", "Paris", "null", "75016", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10739, Customer = customers.Where(c => c.CompanyName == "Vins et alcools Chevalier").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 10 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 17 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 11.08M, ShippedTo = "Vins et alcools Chevalier", ShippingAddress = new Address("59 rue de l'Abbaye", "Reims", "null", "51100", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10740, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov 13 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 11 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 81.88M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10741, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Nov 28 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 10.96M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10742, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 14 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 12 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 243.73M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10743, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Nov 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 23.72M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10744, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Nov 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 15 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 69.19M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10745, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Nov 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 16 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 3.52M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10746, Customer = customers.Where(c => c.CompanyName == "Chop-suey Chinese").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Nov 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 21 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 31.43M, ShippedTo = "Chop-suey Chinese", ShippingAddress = new Address("Hauptstr. 31", "Bern", "null", "3012", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10747, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Nov 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 17 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 117.33M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10748, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 232.55M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10749, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov 20 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 18 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 61.53M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10750, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Nov 21 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 19 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 79.30M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10751, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  3 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 130.79M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10752, Customer = customers.Where(c => c.CompanyName == "North/South").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Nov 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 22 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.39M, ShippedTo = "North/South", ShippingAddress = new Address("South House 300 Queensbridge", "London", "null", "SW7 1RZ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10753, Customer = customers.Where(c => c.CompanyName == "Franchi S.p.A.").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 7.70M, ShippedTo = "Franchi S.p.A.", ShippingAddress = new Address("Via Monte Bianco 34", "Torino", "null", "10100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10754, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Nov 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 23 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 27 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 2.38M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10755, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Nov 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 24 1997 12:00AM"), ShippingDate = DateTime.Parse("Nov 28 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 16.71M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10756, Customer = customers.Where(c => c.CompanyName == "Split Rail Beer & Ale").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Nov 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  2 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 73.21M, ShippedTo = "Split Rail Beer & Ale", ShippingAddress = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10757, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Nov 27 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 8.19M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10758, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 26 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  4 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 138.17M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10759, Customer = customers.Where(c => c.CompanyName == "Ana Trujillo Emparedados y helados").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Nov 28 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 26 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 11.99M, ShippedTo = "Ana Trujillo Emparedados y helados", ShippingAddress = new Address("Avda. de la Constitucin 2222", "Mxico D.F.", "null", "05021", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10760, Customer = customers.Where(c => c.CompanyName == "Maison Dewey").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec  1 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 10 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 155.64M, ShippedTo = "Maison Dewey", ShippingAddress = new Address("Rue Joseph-Bens 532", "Bruxelles", "null", "B-1180", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10761, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Dec  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 18.66M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10762, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec  2 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 328.74M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10763, Customer = customers.Where(c => c.CompanyName == "Folies gourmandes").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 31 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 37.35M, ShippedTo = "Folies gourmandes", ShippingAddress = new Address("184, chausse de Tournai", "Lille", "null", "59000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10764, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Dec  3 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 31 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec  8 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 145.45M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10765, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec  4 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  1 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 42.74M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10766, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec  9 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 157.55M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10767, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec  5 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.59M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10768, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 15 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 146.32M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10769, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec  8 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 65.06M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10770, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Dec  9 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 17 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 5.32M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10771, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Dec 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  7 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 11.19M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10772, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec 10 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  7 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 91.28M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10773, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  8 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 16 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 96.43M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10774, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec 11 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 25 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 12 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 48.20M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10775, Customer = customers.Where(c => c.CompanyName == "The Cracker Box").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Dec 12 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  9 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 20.25M, ShippedTo = "The Cracker Box", ShippingAddress = new Address("55 Grizzly Peak Rd.", "Butte", "MT", "59801", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10776, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 12 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 18 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 351.53M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10777, Customer = customers.Where(c => c.CompanyName == "Gourmet Lanchonetes").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Dec 15 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 29 1997 12:00AM"), ShippingDate = DateTime.Parse("Jan 21 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.01M, ShippedTo = "Gourmet Lanchonetes", ShippingAddress = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10778, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 6.79M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10779, Customer = customers.Where(c => c.CompanyName == "Morgenstern Gesundkost").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 14 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 58.13M, ShippedTo = "Morgenstern Gesundkost", ShippingAddress = new Address("Heerstr. 22", "Leipzig", "null", "04179", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10780, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Dec 16 1997 12:00AM"), RequiredDate = DateTime.Parse("Dec 30 1997 12:00AM"), ShippingDate = DateTime.Parse("Dec 25 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 42.13M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10781, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Dec 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 14 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 73.16M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10782, Customer = customers.Where(c => c.CompanyName == "Cactus Comidas para llevar").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Dec 17 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 14 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.10M, ShippedTo = "Cactus Comidas para llevar", ShippingAddress = new Address("Cerrito 333", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10783, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 19 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 124.98M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10784, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 22 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 70.09M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10785, Customer = customers.Where(c => c.CompanyName == "GROSELLA-Restaurante").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 18 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 24 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.51M, ShippedTo = "GROSELLA-Restaurante", ShippingAddress = new Address("5 Ave. Los Palos Grandes", "Caracas", "DF", "1081", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10786, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Dec 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 23 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 110.87M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10787, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Dec 19 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 249.93M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10788, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 19 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 42.70M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10789, Customer = customers.Where(c => c.CompanyName == "Folies gourmandes").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 19 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 31 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 100.60M, ShippedTo = "Folies gourmandes", ShippingAddress = new Address("184, chausse de Tournai", "Lille", "null", "59000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10790, Customer = customers.Where(c => c.CompanyName == "Gourmet Lanchonetes").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Dec 22 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 19 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 26 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 28.23M, ShippedTo = "Gourmet Lanchonetes", ShippingAddress = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10791, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Dec 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  1 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 16.85M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10792, Customer = customers.Where(c => c.CompanyName == "Wolski  Zajazd").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 23 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 31 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 23.79M, ShippedTo = "Wolski Zajazd", ShippingAddress = new Address("ul. Filtrowa 68", "Warszawa", "null", "01-012", "Poland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10793, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 21 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  8 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.52M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10794, Customer = customers.Where(c => c.CompanyName == "Que Delcia").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Dec 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 21 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 21.49M, ShippedTo = "Que Delcia", ShippingAddress = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10795, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Dec 24 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 21 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 126.66M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10796, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 22 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 14 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 26.52M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10797, Customer = customers.Where(c => c.CompanyName == "Drachenblut Delikatessen").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Dec 25 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 22 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 33.35M, ShippedTo = "Drachenblut Delikatessen", ShippingAddress = new Address("Walserweg 21", "Aachen", "null", "52066", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10798, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Dec 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 23 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 2.33M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10799, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Dec 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Feb  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 30.76M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10800, Customer = customers.Where(c => c.CompanyName == "Seven Seas Imports").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Dec 26 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 23 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 137.44M, ShippedTo = "Seven Seas Imports", ShippingAddress = new Address("90 Wadhurst Rd.", "London", "null", "OX15 4NB", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10801, Customer = customers.Where(c => c.CompanyName == "Blido Comidas preparadas").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Dec 31 1997 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 97.09M, ShippedTo = "Blido Comidas preparadas", ShippingAddress = new Address("C/ Araquil, 67", "Madrid", "null", "28023", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10802, Customer = customers.Where(c => c.CompanyName == "Simons bistro").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec 29 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 257.26M, ShippedTo = "Simons bistro", ShippingAddress = new Address("Vinbltet 34", "Kobenhavn", "null", "1734", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10803, Customer = customers.Where(c => c.CompanyName == "Wellington Importadora").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 55.23M, ShippedTo = "Wellington Importadora", ShippingAddress = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10804, Customer = customers.Where(c => c.CompanyName == "Seven Seas Imports").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Dec 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  7 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 27.33M, ShippedTo = "Seven Seas Imports", ShippingAddress = new Address("90 Wadhurst Rd.", "London", "null", "OX15 4NB", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10805, Customer = customers.Where(c => c.CompanyName == "The Big Cheese").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Dec 30 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 237.34M, ShippedTo = "The Big Cheese", ShippingAddress = new Address("89 Jefferson Way Suite 2", "Portland", "OR", "97201", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10806, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Dec 31 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 28 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 22.11M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10807, Customer = customers.Where(c => c.CompanyName == "Franchi S.p.A.").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Dec 31 1997 12:00AM"), RequiredDate = DateTime.Parse("Jan 28 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 1.36M, ShippedTo = "Franchi S.p.A.", ShippingAddress = new Address("Via Monte Bianco 34", "Torino", "null", "10100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10808, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan  1 1998 12:00AM"), RequiredDate = DateTime.Parse("Jan 29 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 45.53M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10809, Customer = customers.Where(c => c.CompanyName == "Wellington Importadora").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jan  1 1998 12:00AM"), RequiredDate = DateTime.Parse("Jan 29 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  7 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 4.87M, ShippedTo = "Wellington Importadora", ShippingAddress = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10810, Customer = customers.Where(c => c.CompanyName == "Laughing Bacchus Wine Cellars").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan  1 1998 12:00AM"), RequiredDate = DateTime.Parse("Jan 29 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  7 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.33M, ShippedTo = "Laughing Bacchus Wine Cellars", ShippingAddress = new Address("2319 Elm St.", "Vancouver", "BC", "V3F 2K1", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10811, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Jan 30 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  8 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 31.22M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10812, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jan  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Jan 30 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 12 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 59.78M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10813, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 47.38M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10814, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 14 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 130.94M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10815, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 14 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 14.62M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10816, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  3 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 719.78M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10817, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Jan 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 306.07M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10818, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jan  7 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 12 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 65.48M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10819, Customer = customers.Where(c => c.CompanyName == "Cactus Comidas para llevar").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan  7 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 19.76M, ShippedTo = "Cactus Comidas para llevar", ShippingAddress = new Address("Cerrito 333", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10820, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan  7 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 37.52M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10821, Customer = customers.Where(c => c.CompanyName == "Split Rail Beer & Ale").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan  8 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 15 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 36.68M, ShippedTo = "Split Rail Beer & Ale", ShippingAddress = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10822, Customer = customers.Where(c => c.CompanyName == "Trail's Head Gourmet Provisioners").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jan  8 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 7.00M, ShippedTo = "Trail's Head Gourmet Provisioners", ShippingAddress = new Address("722 DaVinci Blvd.", "Kirkland", "WA", "98034", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10823, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jan  9 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 163.97M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10824, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan  9 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 1.23M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10825, Customer = customers.Where(c => c.CompanyName == "Drachenblut Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan  9 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 14 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 79.25M, ShippedTo = "Drachenblut Delikatessen", ShippingAddress = new Address("Walserweg 21", "Aachen", "null", "52066", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10826, Customer = customers.Where(c => c.CompanyName == "Blondesddsl pre et fils").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jan 12 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  9 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 7.09M, ShippedTo = "Blondel pre et fils", ShippingAddress = new Address("24, place Klber", "Strasbourg", "null", "67000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10827, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan 12 1998 12:00AM"), RequiredDate = DateTime.Parse("Jan 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 63.54M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10828, Customer = customers.Where(c => c.CompanyName == "Rancho grande").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jan 13 1998 12:00AM"), RequiredDate = DateTime.Parse("Jan 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 90.85M, ShippedTo = "Rancho grande", ShippingAddress = new Address("Av. del Libertador 900", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10829, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jan 13 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 10 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 154.72M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10830, Customer = customers.Where(c => c.CompanyName == "Tradio Hipermercados").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 13 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 21 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 81.83M, ShippedTo = "Tradiao Hipermercados", ShippingAddress = new Address("Av. Ins de Castro, 414", "Sao Paulo", "SP", "05634-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10831, Customer = customers.Where(c => c.CompanyName == "Sant Gourmet").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 14 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 11 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 72.19M, ShippedTo = "Sant Gourmet", ShippingAddress = new Address("Erling Skakkes gate 78", "Stavern", "null", "4110", "Norway", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10832, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan 14 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 11 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 43.26M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10833, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Jan 15 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 12 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 71.49M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10834, Customer = customers.Where(c => c.CompanyName == "Tradio Hipermercados").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan 15 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 12 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 29.78M, ShippedTo = "Tradiao Hipermercados", ShippingAddress = new Address("Av. Ins de Castro, 414", "Sao Paulo", "SP", "05634-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10835, Customer = customers.Where(c => c.CompanyName == "Alfreds Futterkiste").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan 15 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 12 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 21 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 69.53M, ShippedTo = "Alfred's Futterkiste", ShippingAddress = new Address("Obere Str. 57", "Berlin", "null", "12209", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10836, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jan 16 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 21 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 411.88M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10837, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jan 16 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 13.32M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10838, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 19 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 59.28M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10839, Customer = customers.Where(c => c.CompanyName == "Tradio Hipermercados").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 19 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 22 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 35.43M, ShippedTo = "Tradiao Hipermercados", ShippingAddress = new Address("Av. Ins de Castro, 414", "Sao Paulo", "SP", "05634-030", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10840, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 19 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 2.71M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10841, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jan 20 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 29 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 424.30M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10842, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan 20 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 29 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 54.42M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10843, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 21 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 18 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 26 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 9.26M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10844, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan 21 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 18 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 26 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 25.22M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10845, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan 21 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 212.98M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10846, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan 22 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 56.46M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10847, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 22 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 487.57M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10848, Customer = customers.Where(c => c.CompanyName == "Consolidated Holdings").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Jan 23 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 29 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 38.24M, ShippedTo = "Consolidated Holdings", ShippingAddress = new Address("Berkeley Gardens 12  Brewery", "London", "null", "WX1 6LT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10849, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jan 23 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 0.56M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10850, Customer = customers.Where(c => c.CompanyName == "Victuailles en stock").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan 23 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 49.19M, ShippedTo = "Victuailles en stock", ShippingAddress = new Address("2, rue du Commerce", "Lyon", "null", "69004", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10851, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Jan 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 23 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 160.55M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10852, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb  9 1998 12:00AM"), ShippingDate = DateTime.Parse("Jan 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 174.05M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10853, Customer = customers.Where(c => c.CompanyName == "Blauer See Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Jan 27 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  3 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 53.83M, ShippedTo = "Blauer See Delikatessen", ShippingAddress = new Address("Forsterstr. 57", "Mannheim", "null", "68306", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10854, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 27 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 100.22M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10855, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 27 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 170.97M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10856, Customer = customers.Where(c => c.CompanyName == "Antonio Moreno Taquera").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 28 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 25 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 58.43M, ShippedTo = "Antonio Moreno Taquera", ShippingAddress = new Address("Mataderos  2312", "Mxico D.F.", "null", "05023", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10857, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan 28 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 25 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 188.85M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10858, Customer = customers.Where(c => c.CompanyName == "La corne d'abondance").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Jan 29 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  3 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 52.51M, ShippedTo = "La corne d'abondance", ShippingAddress = new Address("67, avenue de l'Europe", "Versailles", "null", "78000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10859, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Jan 29 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 76.10M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10860, Customer = customers.Where(c => c.CompanyName == "France restauration").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Jan 29 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 19.26M, ShippedTo = "France restauration", ShippingAddress = new Address("54, rue Royale", "Nantes", "null", "44000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10861, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Jan 30 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 17 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 14.93M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10862, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Jan 30 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 53.23M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10863, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 17 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 30.26M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10864, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.04M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10865, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Feb  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Feb 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 12 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 348.14M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10866, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Feb  3 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  3 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 12 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 109.11M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10867, Customer = customers.Where(c => c.CompanyName == "Lonesome Pine Restaurant").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Feb  3 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 11 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 1.93M, ShippedTo = "Lonesome Pine Restaurant", ShippingAddress = new Address("89 Chiaroscuro Rd.", "Portland", "OR", "97219", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10868, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Feb  4 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 191.27M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10869, Customer = customers.Where(c => c.CompanyName == "Seven Seas Imports").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Feb  4 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 143.28M, ShippedTo = "Seven Seas Imports", ShippingAddress = new Address("90 Wadhurst Rd.", "London", "null", "OX15 4NB", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10870, Customer = customers.Where(c => c.CompanyName == "Wolski  Zajazd").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Feb  4 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 12.04M, ShippedTo = "Wolski Zajazd", ShippingAddress = new Address("ul. Filtrowa 68", "Warszawa", "null", "01-012", "Poland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10871, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Feb  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 112.27M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10872, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Feb  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 175.32M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10873, Customer = customers.Where(c => c.CompanyName == "Wilman Kala").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.82M, ShippedTo = "Wilman Kala", ShippingAddress = new Address("Keskuskatu 45", "Helsinki", "null", "21240", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10874, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Feb  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 11 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 19.58M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10875, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  3 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 32.37M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10876, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Feb  9 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  9 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 12 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 60.42M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10877, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb  9 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar  9 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 38.06M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10878, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 10 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 10 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 12 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 46.69M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10879, Customer = customers.Where(c => c.CompanyName == "Wilman Kala").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 10 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 10 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 12 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 8.50M, ShippedTo = "Wilman Kala", ShippingAddress = new Address("Keskuskatu 45", "Helsinki", "null", "21240", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10880, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Feb 10 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 88.01M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10881, Customer = customers.Where(c => c.CompanyName == "Cactus Comidas para llevar").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 11 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 11 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 2.84M, ShippedTo = "Cactus Comidas para llevar", ShippingAddress = new Address("Cerrito 333", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10882, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 11 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 11 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 23.10M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10883, Customer = customers.Where(c => c.CompanyName == "Lonesome Pine Restaurant").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb 12 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 12 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 0.53M, ShippedTo = "Lonesome Pine Restaurant", ShippingAddress = new Address("89 Chiaroscuro Rd.", "Portland", "OR", "97219", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10884, Customer = customers.Where(c => c.CompanyName == "Let's Stop N Shop").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 12 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 12 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 90.97M, ShippedTo = "Let's Stop N Shop", ShippingAddress = new Address("87 Polk St. Suite 5", "San Francisco", "CA", "94117", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10885, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Feb 12 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 12 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 5.64M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10886, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 13 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 4.99M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10887, Customer = customers.Where(c => c.CompanyName == "Galera del gastrnomo").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Feb 13 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.25M, ShippedTo = "Galera del gastronmo", ShippingAddress = new Address("Rambla de Catalua, 23", "Barcelona", "null", "8022", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10888, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 16 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 51.87M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10889, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Feb 16 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 280.61M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10890, Customer = customers.Where(c => c.CompanyName == "Du monde entier").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Feb 16 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 32.76M, ShippedTo = "Du monde entier", ShippingAddress = new Address("67, rue des Cinquante Otages", "Nantes", "null", "44000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10891, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Feb 17 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 20.37M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10892, Customer = customers.Where(c => c.CompanyName == "Maison Dewey").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 17 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 120.27M, ShippedTo = "Maison Dewey", ShippingAddress = new Address("Rue Joseph-Bens 532", "Bruxelles", "null", "B-1180", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10893, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Feb 18 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 18 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 77.78M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10894, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 18 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 18 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 116.13M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10895, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 18 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 18 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 162.75M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10896, Customer = customers.Where(c => c.CompanyName == "Maison Dewey").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Feb 19 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 19 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 32.45M, ShippedTo = "Maison Dewey", ShippingAddress = new Address("Rue Joseph-Bens 532", "Bruxelles", "null", "B-1180", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10897, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 19 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 19 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 25 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 603.54M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10898, Customer = customers.Where(c => c.CompanyName == "Ocano Atlntico Ltda.").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 20 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.27M, ShippedTo = "Ocano Atlntico Ltda.", ShippingAddress = new Address("Ing. Gustavo Moncada 8585 Piso 20-A", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10899, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Feb 20 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 26 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 1.21M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10900, Customer = customers.Where(c => c.CompanyName == "Wellington Importadora").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 20 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.66M, ShippedTo = "Wellington Importadora", ShippingAddress = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10901, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 23 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 23 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 26 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 62.09M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10902, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 23 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 23 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  3 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 44.15M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10903, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 24 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 36.71M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10904, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 24 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 162.95M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10905, Customer = customers.Where(c => c.CompanyName == "Wellington Importadora").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Feb 24 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 13.72M, ShippedTo = "Wellington Importadora", ShippingAddress = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10906, Customer = customers.Where(c => c.CompanyName == "Wolski  Zajazd").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 25 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 11 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  3 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 26.29M, ShippedTo = "Wolski Zajazd", ShippingAddress = new Address("ul. Filtrowa 68", "Warszawa", "null", "01-012", "Poland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10907, Customer = customers.Where(c => c.CompanyName == "Spcialits du monde").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Feb 25 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 25 1998 12:00AM"), ShippingDate = DateTime.Parse("Feb 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 9.19M, ShippedTo = "Spcialits du monde", ShippingAddress = new Address("25, rue Lauriston", "Paris", "null", "75016", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10908, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 26 1998 12:01AM"), RequiredDate = DateTime.Parse("Mar 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 32.96M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10909, Customer = customers.Where(c => c.CompanyName == "Sant Gourmet").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 53.05M, ShippedTo = "Sant Gourmet", ShippingAddress = new Address("Erling Skakkes gate 78", "Stavern", "null", "4110", "Norway", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10910, Customer = customers.Where(c => c.CompanyName == "Wilman Kala").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 38.11M, ShippedTo = "Wilman Kala", ShippingAddress = new Address("Keskuskatu 45", "Helsinki", "null", "21240", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10911, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Feb 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 38.19M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10912, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Feb 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 580.91M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10913, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Feb 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 33.05M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10914, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Feb 27 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 21.19M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10915, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Feb 27 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.51M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10916, Customer = customers.Where(c => c.CompanyName == "Rancho grande").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Feb 27 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 63.77M, ShippedTo = "Rancho grande", ShippingAddress = new Address("Av. del Libertador 900", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10917, Customer = customers.Where(c => c.CompanyName == "Romero y tomillo").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 30 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 11 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 8.29M, ShippedTo = "Romero y tomillo", ShippingAddress = new Address("Gran Va, 1", "Madrid", "null", "28001", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10918, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 30 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 11 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 48.83M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10919, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 30 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 19.80M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10920, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar  3 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 31 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 29.61M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10921, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar  3 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 14 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 176.48M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10922, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Mar  3 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 31 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 62.74M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10923, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Mar  3 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 14 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 68.26M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10924, Customer = customers.Where(c => c.CompanyName == "Berglunds snabbkp").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar  4 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  1 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  8 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 151.52M, ShippedTo = "Berglunds snabbkp", ShippingAddress = new Address("Berguvsvgen  8", "Lule", "null", "S-958 22", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10925, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar  4 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  1 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 2.27M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10926, Customer = customers.Where(c => c.CompanyName == "Ana Trujillo Emparedados y helados").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar  4 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  1 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 11 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 39.92M, ShippedTo = "Ana Trujillo Emparedados y helados", ShippingAddress = new Address("Avda. de la Constitucin 2222", "Mxico D.F.", "null", "05021", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10927, Customer = customers.Where(c => c.CompanyName == "La corne d'abondance").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  8 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 19.79M, ShippedTo = "La corne d'abondance", ShippingAddress = new Address("67, avenue de l'Europe", "Versailles", "null", "78000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10928, Customer = customers.Where(c => c.CompanyName == "Galera del gastrnomo").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 1.36M, ShippedTo = "Galera del gastronmo", ShippingAddress = new Address("Rambla de Catalua, 23", "Barcelona", "null", "8022", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10929, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Mar  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 12 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 33.93M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10930, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 15.55M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10931, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 13.60M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10932, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  3 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 24 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 134.64M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10933, Customer = customers.Where(c => c.CompanyName == "Island Trading").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Mar  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  3 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 54.15M, ShippedTo = "Island Trading", ShippingAddress = new Address("Garden House Crowther Way", "Cowes", "Isle of Wight", "PO31 7PJ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10934, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar  9 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 12 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 32.01M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10935, Customer = customers.Where(c => c.CompanyName == "Wellington Importadora").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar  9 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 47.59M, ShippedTo = "Wellington Importadora", ShippingAddress = new Address("Rua do Mercado, 12", "Resende", "SP", "08737-363", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10936, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar  9 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 33.68M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10937, Customer = customers.Where(c => c.CompanyName == "Cactus Comidas para llevar").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Mar 10 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 31.51M, ShippedTo = "Cactus Comidas para llevar", ShippingAddress = new Address("Cerrito 333", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10938, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar 10 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  7 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 31.89M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10939, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 10 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  7 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 76.33M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10940, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 11 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  8 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 19.77M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10941, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Mar 11 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  8 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 400.81M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10942, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Mar 11 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  8 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 17.95M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10943, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar 11 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  8 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 2.17M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10944, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Mar 12 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 26 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 52.92M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10945, Customer = customers.Where(c => c.CompanyName == "Morgenstern Gesundkost").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar 12 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  9 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 18 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 10.22M, ShippedTo = "Morgenstern Gesundkost", ShippingAddress = new Address("Heerstr. 22", "Leipzig", "null", "04179", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10946, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 12 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  9 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 27.20M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10947, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar 13 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 10 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.26M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10948, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar 13 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 10 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 19 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 23.39M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10949, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 13 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 10 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 17 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 74.44M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10950, Customer = customers.Where(c => c.CompanyName == "Magazzini Alimentari Riuniti").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 16 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 2.50M, ShippedTo = "Magazzini Alimentari Riuniti", ShippingAddress = new Address("Via Ludovico il Moro 22", "Bergamo", "null", "24100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10951, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Mar 16 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  7 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 30.85M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10952, Customer = customers.Where(c => c.CompanyName == "Alfreds Futterkiste").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 16 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 24 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 40.42M, ShippedTo = "Alfred's Futterkiste", ShippingAddress = new Address("Obere Str. 57", "Berlin", "null", "12209", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10953, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Mar 16 1998 12:00AM"), RequiredDate = DateTime.Parse("Mar 30 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 25 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 23.72M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10954, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Mar 17 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 28 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 27.91M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10955, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 17 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 14 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 3.26M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10956, Customer = customers.Where(c => c.CompanyName == "Blauer See Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Mar 17 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 28 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 44.65M, ShippedTo = "Blauer See Delikatessen", ShippingAddress = new Address("Forsterstr. 57", "Mannheim", "null", "68306", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10957, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 18 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 105.36M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10958, Customer = customers.Where(c => c.CompanyName == "Ocano Atlntico Ltda.").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Mar 18 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 49.56M, ShippedTo = "Ocano Atlntico Ltda.", ShippingAddress = new Address("Ing. Gustavo Moncada 8585 Piso 20-A", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10959, Customer = customers.Where(c => c.CompanyName == "Gourmet Lanchonetes").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Mar 18 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 29 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.98M, ShippedTo = "Gourmet Lanchonetes", ShippingAddress = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10960, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar 19 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  2 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  8 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 2.08M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10961, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 19 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 104.47M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10962, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 19 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 275.79M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10963, Customer = customers.Where(c => c.CompanyName == "Furia Bacalhau e Frutos do Mar").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Mar 19 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 26 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 2.70M, ShippedTo = "Furia Bacalhau e Frutos do Mar", ShippingAddress = new Address("Jardim das rosas n. 32", "Lisboa", "null", "1675", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10964, Customer = customers.Where(c => c.CompanyName == "Spcialits du monde").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar 20 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 24 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 87.38M, ShippedTo = "Spcialits du monde", ShippingAddress = new Address("25, rue Lauriston", "Paris", "null", "75016", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10965, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Mar 20 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 144.38M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10966, Customer = customers.Where(c => c.CompanyName == "Chop-suey Chinese").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar 20 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  8 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 27.19M, ShippedTo = "Chop-suey Chinese", ShippingAddress = new Address("Hauptstr. 31", "Bern", "null", "3012", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10967, Customer = customers.Where(c => c.CompanyName == "Toms Spezialitten").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 23 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 62.22M, ShippedTo = "Toms Spezialitten", ShippingAddress = new Address("Luisenstr. 48", "Mnster", "null", "44087", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10968, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 23 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  1 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 74.60M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10969, Customer = customers.Where(c => c.CompanyName == "Comrcio Mineiro").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 23 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 0.21M, ShippedTo = "Comrcio Mineiro", ShippingAddress = new Address("Av. dos Lusadas, 23", "Sao Paulo", "SP", "05432-043", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10970, Customer = customers.Where(c => c.CompanyName == "Blido Comidas preparadas").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Mar 24 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  7 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 24 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 16.16M, ShippedTo = "Blido Comidas preparadas", ShippingAddress = new Address("C/ Araquil, 67", "Madrid", "null", "28023", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10971, Customer = customers.Where(c => c.CompanyName == "France restauration").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 24 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 21 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 121.82M, ShippedTo = "France restauration", ShippingAddress = new Address("54, rue Royale", "Nantes", "null", "44000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10972, Customer = customers.Where(c => c.CompanyName == "La corne d'abondance").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar 24 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 21 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 26 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 0.02M, ShippedTo = "La corne d'abondance", ShippingAddress = new Address("67, avenue de l'Europe", "Versailles", "null", "78000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10973, Customer = customers.Where(c => c.CompanyName == "La corne d'abondance").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Mar 24 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 21 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 15.17M, ShippedTo = "La corne d'abondance", ShippingAddress = new Address("67, avenue de l'Europe", "Versailles", "null", "78000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10974, Customer = customers.Where(c => c.CompanyName == "Split Rail Beer & Ale").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar 25 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr  8 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  3 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 12.96M, ShippedTo = "Split Rail Beer & Ale", ShippingAddress = new Address("P.O. Box 555", "Lander", "WY", "82520", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10975, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 25 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 22 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 32.27M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10976, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 25 1998 12:00AM"), RequiredDate = DateTime.Parse("May  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  3 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 37.97M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10977, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 23 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 208.50M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10978, Customer = customers.Where(c => c.CompanyName == "Maison Dewey").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Mar 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 23 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 32.82M, ShippedTo = "Maison Dewey", ShippingAddress = new Address("Rue Joseph-Bens 532", "Bruxelles", "null", "B-1180", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10979, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 26 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 23 1998 12:00AM"), ShippingDate = DateTime.Parse("Mar 31 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 353.07M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10980, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Mar 27 1998 12:00AM"), RequiredDate = DateTime.Parse("May  8 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 17 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 1.26M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10981, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 27 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 193.37M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10982, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 27 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  8 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 14.01M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10983, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 27 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 657.54M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10984, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Mar 30 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  3 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 211.22M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10985, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 30 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 91.51M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10986, Customer = customers.Where(c => c.CompanyName == "Ocano Atlntico Ltda.").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 30 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 27 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 21 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 217.86M, ShippedTo = "Ocano Atlntico Ltda.", ShippingAddress = new Address("Ing. Gustavo Moncada 8585 Piso 20-A", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10987, Customer = customers.Where(c => c.CompanyName == "Eastern Connection").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Mar 31 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 28 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 185.48M, ShippedTo = "Eastern Connection", ShippingAddress = new Address("35 King George", "London", "null", "WX3 6FW", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10988, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Mar 31 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 28 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 61.14M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10989, Customer = customers.Where(c => c.CompanyName == "Que Delcia").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Mar 31 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 28 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  2 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 34.76M, ShippedTo = "Que Delcia", ShippingAddress = new Address("Rua da Panificadora, 12", "Rio de Janeiro", "RJ", "02389-673", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10990, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr  1 1998 12:00AM"), RequiredDate = DateTime.Parse("May 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  7 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 117.61M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10991, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Apr  1 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 29 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  7 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 38.51M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10992, Customer = customers.Where(c => c.CompanyName == "The Big Cheese").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Apr  1 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 29 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  3 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.27M, ShippedTo = "The Big Cheese", ShippingAddress = new Address("89 Jefferson Way Suite 2", "Portland", "OR", "97201", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10993, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr  1 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 29 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 8.81M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10994, Customer = customers.Where(c => c.CompanyName == "Vaffeljernet").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 16 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  9 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 65.53M, ShippedTo = "Vaffeljernet", ShippingAddress = new Address("Smagsloget 45", "rhus", "null", "8200", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10995, Customer = customers.Where(c => c.CompanyName == "Pericles Comidas clsicas").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Apr  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 30 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 46.00M, ShippedTo = "Pericles Comidas clsicas", ShippingAddress = new Address("Calle Dr. Jorge Cash 321", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10996, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr  2 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 30 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 1.12M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10997, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr  3 1998 12:00AM"), RequiredDate = DateTime.Parse("May 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 73.91M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10998, Customer = customers.Where(c => c.CompanyName == "Wolski  Zajazd").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr  3 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 17 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 17 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 20.31M, ShippedTo = "Wolski Zajazd", ShippingAddress = new Address("ul. Filtrowa 68", "Warszawa", "null", "01-012", "Poland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 10999, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Apr  3 1998 12:00AM"), RequiredDate = DateTime.Parse("May  1 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 96.35M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11000, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr  6 1998 12:00AM"), RequiredDate = DateTime.Parse("May  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 14 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 55.12M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11001, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr  6 1998 12:00AM"), RequiredDate = DateTime.Parse("May  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 14 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 197.30M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11002, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr  6 1998 12:00AM"), RequiredDate = DateTime.Parse("May  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 141.16M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11003, Customer = customers.Where(c => c.CompanyName == "The Cracker Box").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr  6 1998 12:00AM"), RequiredDate = DateTime.Parse("May  4 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr  8 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 14.91M, ShippedTo = "The Cracker Box", ShippingAddress = new Address("55 Grizzly Peak Rd.", "Butte", "MT", "59801", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11004, Customer = customers.Where(c => c.CompanyName == "Maison Dewey").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr  7 1998 12:00AM"), RequiredDate = DateTime.Parse("May  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 44.84M, ShippedTo = "Maison Dewey", ShippingAddress = new Address("Rue Joseph-Bens 532", "Bruxelles", "null", "B-1180", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11005, Customer = customers.Where(c => c.CompanyName == "Wilman Kala").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr  7 1998 12:00AM"), RequiredDate = DateTime.Parse("May  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.75M, ShippedTo = "Wilman Kala", ShippingAddress = new Address("Keskuskatu 45", "Helsinki", "null", "21240", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11006, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr  7 1998 12:00AM"), RequiredDate = DateTime.Parse("May  5 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 15 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 25.19M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11007, Customer = customers.Where(c => c.CompanyName == "Princesa Isabel Vinhos").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr  8 1998 12:00AM"), RequiredDate = DateTime.Parse("May  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 202.24M, ShippedTo = "Princesa Isabel Vinhos", ShippingAddress = new Address("Estrada da sade n. 58", "Lisboa", "null", "1756", "Portugal", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11008, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr  8 1998 12:00AM"), RequiredDate = DateTime.Parse("May  6 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 79.46M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11009, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr  8 1998 12:00AM"), RequiredDate = DateTime.Parse("May  6 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 59.11M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11010, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr  9 1998 12:00AM"), RequiredDate = DateTime.Parse("May  7 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 21 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 28.71M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11011, Customer = customers.Where(c => c.CompanyName == "Alfreds Futterkiste").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr  9 1998 12:00AM"), RequiredDate = DateTime.Parse("May  7 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 1.21M, ShippedTo = "Alfred's Futterkiste", ShippingAddress = new Address("Obere Str. 57", "Berlin", "null", "12209", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11012, Customer = customers.Where(c => c.CompanyName == "Frankenversand").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Apr  9 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 23 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 17 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 242.95M, ShippedTo = "Frankenversand", ShippingAddress = new Address("Berliner Platz 43", "Mnchen", "null", "80805", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11013, Customer = customers.Where(c => c.CompanyName == "Romero y tomillo").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr  9 1998 12:00AM"), RequiredDate = DateTime.Parse("May  7 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 10 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 32.99M, ShippedTo = "Romero y tomillo", ShippingAddress = new Address("Gran Va, 1", "Madrid", "null", "28001", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11014, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 10 1998 12:00AM"), RequiredDate = DateTime.Parse("May  8 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 15 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 23.60M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11015, Customer = customers.Where(c => c.CompanyName == "Sant Gourmet").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 10 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 24 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 4.62M, ShippedTo = "Sant Gourmet", ShippingAddress = new Address("Erling Skakkes gate 78", "Stavern", "null", "4110", "Norway", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11016, Customer = customers.Where(c => c.CompanyName == "Around the Horn").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Apr 10 1998 12:00AM"), RequiredDate = DateTime.Parse("May  8 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 13 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 33.80M, ShippedTo = "Around the Horn", ShippingAddress = new Address("Brook Farm Stratford St. Mary", "Colchester", "Essex", "CO7 6JX", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11017, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Apr 13 1998 12:00AM"), RequiredDate = DateTime.Parse("May 11 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 754.26M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11018, Customer = customers.Where(c => c.CompanyName == "Lonesome Pine Restaurant").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 13 1998 12:00AM"), RequiredDate = DateTime.Parse("May 11 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 11.65M, ShippedTo = "Lonesome Pine Restaurant", ShippingAddress = new Address("89 Chiaroscuro Rd.", "Portland", "OR", "97219", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11019, Customer = customers.Where(c => c.CompanyName == "Rancho grande").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Apr 13 1998 12:00AM"), RequiredDate = DateTime.Parse("May 11 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 3.17M, ShippedTo = "Rancho grande", ShippingAddress = new Address("Av. del Libertador 900", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11020, Customer = customers.Where(c => c.CompanyName == "Ottilies Kseladen").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 14 1998 12:00AM"), RequiredDate = DateTime.Parse("May 12 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 16 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 43.30M, ShippedTo = "Ottilies Kseladen", ShippingAddress = new Address("Mehrheimerstr. 369", "Kln", "null", "50739", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11021, Customer = customers.Where(c => c.CompanyName == "QUICK-Stop").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr 14 1998 12:00AM"), RequiredDate = DateTime.Parse("May 12 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 21 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 297.18M, ShippedTo = "QUICK-Stop", ShippingAddress = new Address("Taucherstrae 10", "Cunewalde", "null", "01307", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11022, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Apr 14 1998 12:00AM"), RequiredDate = DateTime.Parse("May 12 1998 12:00AM"), ShippingDate = DateTime.Parse("May  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 6.27M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11023, Customer = customers.Where(c => c.CompanyName == "B's Beverages").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Apr 14 1998 12:00AM"), RequiredDate = DateTime.Parse("Apr 28 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 24 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 123.83M, ShippedTo = "B's Beverages", ShippingAddress = new Address("Fauntleroy Circus", "London", "null", "EC2 5NT", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11024, Customer = customers.Where(c => c.CompanyName == "Eastern Connection").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 15 1998 12:00AM"), RequiredDate = DateTime.Parse("May 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 74.36M, ShippedTo = "Eastern Connection", ShippingAddress = new Address("35 King George", "London", "null", "WX3 6FW", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11025, Customer = customers.Where(c => c.CompanyName == "Wartian Herkku").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Apr 15 1998 12:00AM"), RequiredDate = DateTime.Parse("May 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 24 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 29.17M, ShippedTo = "Wartian Herkku", ShippingAddress = new Address("Torikatu 38", "Oulu", "null", "90110", "Finland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11026, Customer = customers.Where(c => c.CompanyName == "Franchi S.p.A.").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 15 1998 12:00AM"), RequiredDate = DateTime.Parse("May 13 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 28 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 47.09M, ShippedTo = "Franchi S.p.A.", ShippingAddress = new Address("Via Monte Bianco 34", "Torino", "null", "10100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11027, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Apr 16 1998 12:00AM"), RequiredDate = DateTime.Parse("May 14 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 20 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 52.52M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11028, Customer = customers.Where(c => c.CompanyName == "Kniglich Essen").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 16 1998 12:00AM"), RequiredDate = DateTime.Parse("May 14 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 22 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 29.59M, ShippedTo = "Kniglich Essen", ShippingAddress = new Address("Maubelstr. 90", "Brandenburg", "null", "14776", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11029, Customer = customers.Where(c => c.CompanyName == "Chop-suey Chinese").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 16 1998 12:00AM"), RequiredDate = DateTime.Parse("May 14 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 47.84M, ShippedTo = "Chop-suey Chinese", ShippingAddress = new Address("Hauptstr. 31", "Bern", "null", "3012", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11030, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 17 1998 12:00AM"), RequiredDate = DateTime.Parse("May 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 830.75M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11031, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Apr 17 1998 12:00AM"), RequiredDate = DateTime.Parse("May 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 24 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 227.22M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11032, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 17 1998 12:00AM"), RequiredDate = DateTime.Parse("May 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 606.19M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11033, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 17 1998 12:00AM"), RequiredDate = DateTime.Parse("May 15 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 23 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 84.74M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11034, Customer = customers.Where(c => c.CompanyName == "Old World Delicatessen").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr 20 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  1 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 40.32M, ShippedTo = "Old World Delicatessen", ShippingAddress = new Address("2743 Bering St.", "Anchorage", "AK", "99508", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11035, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 20 1998 12:00AM"), RequiredDate = DateTime.Parse("May 18 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 24 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 0.17M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11036, Customer = customers.Where(c => c.CompanyName == "Drachenblut Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr 20 1998 12:00AM"), RequiredDate = DateTime.Parse("May 18 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 22 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 149.47M, ShippedTo = "Drachenblut Delikatessen", ShippingAddress = new Address("Walserweg 21", "Aachen", "null", "52066", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11037, Customer = customers.Where(c => c.CompanyName == "Godos Cocina Tpica").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 21 1998 12:00AM"), RequiredDate = DateTime.Parse("May 19 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 27 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 3.20M, ShippedTo = "Godos Cocina Tpica", ShippingAddress = new Address("C/ Romero, 33", "Sevilla", "null", "41101", "Spain", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11038, Customer = customers.Where(c => c.CompanyName == "Suprmes dlices").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Apr 21 1998 12:00AM"), RequiredDate = DateTime.Parse("May 19 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 29.59M, ShippedTo = "Suprmes dlices", ShippingAddress = new Address("Boulevard Tirou, 255", "Charleroi", "null", "B-6000", "Belgium", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11039, Customer = customers.Where(c => c.CompanyName == "LINO-Delicateses").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("Apr 21 1998 12:00AM"), RequiredDate = DateTime.Parse("May 19 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 65.00M, ShippedTo = "LINO-Delicateses", ShippingAddress = new Address("Ave. 5 de Mayo Porlamar", "I. de Margarita", "Nueva Esparta", "4980", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11040, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 22 1998 12:00AM"), RequiredDate = DateTime.Parse("May 20 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 18.84M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11041, Customer = customers.Where(c => c.CompanyName == "Chop-suey Chinese").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr 22 1998 12:00AM"), RequiredDate = DateTime.Parse("May 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 28 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 48.22M, ShippedTo = "Chop-suey Chinese", ShippingAddress = new Address("Hauptstr. 31", "Bern", "null", "3012", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11042, Customer = customers.Where(c => c.CompanyName == "Comrcio Mineiro").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 22 1998 12:00AM"), RequiredDate = DateTime.Parse("May  6 1998 12:00AM"), ShippingDate = DateTime.Parse("May  1 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 29.99M, ShippedTo = "Comrcio Mineiro", ShippingAddress = new Address("Av. dos Lusadas, 23", "Sao Paulo", "SP", "05432-043", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11043, Customer = customers.Where(c => c.CompanyName == "Spcialits du monde").First(), Employee = employees.Where(e => e.LastName == "Buchanan").First(), OrderDate = DateTime.Parse("Apr 22 1998 12:00AM"), RequiredDate = DateTime.Parse("May 20 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 29 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 8.80M, ShippedTo = "Spcialits du monde", ShippingAddress = new Address("25, rue Lauriston", "Paris", "null", "75016", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11044, Customer = customers.Where(c => c.CompanyName == "Wolski  Zajazd").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 23 1998 12:00AM"), RequiredDate = DateTime.Parse("May 21 1998 12:00AM"), ShippingDate = DateTime.Parse("May  1 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 8.72M, ShippedTo = "Wolski Zajazd", ShippingAddress = new Address("ul. Filtrowa 68", "Warszawa", "null", "01-012", "Poland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11045, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "Suyama").First(), OrderDate = DateTime.Parse("Apr 23 1998 12:00AM"), RequiredDate = DateTime.Parse("May 21 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 70.58M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11046, Customer = customers.Where(c => c.CompanyName == "Die Wandernde Kuh").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr 23 1998 12:00AM"), RequiredDate = DateTime.Parse("May 21 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 24 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 71.64M, ShippedTo = "Die Wandernde Kuh", ShippingAddress = new Address("Adenauerallee 900", "Stuttgart", "null", "70563", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11047, Customer = customers.Where(c => c.CompanyName == "Eastern Connection").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 24 1998 12:00AM"), RequiredDate = DateTime.Parse("May 22 1998 12:00AM"), ShippingDate = DateTime.Parse("May  1 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 46.62M, ShippedTo = "Eastern Connection", ShippingAddress = new Address("35 King George", "London", "null", "WX3 6FW", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11048, Customer = customers.Where(c => c.CompanyName == "Bottom-Dollar Markets").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 24 1998 12:00AM"), RequiredDate = DateTime.Parse("May 22 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 30 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 24.12M, ShippedTo = "Bottom-Dollar Markets", ShippingAddress = new Address("23 Tsawassen Blvd.", "Tsawassen", "BC", "T2F 8M4", "Canada", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11049, Customer = customers.Where(c => c.CompanyName == "Gourmet Lanchonetes").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr 24 1998 12:00AM"), RequiredDate = DateTime.Parse("May 22 1998 12:00AM"), ShippingDate = DateTime.Parse("May  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 8.34M, ShippedTo = "Gourmet Lanchonetes", ShippingAddress = new Address("Av. Brasil, 442", "Campinas", "SP", "04876-786", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11050, Customer = customers.Where(c => c.CompanyName == "Folk och f HB").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr 27 1998 12:00AM"), RequiredDate = DateTime.Parse("May 25 1998 12:00AM"), ShippingDate = DateTime.Parse("May  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 59.41M, ShippedTo = "Folk och f HB", ShippingAddress = new Address("kergatan 24", "Brcke", "null", "S-844 67", "Sweden", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11051, Customer = customers.Where(c => c.CompanyName == "La maison d'Asie").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 27 1998 12:00AM"), RequiredDate = DateTime.Parse("May 25 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 2.79M, ShippedTo = "La maison d'Asie", ShippingAddress = new Address("1 rue Alsace-Lorraine", "Toulouse", "null", "31000", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11052, Customer = customers.Where(c => c.CompanyName == "Hanari Carnes").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr 27 1998 12:00AM"), RequiredDate = DateTime.Parse("May 25 1998 12:00AM"), ShippingDate = DateTime.Parse("May  1 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 67.26M, ShippedTo = "Hanari Carnes", ShippingAddress = new Address("Rua do Pao, 67", "Rio de Janeiro", "RJ", "05454-876", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11053, Customer = customers.Where(c => c.CompanyName == "Piccolo und mehr").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 27 1998 12:00AM"), RequiredDate = DateTime.Parse("May 25 1998 12:00AM"), ShippingDate = DateTime.Parse("Apr 29 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 53.05M, ShippedTo = "Piccolo und mehr", ShippingAddress = new Address("Geislweg 14", "Salzburg", "null", "5020", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11054, Customer = customers.Where(c => c.CompanyName == "Cactus Comidas para llevar").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr 28 1998 12:00AM"), RequiredDate = DateTime.Parse("May 26 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.33M, ShippedTo = "Cactus Comidas para llevar", ShippingAddress = new Address("Cerrito 333", "Buenos Aires", "null", "1010", "Argentina", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11055, Customer = customers.Where(c => c.CompanyName == "HILARION-Abastos").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("Apr 28 1998 12:00AM"), RequiredDate = DateTime.Parse("May 26 1998 12:00AM"), ShippingDate = DateTime.Parse("May  5 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 120.92M, ShippedTo = "HILARION-Abastos", ShippingAddress = new Address("Carrera 22 con Ave. Carlos Soublette #8-35", "San Cristbal", "Tchira", "5022", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11056, Customer = customers.Where(c => c.CompanyName == "Eastern Connection").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("Apr 28 1998 12:00AM"), RequiredDate = DateTime.Parse("May 12 1998 12:00AM"), ShippingDate = DateTime.Parse("May  1 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 278.96M, ShippedTo = "Eastern Connection", ShippingAddress = new Address("35 King George", "London", "null", "WX3 6FW", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11057, Customer = customers.Where(c => c.CompanyName == "North/South").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr 29 1998 12:00AM"), RequiredDate = DateTime.Parse("May 27 1998 12:00AM"), ShippingDate = DateTime.Parse("May  1 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 4.13M, ShippedTo = "North/South", ShippingAddress = new Address("South House 300 Queensbridge", "London", "null", "SW7 1RZ", "UK", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11058, Customer = customers.Where(c => c.CompanyName == "Blauer See Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Dodsworth").First(), OrderDate = DateTime.Parse("Apr 29 1998 12:00AM"), RequiredDate = DateTime.Parse("May 27 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 31.14M, ShippedTo = "Blauer See Delikatessen", ShippingAddress = new Address("Forsterstr. 57", "Mannheim", "null", "68306", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11059, Customer = customers.Where(c => c.CompanyName == "Ricardo Adocicados").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 29 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun 10 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 85.80M, ShippedTo = "Ricardo Adocicados", ShippingAddress = new Address("Av. Copacabana, 267", "Rio de Janeiro", "RJ", "02389-890", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11060, Customer = customers.Where(c => c.CompanyName == "Franchi S.p.A.").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("Apr 30 1998 12:00AM"), RequiredDate = DateTime.Parse("May 28 1998 12:00AM"), ShippingDate = DateTime.Parse("May  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 10.98M, ShippedTo = "Franchi S.p.A.", ShippingAddress = new Address("Via Monte Bianco 34", "Torino", "null", "10100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11061, Customer = customers.Where(c => c.CompanyName == "Great Lakes Food Market").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 30 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun 11 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Federal Shipping").First(), Freight = 14.01M, ShippedTo = "Great Lakes Food Market", ShippingAddress = new Address("2732 Baker Blvd.", "Eugene", "OR", "97403", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11062, Customer = customers.Where(c => c.CompanyName == "Reggiani Caseifici").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("Apr 30 1998 12:00AM"), RequiredDate = DateTime.Parse("May 28 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 29.93M, ShippedTo = "Reggiani Caseifici", ShippingAddress = new Address("Strada Provinciale 124", "Reggio Emilia", "null", "42100", "Italy", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11063, Customer = customers.Where(c => c.CompanyName == "Hungry Owl All-Night Grocers").First(), Employee = employees.Where(e => e.LastName == "Leverling").First(), OrderDate = DateTime.Parse("Apr 30 1998 12:00AM"), RequiredDate = DateTime.Parse("May 28 1998 12:00AM"), ShippingDate = DateTime.Parse("May  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 81.73M, ShippedTo = "Hungry Owl All-Night Grocers", ShippingAddress = new Address("8 Johnstown Road", "Cork", "Co. Cork", "null", "Ireland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11064, Customer = customers.Where(c => c.CompanyName == "Save-a-lot Markets").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May  1 1998 12:00AM"), RequiredDate = DateTime.Parse("May 29 1998 12:00AM"), ShippingDate = DateTime.Parse("May  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 30.09M, ShippedTo = "Save-a-lot Markets", ShippingAddress = new Address("187 Suffolk Ln.", "Boise", "ID", "83720", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11065, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("May  1 1998 12:00AM"), RequiredDate = DateTime.Parse("May 29 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 12.91M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11066, Customer = customers.Where(c => c.CompanyName == "White Clover Markets").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("May  1 1998 12:00AM"), RequiredDate = DateTime.Parse("May 29 1998 12:00AM"), ShippingDate = DateTime.Parse("May  4 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 44.72M, ShippedTo = "White Clover Markets", ShippingAddress = new Address("1029 - 12th Ave. S.", "Seattle", "WA", "98124", "USA", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11067, Customer = customers.Where(c => c.CompanyName == "Drachenblut Delikatessen").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May  4 1998 12:00AM"), RequiredDate = DateTime.Parse("May 18 1998 12:00AM"), ShippingDate = DateTime.Parse("May  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 7.98M, ShippedTo = "Drachenblut Delikatessen", ShippingAddress = new Address("Walserweg 21", "Aachen", "null", "52066", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11068, Customer = customers.Where(c => c.CompanyName == "Queen Cozinha").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("May  4 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  1 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 81.75M, ShippedTo = "Queen Cozinha", ShippingAddress = new Address("Alameda dos Canrios, 891", "Sao Paulo", "SP", "05487-020", "Brazil", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11069, Customer = customers.Where(c => c.CompanyName == "Tortuga Restaurante").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May  4 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  1 1998 12:00AM"), ShippingDate = DateTime.Parse("May  6 1998 12:00AM"), Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 15.67M, ShippedTo = "Tortuga Restaurante", ShippingAddress = new Address("Avda. Azteca 123", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11070, Customer = customers.Where(c => c.CompanyName == "Lehmanns Marktstand").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("May  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  2 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 136.00M, ShippedTo = "Lehmanns Marktstand", ShippingAddress = new Address("Magazinweg 7", "Frankfurt a.M.", "null", "60528", "Germany", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11071, Customer = customers.Where(c => c.CompanyName == "LILA-Supermercado").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  2 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "Speedy Express").First(), Freight = 0.93M, ShippedTo = "LILA-Supermercado", ShippingAddress = new Address("Carrera 52 con Ave. Bolvar #65-98 Llano Largo", "Barquisimeto", "Lara", "3508", "Venezuela", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11072, Customer = customers.Where(c => c.CompanyName == "Ernst Handel").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("May  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  2 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 258.64M, ShippedTo = "Ernst Handel", ShippingAddress = new Address("Kirchgasse 6", "Graz", "null", "8010", "Austria", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11073, Customer = customers.Where(c => c.CompanyName == "Pericles Comidas clsicas").First(), Employee = employees.Where(e => e.LastName == "Fuller").First(), OrderDate = DateTime.Parse("May  5 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  2 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 24.95M, ShippedTo = "Pericles Comidas clsicas", ShippingAddress = new Address("Calle Dr. Jorge Cash 321", "Mxico D.F.", "null", "05033", "Mexico", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11074, Customer = customers.Where(c => c.CompanyName == "Simons bistro").First(), Employee = employees.Where(e => e.LastName == "King").First(), OrderDate = DateTime.Parse("May  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  3 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 18.44M, ShippedTo = "Simons bistro", ShippingAddress = new Address("Vinbltet 34", "Kobenhavn", "null", "1734", "Denmark", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11075, Customer = customers.Where(c => c.CompanyName == "Richter Supermarkt").First(), Employee = employees.Where(e => e.LastName == "Callahan").First(), OrderDate = DateTime.Parse("May  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  3 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 6.19M, ShippedTo = "Richter Supermarkt", ShippingAddress = new Address("Starenweg 5", "Genve", "null", "1204", "Switzerland", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11076, Customer = customers.Where(c => c.CompanyName == "Bon app'").First(), Employee = employees.Where(e => e.LastName == "Peacock").First(), OrderDate = DateTime.Parse("May  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  3 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 38.28M, ShippedTo = "Bon app'", ShippingAddress = new Address("12, rue des Bouchers", "Marseille", "null", "13008", "France", null, null) }; session.Insert(order); orders.Add(order);
			order = new Order { OrderId = 11077, Customer = customers.Where(c => c.CompanyName == "Rattlesnake Canyon Grocery").First(), Employee = employees.Where(e => e.LastName == "Davolio").First(), OrderDate = DateTime.Parse("May  6 1998 12:00AM"), RequiredDate = DateTime.Parse("Jun  3 1998 12:00AM"), ShippingDate = null, Shipper = shippers.Where(s => s.CompanyName == "United Package").First(), Freight = 8.53M, ShippedTo = "Rattlesnake Canyon Grocery", ShippingAddress = new Address("2817 Milton Dr.", "Albuquerque", "NM", "87110", "USA", null, null) }; session.Insert(order); orders.Add(order);

			var orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10248).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 14.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10248).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 9.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10248).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10249).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 18.60M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10249).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 42.40M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10250).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10250).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 42.40M, Quantity = 35, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10250).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10251).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 16.80M, Quantity = 6, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10251).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 15.60M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10251).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10252).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 64.80M, Quantity = 40, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10252).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 25, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10252).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10253).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10253).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 42, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10253).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 16.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10254).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10254).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 21, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10254).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 8.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10255).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10255).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10255).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 15.20M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10255).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10256).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10256).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10257).First(), Product = products.Where(p => p.Name == "Schoggi Schokolade").First(), UnitPrice = 35.10M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10257).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10257).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10258).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 50, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10258).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 17.00M, Quantity = 65, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10258).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 25.60M, Quantity = 6, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10259).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10259).First(), Product = products.Where(p => p.Name == "Gravad lax").First(), UnitPrice = 20.80M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10260).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 16, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10260).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 15.60M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10260).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10260).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 21, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10261).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10261).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10262).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 17.00M, Quantity = 12, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10262).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 24.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10262).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10263).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 60, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10263).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10263).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 60, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10263).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 8.00M, Quantity = 36, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10264).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10264).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 25, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10265).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10265).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10266).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 30.40M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10267).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10267).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 70, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10267).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10268).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10268).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10269).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 60, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10269).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10270).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 15.20M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10270).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 36.80M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10271).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10272).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 64.80M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10272).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10272).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10273).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 24.80M, Quantity = 24, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10273).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10273).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10273).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 60, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10273).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 33, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10274).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10274).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10275).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10275).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 6, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10276).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 24.80M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10276).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 4.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10277).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10277).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10278).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 15.50M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10278).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10278).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 35.10M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10278).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 12.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10279).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10280).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10280).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10280).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10281).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10281).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10281).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10282).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10282).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 15.60M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10283).First(), Product = products.Where(p => p.Name == "Genen Shouyu").First(), UnitPrice = 12.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10283).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10283).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10283).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10284).First(), Product = products.Where(p => p.Name == "Schoggi Schokolade").First(), UnitPrice = 35.10M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10284).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 15.50M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10284).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10284).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 11.20M, Quantity = 5, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10285).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 14.40M, Quantity = 45, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10285).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10285).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 36, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10286).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 100, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10286).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10287).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10287).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 11.20M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10287).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10288).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10288).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 3, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10289).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 8.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10289).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10290).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 17.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10290).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10290).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 16.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10290).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10291).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 4.80M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10291).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 15.50M, Quantity = 24, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10291).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 42.40M, Quantity = 2, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10292).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 64.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10293).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 50.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10293).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10293).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 35.10M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10293).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10294).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 14.40M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10294).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10294).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 36.80M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10294).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10294).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10295).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10296).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10296).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10296).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10297).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10297).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10298).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10298).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 15.20M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10298).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10298).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10299).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10299).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10300).First(), Product = products.Where(p => p.Name == "Louisiana Hot Spiced Okra").First(), UnitPrice = 13.60M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10300).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10301).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10301).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10302).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10302).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10302).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 36.80M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10303).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 40, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10303).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10303).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10304).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 16.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10304).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10304).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10305).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 50.00M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10305).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10305).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10306).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10306).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10306).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10307).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10307).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10308).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10308).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10309).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 17.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10309).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 20.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10309).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 11.20M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10309).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 36.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10309).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10310).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10310).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10311).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 11.20M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10311).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10312).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10312).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 36.80M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10312).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10312).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10313).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 15.20M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10314).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 25.60M, Quantity = 40, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10314).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 10.60M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10314).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10315).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 11.20M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10315).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10316).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10316).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10317).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 14.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10318).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10318).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10319).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10319).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10319).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10320).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10321).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10322).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 5.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10323).First(), Product = products.Where(p => p.Name == "Genen Shouyu").First(), UnitPrice = 12.40M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10323).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 11.20M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10323).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10324).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 21, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10324).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 70, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10324).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10324).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10324).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 35.10M, Quantity = 80, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10325).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 20.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10325).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 4.80M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10325).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 18.60M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10325).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10325).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10326).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 17.60M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10326).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 15.60M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10326).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10327).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 25, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10327).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 50, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10327).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 35, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10327).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 10.60M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10328).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10328).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10328).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10329).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10329).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 8, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10329).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 210.80M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10329).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10330).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 50, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10330).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 25, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10331).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10332).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 50.00M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10332).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 11.20M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10332).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 7.60M, Quantity = 16, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10333).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 18.60M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10333).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10333).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 40, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10334).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 5.60M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10334).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10335).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 7, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10335).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 25, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10335).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 25.60M, Quantity = 6, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10335).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 42.40M, Quantity = 48, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10336).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 17.60M, Quantity = 18, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10337).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 7.20M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10337).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10337).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 15.20M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10337).First(), Product = products.Where(p => p.Name == "Gravad lax").First(), UnitPrice = 20.80M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10337).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10338).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10338).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10339).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 17.60M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10339).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 70, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10339).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10340).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 50.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10340).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10340).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 36.80M, Quantity = 40, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10341).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10341).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 9, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10342).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 24, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10342).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 56, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10342).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 15.20M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10342).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10343).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10343).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 4, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10343).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10344).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 17.60M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10344).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 32.00M, Quantity = 70, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10345).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 32.00M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10345).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 80, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10345).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 11.20M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10346).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 36, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10346).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10347).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 11.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10347).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 50, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10347).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10347).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 6, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10348).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 14.40M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10348).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 7.20M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10349).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10350).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 13.00M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10350).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 18, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10351).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 210.80M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10351).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 13, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10351).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 15.50M, Quantity = 77, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10351).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10352).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10352).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10353).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 12, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10353).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 210.80M, Quantity = 50, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10354).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 14.40M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10354).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10355).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10355).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 15.60M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10356).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10356).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10356).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10357).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 24.80M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10357).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10357).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 8, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10358).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10358).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 11.20M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10358).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 15.20M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10359).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 56, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10359).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 70, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10359).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 80, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10360).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10360).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10360).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 210.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10360).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 16.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10360).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10361).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 54, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10361).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 55, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10362).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 11.20M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10362).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 42.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10362).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10363).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10363).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10363).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10364).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10364).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10365).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10366).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10366).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10367).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 11.20M, Quantity = 36, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10367).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10367).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10367).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10368).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 5, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10368).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 13, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10368).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 15.60M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10368).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 35, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10369).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10369).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 18, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10370).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 14.40M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10370).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10370).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 8.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10371).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 15.20M, Quantity = 6, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10372).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 64.80M, Quantity = 12, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10372).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 210.80M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10372).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 70, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10372).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 42, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10373).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 10.60M, Quantity = 80, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10373).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 50, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10374).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10374).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 10.60M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10375).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 18.60M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10375).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10376).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 42, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10377).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10377).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10378).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10379).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 8, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10379).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 35.10M, Quantity = 16, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10379).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10380).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 18, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10380).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10380).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 6, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10380).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10381).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 8.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10382).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 17.00M, Quantity = 32, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10382).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 50.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10382).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10382).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10382).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 8.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10383).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 4.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10383).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 13.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10383).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10384).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 64.80M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10384).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10385).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 24.00M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10385).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10385).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 8, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10386).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10386).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 11.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10387).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10387).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10387).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10387).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10388).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 7.60M, Quantity = 15, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10388).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 5.60M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10388).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10389).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 24.80M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10389).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10389).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10389).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10390).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 60, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10390).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 40, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10390).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 45, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10390).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 24, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10391).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 4.80M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10392).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10393).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 25, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10393).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 18.60M, Quantity = 42, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10393).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 11.20M, Quantity = 7, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10393).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 70, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10393).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 32, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10394).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 4.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10394).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10395).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 28, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10395).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 70, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10395).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10396).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 7.20M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10396).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10396).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10397).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 10, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10397).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 42.40M, Quantity = 18, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10398).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10398).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 120, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10399).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10399).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10399).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10399).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10400).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10400).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10400).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 16.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10401).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10401).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10401).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10401).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10402).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 7.20M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10402).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 35.10M, Quantity = 65, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10403).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 21, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10403).First(), Product = products.Where(p => p.Name == "Chocolade").First(), UnitPrice = 10.20M, Quantity = 70, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10404).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10404).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 11.20M, Quantity = 40, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10404).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 16.00M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10405).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 8.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10406).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 14.40M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10406).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10406).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 42, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10406).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 15.20M, Quantity = 5, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10406).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 2, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10407).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10407).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10407).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10408).First(), Product = products.Where(p => p.Name == "Gravad lax").First(), UnitPrice = 20.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10408).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10408).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10409).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 18.60M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10409).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10410).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 49, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10410).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10411).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 25, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10411).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 15.50M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10411).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 9, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10412).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 18.60M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10413).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 14.40M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10413).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10413).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10414).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 18, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10414).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10415).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10415).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10416).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10416).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10416).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 15.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10417).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 210.80M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10417).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 2, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10417).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 36, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10417).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10418).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10418).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 7.60M, Quantity = 55, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10418).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 22.80M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10418).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 8.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10419).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 60, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10419).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10420).First(), Product = products.Where(p => p.Name == "Mishi Kobe Niku").First(), UnitPrice = 77.60M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10420).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 4.80M, Quantity = 2, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10420).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 8, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10420).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 12.00M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10421).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 4, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10421).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10421).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10421).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 10, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10422).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10423).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10423).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10424).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 60, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10424).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 210.80M, Quantity = 49, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10424).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10425).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 10, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10425).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10426).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10426).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10427).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 18.60M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10428).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10429).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 13.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10429).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 35.10M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10430).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 45, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10430).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10430).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10430).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 70, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10431).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 50, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10431).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 50, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10431).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 7.60M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10432).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10432).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10433).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10434).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10434).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 18, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10435).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10435).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 16.80M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10435).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10436).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10436).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 40, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10436).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10436).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 24, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10437).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10438).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 15, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10438).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 11.20M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10438).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 15.60M, Quantity = 15, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10439).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 30.40M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10439).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10439).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10439).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 8.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10440).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 45, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10440).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 49, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10440).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 24, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10440).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 22.80M, Quantity = 90, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10441).First(), Product = products.Where(p => p.Name == "Schoggi Schokolade").First(), UnitPrice = 35.10M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10442).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10442).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 80, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10442).First(), Product = products.Where(p => p.Name == "Louisiana Hot Spiced Okra").First(), UnitPrice = 13.60M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10443).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 6, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10443).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10444).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 31.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10444).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10444).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 14.40M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10444).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10445).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10445).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10446).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 12, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10446).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10446).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 3, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10446).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 5.60M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10447).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10447).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10447).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10448).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10448).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10449).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 24.80M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10449).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 5.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10449).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 39.40M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10450).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 24.80M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10450).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 6, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10451).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 120, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10451).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 35, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10451).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 28, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10451).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 55, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10452).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10452).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 15.50M, Quantity = 100, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10453).First(), Product = products.Where(p => p.Name == "Chocolade").First(), UnitPrice = 10.20M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10453).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10454).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10454).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10454).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10455).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10455).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10455).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 22.80M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10455).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10456).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10456).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 16.00M, Quantity = 21, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10457).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 36, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10458).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10458).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10458).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 36.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10458).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10458).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10459).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 24.00M, Quantity = 16, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10459).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10459).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10460).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 21, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10460).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 4, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10461).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10461).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 28, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10461).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 60, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10462).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 4.80M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10462).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 7.20M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10463).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10463).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 11.20M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10464).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 17.60M, Quantity = 16, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10464).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 36.80M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10464).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10464).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10465).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10465).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 99.00M, Quantity = 18, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10465).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10465).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 7.60M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10465).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 13.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10466).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10466).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 9.60M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10467).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10467).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 11.20M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10468).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 20.70M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10468).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 36.80M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10469).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10469).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 35, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10469).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 15.50M, Quantity = 2, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10470).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 50.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10470).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 7.20M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10470).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10471).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 24.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10471).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10472).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 3.60M, Quantity = 80, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10472).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 42.40M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10473).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10473).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 17.20M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10474).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 18.60M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10474).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 36.40M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10474).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10474).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10475).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 35, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10475).First(), Product = products.Where(p => p.Name == "Louisiana Hot Spiced Okra").First(), UnitPrice = 13.60M, Quantity = 60, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10475).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 14.40M, Quantity = 42, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10476).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 2, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10476).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10477).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 14.40M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10477).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 21, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10477).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 14.40M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10478).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 24.80M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10479).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 210.80M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10479).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 26.20M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10479).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10479).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 26.60M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10480).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 7.60M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10480).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10481).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 16.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10481).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 27.20M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10482).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10483).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 11.20M, Quantity = 35, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10483).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10484).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 8.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10484).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 14.70M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10484).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 42.40M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10485).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 15.20M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10485).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 8.00M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10485).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 19.20M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10485).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 12.00M, Quantity = 60, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10486).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10486).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 42.40M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10486).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 8.00M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10487).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 7.30M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10487).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 24.90M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10487).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 5.90M, Quantity = 24, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10488).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10488).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 12.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10489).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 16.80M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10489).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 13.90M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10490).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 44.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10490).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 10.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10490).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 6.20M, Quantity = 36, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10491).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 15.50M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10491).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 7, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10492).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 11.20M, Quantity = 60, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10492).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 11.20M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10493).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 16.80M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10493).First(), Product = products.Where(p => p.Name == "Louisiana Hot Spiced Okra").First(), UnitPrice = 13.60M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10493).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 28.80M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10494).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10495).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 7.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10495).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 7.70M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10495).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10496).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10497).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 30.40M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10497).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 27.80M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10497).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 10.40M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10498).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10498).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10498).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10499).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10499).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10500).First(), Product = products.Where(p => p.Name == "Genen Shouyu").First(), UnitPrice = 15.50M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10500).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 8, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10501).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10502).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10502).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10502).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 14.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10503).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10503).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10504).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10504).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10504).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10504).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10505).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10506).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 18, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10506).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 14, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10507).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10507).First(), Product = products.Where(p => p.Name == "Chocolade").First(), UnitPrice = 12.75M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10508).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10508).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10509).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10510).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 36, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10510).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 36, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10511).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 50, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10511).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 50, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10511).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 10, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10512).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 10, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10512).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 9, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10512).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 6, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10512).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 12, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10513).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10513).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 50, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10513).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 15, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10514).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 39, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10514).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10514).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10514).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 39, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10514).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10515).First(), Product = products.Where(p => p.Name == "Mishi Kobe Niku").First(), UnitPrice = 97.00M, Quantity = 16, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10515).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10515).First(), Product = products.Where(p => p.Name == "Schoggi Schokolade").First(), UnitPrice = 43.90M, Quantity = 120, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10515).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 16, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10515).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 84, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10516).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10516).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 80, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10516).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10517).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10517).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10517).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10518).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10518).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10518).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10519).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 16, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10519).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10519).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10520).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10520).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10521).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10521).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10521).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10522).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10522).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10522).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10522).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 25, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10523).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10523).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10523).First(), Product = products.Where(p => p.Name == "Gravad lax").First(), UnitPrice = 26.00M, Quantity = 18, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10523).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 6, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10524).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10524).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10524).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10524).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10525).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10525).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10526).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 8, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10526).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10526).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10527).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 50, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10527).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10528).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10528).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 8, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10528).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10529).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10529).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10529).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10530).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10530).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10530).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10530).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10531).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10532).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10532).First(), Product = products.Where(p => p.Name == "Louisiana Hot Spiced Okra").First(), UnitPrice = 17.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10533).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10533).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10533).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 24, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10534).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10534).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10534).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10535).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 50, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10535).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10535).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 5, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10535).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10536).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10536).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10536).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10536).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10537).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10537).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10537).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10537).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10537).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10538).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10538).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10539).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10539).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10539).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10539).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10540).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 10.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10540).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10540).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10540).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10541).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 35, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10541).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 4, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10541).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 36, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10541).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 9, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10542).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10542).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 24, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10543).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10543).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 70, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10544).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10544).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 14.00M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10545).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10546).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10546).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10546).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10547).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 24, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10547).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10548).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 10, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10548).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10549).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 55, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10549).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 100, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10549).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 48, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10550).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 8, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10550).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10550).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 6, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10550).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10551).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10551).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10551).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10552).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10552).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10553).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10553).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10553).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10553).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10553).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10554).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10554).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10554).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10554).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10555).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10555).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 35, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10555).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 18, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10555).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10555).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10556).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10557).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10557).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10558).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10558).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10558).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10558).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10558).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10559).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10559).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 18, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10560).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10560).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10561).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10561).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10562).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10562).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10563).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10563).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10564).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 16, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10564).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 6, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10564).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 25, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10565).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10565).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 18, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10566).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 35, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10566).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 18, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10566).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10567).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 60, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10567).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10567).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10568).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10569).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 35, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10569).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10570).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10570).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 60, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10571).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 11, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10571).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 28, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10572).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 12, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10572).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10572).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10572).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10573).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10573).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10573).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10574).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10574).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10574).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10574).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10575).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10575).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10575).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10575).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10576).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10576).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10576).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10577).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10577).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10577).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10578).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10578).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10579).First(), Product = products.Where(p => p.Name == "Genen Shouyu").First(), UnitPrice = 15.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10579).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10580).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10580).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 9, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10580).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10581).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 50, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10582).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10582).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10583).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10583).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 24, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10583).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 10, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10584).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10585).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10586).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 4, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10587).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10587).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10587).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10588).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10588).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 100, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10589).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10590).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10590).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 60, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10591).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 10.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10591).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10591).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10592).First(), Product = products.Where(p => p.Name == "Genen Shouyu").First(), UnitPrice = 15.50M, Quantity = 25, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10592).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 5, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10593).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 21, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10593).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10593).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 4, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10594).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10594).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10595).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10595).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 120, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10595).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 65, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10596).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 5, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10596).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 24, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10596).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10597).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 35, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10597).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10597).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 12, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10598).First(), Product = products.Where(p => p.Name == "Schoggi Schokolade").First(), UnitPrice = 43.90M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10598).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10599).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10600).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10600).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10601).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10601).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10602).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 5, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10603).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 48, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10603).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 25, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10604).First(), Product = products.Where(p => p.Name == "Chocolade").First(), UnitPrice = 12.75M, Quantity = 6, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10604).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10605).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10605).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10605).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 70, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10605).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10606).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10606).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10606).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10607).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 45, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10607).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 100, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10607).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10607).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 42, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10607).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10608).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10609).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10609).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10609).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10610).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 21, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10611).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10611).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10611).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10612).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10612).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 55, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10612).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10612).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10612).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 80, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10613).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 8, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10613).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10614).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10614).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10614).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10615).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10616).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10616).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10616).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10616).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10617).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10618).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10618).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10618).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10619).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 42, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10619).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10620).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10620).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10621).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10621).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10621).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10621).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10622).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10622).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 18, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10623).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10623).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10623).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10623).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10623).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10624).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10624).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10624).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10625).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10625).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10625).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10626).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10626).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10626).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10627).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10627).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 35, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10628).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10629).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10629).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10630).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10630).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10631).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 8, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10632).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10632).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10633).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 36, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10633).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 13, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10633).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 35, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10633).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 80, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10634).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10634).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10634).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10634).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10635).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10635).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 21.35M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10635).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10636).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10636).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10637).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10637).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 16.25M, Quantity = 25, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10637).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 60, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10638).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10638).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10638).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10639).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10640).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10640).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10641).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10641).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10642).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10642).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10643).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10643).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 21, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10643).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 2, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10644).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 4, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10644).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10644).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 21, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10645).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10645).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10646).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10646).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 18, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10646).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10646).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10647).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10647).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10648).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10648).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10649).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10649).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10650).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10650).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 25, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10650).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10651).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 12, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10651).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10652).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 2, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10652).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10653).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10653).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10654).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 12, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10654).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10654).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 6, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10655).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10656).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 3, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10656).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 28, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10656).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 6, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10657).First(), Product = products.Where(p => p.Name == "Genen Shouyu").First(), UnitPrice = 15.50M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10657).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10657).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 45, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10657).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10657).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 45, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10657).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10658).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10658).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 70, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10658).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 55, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10658).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 70, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10659).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10659).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 24, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10659).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 40, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10660).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10661).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 3, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10661).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 49, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10662).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10663).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10663).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10663).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10664).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 24, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10664).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 12, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10664).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10665).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10665).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10665).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10666).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 36, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10666).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10667).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 45, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10667).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 14, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10668).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 8, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10668).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 4, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10668).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10669).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10670).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 32, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10670).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10670).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 14.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10670).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10670).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10671).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10671).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10671).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10672).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10672).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10673).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10673).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10673).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10674).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10675).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10675).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10675).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10676).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10676).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10676).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10677).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10677).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 8, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10678).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 100, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10678).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10678).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 120, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10678).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10679).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10680).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 50, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10680).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10680).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10681).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10681).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 12, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10681).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10682).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10682).First(), Product = products.Where(p => p.Name == "Louisiana Hot Spiced Okra").First(), UnitPrice = 17.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10682).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10683).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10684).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10684).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10684).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10685).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10685).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10685).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10686).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10686).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10687).First(), Product = products.Where(p => p.Name == "Mishi Kobe Niku").First(), UnitPrice = 97.00M, Quantity = 50, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10687).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10687).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 6, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10688).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 18, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10688).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 60, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10688).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10689).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10690).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10690).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10691).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10691).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10691).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10691).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10691).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 48, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10692).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10693).First(), Product = products.Where(p => p.Name == "Mishi Kobe Niku").First(), UnitPrice = 97.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10693).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 60, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10693).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10693).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10694).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 90, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10694).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10694).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10695).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10695).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10695).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10696).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10696).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10697).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 7, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10697).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 9, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10697).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10697).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10698).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10698).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 8, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10698).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10698).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 65, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10698).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 8, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10699).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10700).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 5, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10700).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 12, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10700).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10700).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 60, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10701).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 42, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10701).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10701).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 35, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10702).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 10.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10702).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10703).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10703).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10703).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10704).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10704).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10704).First(), Product = products.Where(p => p.Name == "Chocolade").First(), UnitPrice = 12.75M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10705).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10705).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10706).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10706).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10706).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10707).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10707).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10707).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 28, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10708).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 21.35M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10708).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10709).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10709).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10709).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10710).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10710).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10711).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10711).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 42, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10711).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 120, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10712).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 3, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10712).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10713).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10713).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10713).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 110, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10713).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10714).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10714).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 27, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10714).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 50, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10714).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 18, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10714).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 12, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10715).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10715).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10716).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10716).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10716).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10717).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 32, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10717).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10717).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 25, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10718).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 36, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10718).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10718).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10718).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10719).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 12, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10719).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 3, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10719).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10720).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10720).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10721).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10722).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10722).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10722).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 45, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10722).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 42, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10723).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10724).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10724).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10725).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10725).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10725).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10726).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10726).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10727).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10727).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10727).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10728).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10728).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10728).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10728).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10729).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10729).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10729).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 16.25M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10730).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10730).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 3, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10730).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10731).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 40, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10731).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10732).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10733).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10733).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10733).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10734).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10734).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10734).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10735).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10735).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 2, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10736).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10736).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10737).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10737).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10738).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10739).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10739).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10740).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 5, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10740).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 35, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10740).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10740).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 14, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10741).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 15, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10742).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10742).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10742).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10743).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 28, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10744).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 50, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10745).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10745).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10745).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 45, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10745).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10746).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10746).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10746).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10746).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10747).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10747).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10747).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10747).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10748).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 44, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10748).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10748).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10749).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10749).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10749).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10750).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 5, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10750).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10750).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 25, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10751).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 12, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10751).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10751).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 16.25M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10751).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10752).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10752).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10753).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10753).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 10.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10754).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10755).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10755).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10755).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 14, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10755).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 25, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10756).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 21, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10756).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10756).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 6, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10756).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10757).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10757).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10757).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10757).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10758).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10758).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10758).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10759).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10760).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 12, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10760).First(), Product = products.Where(p => p.Name == "Schoggi Schokolade").First(), UnitPrice = 43.90M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10760).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10761).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10761).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10762).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10762).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10762).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10762).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10763).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10763).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10763).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10764).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10764).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 130, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10765).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 80, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10766).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10766).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10766).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10767).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10768).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10768).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10768).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10768).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10769).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10769).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10769).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10769).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10770).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10771).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10772).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10772).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10773).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 33, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10773).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 70, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10773).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 7, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10774).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 2, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10774).First(), Product = products.Where(p => p.Name == "Louisiana Hot Spiced Okra").First(), UnitPrice = 17.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10775).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10775).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 14.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10776).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 16, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10776).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10776).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 27, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10776).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 120, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10777).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10778).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10779).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10779).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10780).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10780).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10781).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 3, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10781).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10781).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 10.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10782).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10783).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10783).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10784).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10784).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 2, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10784).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10785).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10785).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10786).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10786).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 15, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10786).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 42, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10787).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10787).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10788).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10788).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 40, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10789).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10789).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10789).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10789).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10790).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 3, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10790).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10791).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 14, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10791).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10792).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10792).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10792).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10793).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10793).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10794).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 15, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10794).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 6, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10795).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 65, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10795).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10796).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 21, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10796).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10796).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 35, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10796).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 24, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10797).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10798).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10798).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10799).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10799).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10799).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10800).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 50, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10800).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10800).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 7, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10801).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10801).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10802).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 25, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10802).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10802).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 60, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10802).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 5, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10803).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 24, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10803).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10803).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10804).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 36, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10804).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10804).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 4, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10805).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10805).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10806).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10806).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10806).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 10.00M, Quantity = 15, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10807).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10808).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10808).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 50, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10809).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10810).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10810).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10810).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10811).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10811).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10811).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10812).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 16, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10812).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 40, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10812).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10813).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 12, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10813).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10814).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10814).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10814).First(), Product = products.Where(p => p.Name == "Chocolade").First(), UnitPrice = 12.75M, Quantity = 8, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10814).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10815).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10816).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 30, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10816).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10817).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10817).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10817).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 60, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10817).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 25, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10818).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10818).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10819).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10819).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10820).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10821).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10821).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10822).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10822).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10823).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10823).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10823).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 40, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10823).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10824).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10824).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10825).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10825).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10826).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10826).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10827).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10827).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10828).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10828).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10829).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10829).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10829).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10829).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10830).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10830).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10830).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10830).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10831).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10831).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10831).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10831).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10832).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 3, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10832).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10832).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 16, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10832).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10833).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10833).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 9, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10833).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 9, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10834).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 8, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10834).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10835).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10835).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 2, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10836).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 52, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10836).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10836).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10836).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10836).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10837).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10837).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10837).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10837).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 21, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10838).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 4, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10838).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 25, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10838).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 50, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10839).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10839).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10840).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 6, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10840).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10841).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10841).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10841).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10841).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10842).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10842).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10842).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10842).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10843).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 4, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10844).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10845).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 70, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10845).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10845).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 42, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10845).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 60, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10845).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 48, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10846).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10846).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10846).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10847).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 80, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10847).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 12, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10847).First(), Product = products.Where(p => p.Name == "Gravad lax").First(), UnitPrice = 26.00M, Quantity = 60, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10847).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 36, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10847).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 45, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10847).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 55, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10848).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 21.35M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10848).First(), Product = products.Where(p => p.Name == "Mishi Kobe Niku").First(), UnitPrice = 97.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10849).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 10.00M, Quantity = 49, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10849).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 18, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10850).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10850).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 4, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10850).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10851).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 5, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10851).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10851).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10851).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 42, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10852).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10852).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10852).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10853).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10854).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 100, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10854).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 65, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10855).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10855).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10855).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10855).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10856).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10856).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10857).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 10.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10857).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10857).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 10, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10858).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10858).First(), Product = products.Where(p => p.Name == "Schoggi Schokolade").First(), UnitPrice = 43.90M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10858).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10859).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10859).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10859).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10860).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10860).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10861).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 42, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10861).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10861).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10861).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10861).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10862).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10862).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10863).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10863).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 12, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10864).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10864).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 14.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10865).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 60, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10865).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 80, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10866).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 21, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10866).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 6, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10866).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10867).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10868).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10868).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10868).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 42, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10869).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10869).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10869).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10869).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10870).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10870).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10871).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10871).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10871).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 16, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10872).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10872).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10872).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10872).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 21, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10873).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10873).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10874).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10875).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10875).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 21, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10875).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10876).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10876).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10877).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10877).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10878).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10879).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10879).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10879).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10880).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10880).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10880).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 50, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10881).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10882).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10882).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10882).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 32, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10883).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10884).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 40, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10884).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 21, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10884).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 12, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10885).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10885).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10885).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10885).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10886).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10886).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10886).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10887).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10888).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10888).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10889).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10889).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10890).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10890).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10890).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10891).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10892).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 40, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10893).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10893).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10893).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10893).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10893).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10894).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 28, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10894).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10894).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 120, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10895).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 110, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10895).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 45, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10895).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 91, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10895).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 100, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10896).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10896).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10897).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 80, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10897).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 36, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10898).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10899).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 8, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10900).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 3, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10901).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10901).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10902).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10902).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 6, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10903).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10903).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10903).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10904).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10904).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10905).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10906).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10907).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10908).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10908).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 14, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10909).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10909).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10909).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10910).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10910).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10910).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10911).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10911).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10911).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 14.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10912).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10912).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 60, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10913).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10913).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 40, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10913).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10914).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10915).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10915).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10915).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10916).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10916).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10916).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10917).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10917).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10918).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 60, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10918).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 25, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10919).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10919).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10919).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10920).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 16.25M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10921).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10921).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10922).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10922).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10923).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10923).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10923).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 14.00M, Quantity = 24, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10924).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10924).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10924).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10925).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 25, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10925).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 12, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10926).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10926).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10926).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10926).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10927).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10927).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10927).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10928).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10928).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10929).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10929).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 49, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10929).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10930).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 36, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10930).First(), Product = products.Where(p => p.Name == "Schoggi Schokolade").First(), UnitPrice = 43.90M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10930).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 25, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10930).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10931).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 42, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10931).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10932).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10932).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 14, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10932).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10932).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10933).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10933).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10934).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10935).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10935).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 4, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10935).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 8, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10936).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10937).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10937).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10938).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10938).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 24, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10938).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 49, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10938).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10939).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 10, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10939).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 14.00M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10940).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10940).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10941).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 44, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10941).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10941).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 80, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10941).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10942).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10943).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10943).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10943).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10944).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 5, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10944).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 18, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10944).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10945).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10945).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10946).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10946).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10946).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10947).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10948).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 16.25M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10948).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10948).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10949).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10949).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10949).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10949).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10950).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10951).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10951).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 6, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10951).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10952).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 16, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10952).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10953).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10953).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10954).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 28, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10954).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 25, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10954).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10954).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 24, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10955).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 12, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10956).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10956).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10956).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10957).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10957).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10957).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10958).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 21.35M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10958).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10958).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10959).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10960).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 10, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10960).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10961).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 6, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10961).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10962).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 45, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10962).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 77, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10962).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10962).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10962).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 44, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10963).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 2, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10964).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10964).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10964).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10965).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10966).First(), Product = products.Where(p => p.Name == "Gravad lax").First(), UnitPrice = 26.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10966).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 12, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10966).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 12, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10967).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10967).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10968).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10968).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10968).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10969).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10970).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10971).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10972).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10972).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10973).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10973).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10973).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10974).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10975).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10975).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10976).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10977).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10977).First(), Product = products.Where(p => p.Name == "Zaanse koeken").First(), UnitPrice = 9.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10977).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10977).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10978).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10978).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10978).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10978).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 6, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10979).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10979).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10979).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 80, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10979).First(), Product = products.Where(p => p.Name == "Schoggi Schokolade").First(), UnitPrice = 43.90M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10979).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10979).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10980).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 40, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10981).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10982).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10982).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10983).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 84, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10983).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10984).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 55, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10984).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10984).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10985).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 36, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10985).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 8, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10985).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 35, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10986).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10986).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10986).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10986).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10987).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10987).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10987).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10988).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10988).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 40, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10989).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10989).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10989).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10990).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 65, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10990).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 60, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10990).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 65, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10990).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 66, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10991).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 50, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10991).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 20, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10991).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 90, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10992).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10993).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 50, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10993).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 35, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10994).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 18, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10995).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10995).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10996).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10997).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10997).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10997).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10998).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10998).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10998).First(), Product = products.Where(p => p.Name == "Longlife Tofu").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10998).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10999).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10999).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 10999).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 21, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11000).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 25, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11000).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11000).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11001).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11001).First(), Product = products.Where(p => p.Name == "Gustaf's Knckebrd").First(), UnitPrice = 21.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11001).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11001).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11002).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 56, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11002).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 15, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11002).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 24, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11002).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11003).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11003).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11003).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11004).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11004).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11005).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11005).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11006).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11006).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 2, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11007).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11007).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11007).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 14, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11008).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 70, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11008).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 90, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11008).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11009).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11009).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 18, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11009).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11010).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11010).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11011).First(), Product = products.Where(p => p.Name == "Escargots de Bourgogne").First(), UnitPrice = 13.25M, Quantity = 40, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11011).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11012).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 50, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11012).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 36, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11012).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 60, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11013).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11013).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11013).First(), Product = products.Where(p => p.Name == "Rogede sild").First(), UnitPrice = 9.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11013).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11014).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 28, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11015).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11015).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 18, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11016).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11016).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11017).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 10.00M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11017).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 110, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11017).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11018).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11018).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11018).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 5, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11019).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11019).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11020).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 24, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11021).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 11, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11021).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11021).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 63, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11021).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 44, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11021).First(), Product = products.Where(p => p.Name == "Mozzarella di Giovanni").First(), UnitPrice = 34.80M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11022).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11022).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11023).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11023).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11024).First(), Product = products.Where(p => p.Name == "Gumbr Gummibrchen").First(), UnitPrice = 31.23M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11024).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11024).First(), Product = products.Where(p => p.Name == "Louisiana Fiery Hot Pepper Sauce").First(), UnitPrice = 21.05M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11024).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11025).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11025).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 20, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11026).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11026).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11027).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11027).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 21, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11028).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11028).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11029).First(), Product = products.Where(p => p.Name == "Gnocchi di nonna Alice").First(), UnitPrice = 38.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11029).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11030).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 100, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11030).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 21.35M, Quantity = 70, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11030).First(), Product = products.Where(p => p.Name == "Thringer Rostbratwurst").First(), UnitPrice = 123.79M, Quantity = 60, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11030).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 100, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11031).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 45, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11031).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 80, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11031).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11031).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11031).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 16, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11032).First(), Product = products.Where(p => p.Name == "Inlagd Sill").First(), UnitPrice = 19.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11032).First(), Product = products.Where(p => p.Name == "Cte de Blaye").First(), UnitPrice = 263.50M, Quantity = 25, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11032).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11033).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 70, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11033).First(), Product = products.Where(p => p.Name == "Gudbrandsdalsost").First(), UnitPrice = 36.00M, Quantity = 36, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11034).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 15, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11034).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11034).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 6, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11035).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11035).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11035).First(), Product = products.Where(p => p.Name == "Singaporean Hokkien Fried Mee").First(), UnitPrice = 14.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11035).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11036).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 7, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11036).First(), Product = products.Where(p => p.Name == "Raclette Courdavault").First(), UnitPrice = 55.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11037).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11038).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 5, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11038).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11038).First(), Product = products.Where(p => p.Name == "Flotemysost").First(), UnitPrice = 21.50M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11039).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11039).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11039).First(), Product = products.Where(p => p.Name == "Maxilaku").First(), UnitPrice = 20.00M, Quantity = 60, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11039).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 28, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11040).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11041).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11041).First(), Product = products.Where(p => p.Name == "Vegie-spread").First(), UnitPrice = 43.90M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11042).First(), Product = products.Where(p => p.Name == "Gula Malacca").First(), UnitPrice = 19.45M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11042).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11043).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11044).First(), Product = products.Where(p => p.Name == "Tarte au sucre").First(), UnitPrice = 49.30M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11045).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11045).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 24, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11046).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 20, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11046).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11046).First(), Product = products.Where(p => p.Name == "Steeleye Stout").First(), UnitPrice = 18.00M, Quantity = 18, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11047).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 25, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11047).First(), Product = products.Where(p => p.Name == "Chef Anton's Gumbo Mix").First(), UnitPrice = 21.35M, Quantity = 30, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11048).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 42, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11049).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11049).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 4, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11050).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 50, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11051).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11052).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 30, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11052).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11053).First(), Product = products.Where(p => p.Name == "Carnarvon Tigers").First(), UnitPrice = 62.50M, Quantity = 35, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11053).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11053).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 25, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11054).First(), Product = products.Where(p => p.Name == "Geitost").First(), UnitPrice = 2.50M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11054).First(), Product = products.Where(p => p.Name == "Laughing Lumberjack Lager").First(), UnitPrice = 14.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11055).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11055).First(), Product = products.Where(p => p.Name == "NuNuCa Nu-Nougat-Creme").First(), UnitPrice = 14.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11055).First(), Product = products.Where(p => p.Name == "Manjimup Dried Apples").First(), UnitPrice = 53.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11055).First(), Product = products.Where(p => p.Name == "Ravioli Angelo").First(), UnitPrice = 19.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11056).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11056).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11056).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 50, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11057).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11058).First(), Product = products.Where(p => p.Name == "Sir Rodney's Scones").First(), UnitPrice = 10.00M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11058).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 21, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11058).First(), Product = products.Where(p => p.Name == "Sirop d'rable").First(), UnitPrice = 28.50M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11059).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11059).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11059).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11060).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11060).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11061).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 15, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11062).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 10, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11062).First(), Product = products.Where(p => p.Name == "Outback Lager").First(), UnitPrice = 15.00M, Quantity = 12, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11063).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 30, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11063).First(), Product = products.Where(p => p.Name == "Boston Crab Meat").First(), UnitPrice = 18.40M, Quantity = 40, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11063).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 30, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11064).First(), Product = products.Where(p => p.Name == "Alice Mutton").First(), UnitPrice = 39.00M, Quantity = 77, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11064).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 12, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11064).First(), Product = products.Where(p => p.Name == "Perth Pasties").First(), UnitPrice = 32.80M, Quantity = 25, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11064).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 4, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11064).First(), Product = products.Where(p => p.Name == "Scottish Longbreads").First(), UnitPrice = 12.50M, Quantity = 55, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11065).First(), Product = products.Where(p => p.Name == "Nord-Ost Matjeshering").First(), UnitPrice = 25.89M, Quantity = 4, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11065).First(), Product = products.Where(p => p.Name == "Tourtire").First(), UnitPrice = 7.45M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11066).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11066).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 42, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11066).First(), Product = products.Where(p => p.Name == "Sasquatch Ale").First(), UnitPrice = 14.00M, Quantity = 35, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11067).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 9, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11068).First(), Product = products.Where(p => p.Name == "Rssle Sauerkraut").First(), UnitPrice = 45.60M, Quantity = 8, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11068).First(), Product = products.Where(p => p.Name == "Ipoh Coffee").First(), UnitPrice = 46.00M, Quantity = 36, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11068).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 28, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11069).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11070).First(), Product = products.Where(p => p.Name == "Chai").First(), UnitPrice = 18.00M, Quantity = 40, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11070).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 20, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11070).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11070).First(), Product = products.Where(p => p.Name == "Gorgonzola Telino").First(), UnitPrice = 12.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11071).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 15, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11071).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 10, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11072).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 8, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11072).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 40, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11072).First(), Product = products.Where(p => p.Name == "Valkoinen suklaa").First(), UnitPrice = 16.25M, Quantity = 22, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11072).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 130, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11073).First(), Product = products.Where(p => p.Name == "Queso Cabrales").First(), UnitPrice = 21.00M, Quantity = 10, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11073).First(), Product = products.Where(p => p.Name == "Guaran Fantstica").First(), UnitPrice = 4.50M, Quantity = 20, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11074).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 14, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11075).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 10, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11075).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 30, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11075).First(), Product = products.Where(p => p.Name == "Lakkalikri").First(), UnitPrice = 18.00M, Quantity = 2, Discount = 0.15M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11076).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11076).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 20, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11076).First(), Product = products.Where(p => p.Name == "Teatime Chocolate Biscuits").First(), UnitPrice = 9.20M, Quantity = 10, Discount = 0.25M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Chang").First(), UnitPrice = 19.00M, Quantity = 24, Discount = 0.2M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Aniseed Syrup").First(), UnitPrice = 10.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Chef Anton's Cajun Seasoning").First(), UnitPrice = 22.00M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Grandma's Boysenberry Spread").First(), UnitPrice = 25.00M, Quantity = 1, Discount = 0.02M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Uncle Bob's Organic Dried Pears").First(), UnitPrice = 30.00M, Quantity = 1, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Northwoods Cranberry Sauce").First(), UnitPrice = 40.00M, Quantity = 2, Discount = 0.1M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Ikura").First(), UnitPrice = 31.00M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Queso Manchego La Pastora").First(), UnitPrice = 38.00M, Quantity = 2, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Konbu").First(), UnitPrice = 6.00M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Tofu").First(), UnitPrice = 23.25M, Quantity = 1, Discount = 0.03M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Pavlova").First(), UnitPrice = 17.45M, Quantity = 2, Discount = 0.03M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Sir Rodney's Marmalade").First(), UnitPrice = 81.00M, Quantity = 1, Discount = 0.04M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Tunnbrd").First(), UnitPrice = 9.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Mascarpone Fabioli").First(), UnitPrice = 32.00M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Chartreuse verte").First(), UnitPrice = 18.00M, Quantity = 2, Discount = 0.05M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Jack's New England Clam Chowder").First(), UnitPrice = 9.65M, Quantity = 3, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Spegesild").First(), UnitPrice = 12.00M, Quantity = 3, Discount = 0.02M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Filo Mix").First(), UnitPrice = 7.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Pt chinois").First(), UnitPrice = 24.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Camembert Pierrot").First(), UnitPrice = 34.00M, Quantity = 2, Discount = 0.06M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Wimmers gute Semmelkndel").First(), UnitPrice = 33.25M, Quantity = 2, Discount = 0.03M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Louisiana Hot Spiced Okra").First(), UnitPrice = 17.00M, Quantity = 1, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Rd Kaviar").First(), UnitPrice = 15.00M, Quantity = 2, Discount = 0.01M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Rhnbru Klosterbier").First(), UnitPrice = 7.75M, Quantity = 4, Discount = 0M }; session.Insert(orderLine);
			orderLine = new OrderLine { Order = orders.Where(o => o.OrderId == 11077).First(), Product = products.Where(p => p.Name == "Original Frankfurter grne Soe").First(), UnitPrice = 13.00M, Quantity = 2, Discount = 0M }; session.Insert(orderLine);
		}

	}
}